/***************************************************************************
 *            convwindow.h
 *
 *  Fri Jan 21 23:21:00 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef CONVWINDOW_H
#define CONVWINDOW_H
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "basicwindow.h"
#include "processingani.h"
#include "settings.h"
#include "utilities.h"
#include "userutilities.h"
#include "contactsselection.h"
#include "gettext.h"
#include "systemcharset.h"



// message types that can be displayed
enum messageType
{
	MT_INCOMING,
	MT_OUTGOING,
	MT_NOTIFICATION,
	MT_HINCOMING,
	MT_HOUTGOING,	
};



// toolbar button identifiers
enum
{
	TOOL_NONE,
	// info tools
	TOOL_STATUS,
	TOOL_TIME,
	// tool buttons
	TOOL_ENCODING,
	TOOL_ENCRYPT,
	TOOL_HISTORY,
	TOOL_INFO,
	TOOL_SMILEYS,
	TOOL_SENDFILE,
	TOOL_SENDLINK,
	TOOL_STARTCHAT,
	TOOL_FGCOLOR,
	TOOL_BGCOLOR,
	TOOL_SENDSERVER,
	TOOL_SENDURGENT,
	TOOL_SEND,
	TOOL_MULTIPLE,

	// graphical elements
	TOOL_SENDANI,
	TOOL_SEPARATOR,
	TOOL_ALIGNLEFT,
	TOOL_ALIGNRIGHT,
	
	NUM_TOOLS
};



// a toolbar item
typedef struct toolbarItem
{
	char* 		tip;
	gchar* 		stockIcon;
	gint 		id;
	gboolean    alignLeft;
};


typedef struct smallHistoryEntry
{
	gchar			*msg;
	gchar			*url;
	time_t			time;
	messageType 	type;
	IMUserDaemon 	*user;
};



class conversationWindow: public basicWindow
{
	public:
		conversationWindow();
		virtual ~conversationWindow();
	
		virtual void windowDeactivated();
		virtual void windowActivated();
	
		virtual gboolean eventCallback(gint command, gint lastResult, gpointer info);
		virtual void	 settingsChangedCallback(gchar* sname, GList *values);
	
		void	enableURLMode();
		void	disableURLMode();
	
		GtkWidget *createToolbar(gchar *description, guint spacing, gboolean preview=FALSE, GList **entries = NULL);
	
	protected:
	
		GList		*smallHistory;
	
		GtkWidget	*mainHBox,
					*localRemoteDivider,
					*localView,
					*remoteView,
					*sendButton,
					*multipleButton,
					*encryptButton,
					*serverCheckbox,
					*urgentCheckbox,
					*tabLabelImage,
					*tabLabelText,
					*statusEntry,
					*timeEntry,
					*typingImage,
					*smileysMenu,
					*URLbox,
					*URLentry,
					*charsetMenu;
	
		GtkWidget	*toolbar1Container,
					*toolbar2Container,
					*toolbar3Container;
	
	
		guint		timeTimeoutID;
		guint		oldStatus;
		time_t		keyPressTime;
		gboolean	isURLMode;
		gchar		*hoveredLink;
		gchar		*toSendText,
					*toSendLink;
					
		gchar		*tabMessageColor,
					*tabTypingColor,
					*messageTag;
					
		gboolean	interpretSmileys,
					interpretURIs,
					dontApplyColors,
					sendTyping;
	
		processingAni	*sendAni;
		
		contactsSelection* multipleSelection;
	
		virtual GtkWidget* createWindowContent();
		virtual void	   destroyWindowContent();
		virtual GtkWidget* createContainerTabWidget();
	
		void	sendMessage();
		void	sendMessageMultiple(gint count);
		void	finishSendMessage(messageEventInfo* messageEvent, gint result);
		void	stopSendMessage();
		
		void	insertMessage(time_t time, gchar* msg, gchar* url, IMUserDaemon* user, messageType msgType, gboolean add2History);
	
		void	selectColor(gboolean foreground);
		void	updateLocalColors();
	
		// helper functions for creating a window
		void		createCharsetList();
		
		GtkWidget	*createRemoteEntryBox();
		GtkWidget	*createLocalEntryBox();
		void		createToolbars();
		
		void		createSmileysMenu();
		
		void		showOldEntries(gint toSkip);
		
		void		rebuildMessageView();
		
		
		// conversation functions
		gchar		*getTextWithoutSmileys();
		void		insertTextInterpreted(gchar* toAppend, gchar *tagName, gboolean createMargin, GtkTextIter *pos);
		
		
		// URL functions
		void		checkForURLCursor(gint x, gint y);
		
		// multiple selection functions
		void		showMultipleSelectionList();
		void		hideMultipleSelectionList();
		
		// callbacks
		static void cb_sendButtonClicked(conversationWindow *cw);
		static void cb_sendServerButtonClicked(conversationWindow *cw);
		static void cb_toolbarClicked(GtkWidget* clickedWidget, conversationWindow *cw);
		static void cb_multipleSendButtonClicked(conversationWindow *self);
		
		static gboolean cb_keyPressed(conversationWindow *cw, GdkEventKey *event);
		static void		cb_localEntryChanged(conversationWindow *self);
		
		static gboolean cb_setUserTime(conversationWindow *cw);
		
		static void		cb_colorSelectionCheckboxClicked(GtkWidget* w, GtkWidget* d);
		
		static gboolean cb_smileysMenuKeyReleased(GtkWidget *w, GdkEventKey *event, GtkWidget **toDestroy);
		static void		cb_smileyClicked(GtkWidget* w, conversationWindow *cw);
		
		static gboolean	cb_remoteViewMouseMoved(GtkWidget* v, GdkEventMotion* event, conversationWindow *self);
		static gboolean cb_remoteViewVisibilityChanged(GtkWidget* v, GdkEventVisibility* event, conversationWindow *self);

		static void		cb_charsetListItemClicked(GtkWidget *w, conversationWindow *self);
};

#endif
