/***************************************************************************
 *            mainwindow.h
 *
 *  Tue Jan 25 22:12:28 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _MAINWINDOW_H
#define _MAINWINDOW_H

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "icons.h"
#include "imuserdaemon.h"
#include "settings.h"
#include "contactlist.h"
#include "trayicon.h"
#include "utilities.h"
#include "userutilities.h"
#include "gettext.h"



// different types of menus
enum userMenuGroup
{
	MENU_USER,
	MENU_STATUS,
	MENU_SYSTEM,
	MENU_DOUBLECLICK,
	MENU_NOTIFYBOX
};



// id's for menu elements
enum userMenuID
{
	MENU_OPENEVENT = 0xFFFF,
	MENU_REMOVEUSER,
	MENU_SHOWGROUPS,
	MENU_SHOWEMPTYGROUPS,
	MENU_SHOWOFFLINEUSERS,
	MENU_RELOADLIST,
	MENU_OPENALL,
	MENU_UPDATEALL,
	MENU_CHECKINVISIBLEALL,
	MENU_QUIT,
	MENU_NONE	= 0xC0C01A
};

enum tooltipElements
{
	TT_STATUS 			= 1,
	TT_PROTOCOL 		= 2,
	TT_REALNAME 		= 4,
	TT_MSGCOUNT 		= 8,
	TT_IP 				= 16,
	TT_ONLINETIME 		= 32,
	TT_IDLETIME 		= 64,
	TT_AUTORESPONSE 	= 128,
	TT_ID				= 256
};


struct statusButtonType
{
	GtkWidget	*button,
				*statusImage,
				*mainMenuEntry;
	GtkMenu		*statusMenu;
	gpointer	owner;
	glong		status;
	guint		connectTimeout;
	gboolean	connectImg;
};



typedef void(*mwMenuClickFunction)(GtkWidget* item, gint menu, gint selection, gpointer data);


class mainWindow
{
	public:
		GtkWidget	*menu_showEmptyGroups,
					*menu_showGroups,
					*menu_showOfflineUsers,
					*menu_openUserEvent,
					*menu_openAllEvents,
					*menu_userMenuLabel,
					*menu_users,
					*menu_status,
					*menu_editPersonalInfo,
					*menu_reloadContacts,
					*menu_statusInvisible,
					*menu_sendMessage,
					*menu_sendURL,
					*menu_sendFiles,
					*menu_sendChat,
					*menu_sendContacts,
					*menu_sendAuthReq,
					*menu_sendAuth,
					*menu_sendEncryption,
					*statusButton,
					*window;
					
		GList		*statusButtons;
	
	
		contactList	*contacts;	
	
		mainWindow(mwMenuClickFunction mcf, GCallback quitfunc, gpointer cbdata);
		~mainWindow();	
	
		void setTitle(gchar* _title);
		void setPendingEventNotify(gboolean state);
	
		void updateStatus(guint status, gboolean isInvisible);
		void updateSystemInfo(gint noe, gchar* evname, gchar* appendix);
		GtkMenu *addStatusButton(IMOwnerDaemon *owner);
		void removeStatusButton(IMOwnerDaemon *owner);
		void updateStatusButton(IMOwnerDaemon *owner, glong status, gboolean invisible);
	
		void startTrayAnimation();
		void stopTrayAnimation();
		gboolean isTrayAnimation();
		void setTrayBlinkEvent(basicEventInfo *ev);
		
		void setLockUserMenus(gboolean lock);
		
		void destroyWindow();
	
	private:
	
		GtkWidget	*notifyText,
					*notifyIcon,
					*mainMenu,
					*statusBox,
					*statusButtonIcon,
					*statusButtonText,
					*tooltipWindow;
	
		GtkMenu		*statusMenu,
					*userMenu,
					*groupMenu;
	
		GList		*stateIcons;
	
		GdkPixbuf	*lastStatusPixbuf,
					*eventPixbuf;
						
		
		mwMenuClickFunction	menuCallBack;
		gpointer			menuCallBackData;
	
		trayIcon			*tray;
		gchar				trayIter;
		guint				trayConnectSource,
							trayBlinkSource,
							tooltipSource,
							lastStatus;
		
		gboolean			trayShowEventIcon,
							lastInvisible,
							showTooltips;
							
		gulong				tooltipsMask;
		
		gchar				*title;
		
		contactListEntry	*lastTooltipEntry;
	
		gboolean	isObscured;
	
		void 		trayClicked(GdkEventType type, guchar button);
		gboolean 	updateContactListViewMode();
		void 		createMenus();
	
	
		// callbacks
		static gboolean cb_contactListClicked(GtkTreeView* cList, GdkEventButton* event, mainWindow* mw);
		static void	 	cb_contactListActivated(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, mainWindow* mw);
		static void	 	cb_menuClicked(GtkWidget* w, GtkWidget* toshow);
		static void 	cb_menuItemClicked(GtkWidget* item, mainWindow* mw);
		static gboolean	cb_destroyWindow(mainWindow* mw);
		static gboolean cb_visibilityChange(mainWindow* mw, GdkEventVisibility *event);
		static gboolean cb_trayClicked(mainWindow* mw, GdkEventButton* event);
		static gboolean cb_trayAnimation(mainWindow* mw);
		static gboolean cb_trayEventBlinking(mainWindow* mw);
		static gboolean cb_statusButtonBlinking(statusButtonType* but);
		
		static gboolean cb_notifyBoxClicked(GtkWidget* box, GdkEventButton* event, mainWindow* mw);

		static void cb_settingsChangedCallback(gchar* sname, GList *values, mainWindow* self);
	
		// tooltip callbacks
		static gboolean cb_tooltipLeaveNotify(mainWindow* self, GdkEventCrossing* event);
		static gboolean cb_tooltipMoveNotify(GtkWidget* w, GdkEventMotion *event, mainWindow* self);
		static gboolean	cb_tooltipShowTimeout(mainWindow* self);
		static gboolean cb_tooltipPaint(mainWindow* self);
		static void cb_tooltipRealize(mainWindow* self);
};

#endif /* _MAINWINDOW_H */
