/***************************************************************************
 *            icons.cpp
 *
 *  Tue Aug 16 23:52:18 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "icons.h"

iconManager* iconMan = NULL;

iconManager::iconManager()
{
	gchar *base,
		  *eventTheme,
		  *smileyTheme,
		  *statusTheme,
		  *extendedTheme,
		  *searchingPath;
	
	icons = NULL;
	eventIcons = NULL;
	
	settings_getSettings()->getProperties("appearance",
				  						  "EventTheme", &eventTheme,
				  						  "SmileyTheme", &smileyTheme,
				  						  "StatusTheme", &statusTheme,
				  						  "ExtendedTheme", &extendedTheme,
				  						  NULL);
	
	// load status icons
	base = g_strdup_printf("%sicqnd/%s/status", SHARE_DIR, statusTheme);
	loadStatusIcons(base);	
	g_free(base);
	
	// load event icons
	base = g_strdup_printf("%sicqnd/%s/events", SHARE_DIR, eventTheme);
	loadEventIcons(base);
	g_free(base);
	
	// load extended icons
	base = g_strdup_printf("%sicqnd/%s/extended", SHARE_DIR, extendedTheme);
	loadExtendedIcons(base);
	g_free(base);
	
	// load smileys
	base = g_strdup_printf("%sicqnd/%s/smileys", SHARE_DIR, smileyTheme);
	loadSmileys(base);
	g_free(base);
	
	// load the searching animation
	searchingPath = g_strdup_printf("%sicqnd/search.gif", SHARE_DIR);
	images.searchingAni = gtk_image_get_animation( GTK_IMAGE(gtk_image_new_from_file(searchingPath)));
	g_free(searchingPath);
	
	// append a listener for changed icons
	settings_getSettings()->installListener("appearance", (settingChangedFunc)this->cb_iconThemesChanged, this, TRUE);
}



iconManager::~iconManager()
{
	clearStatusIcons();
	clearSmileys();
	g_list_free(eventIcons);
}



void iconManager::clearStatusIcons()
{
	GList			*search,
					*searchi;
	iconManagerCat	*iconCat;
	iconManagerIcon	*icon;
	
	
	search = icons;
	while(search)
	{
		iconCat = (iconManagerCat*)search->data;
		
		searchi = iconCat->icons;
		while(searchi)
		{
			icon = (iconManagerIcon*)searchi->data;
			
			g_object_unref(G_OBJECT(icon->pb));
			g_free(icon);
			
			searchi = searchi->next;
		}
		
		g_free(iconCat->protocolID);
		g_list_free(iconCat->icons);
		g_free(iconCat);
		
		search = search->next;
	}
	
	if (icons)
		g_list_free(icons);

	icons = NULL;	
}


void iconManager::loadStatusIcons(gchar* base)
{
	GDir			*dir,
					*iconDir;
	const gchar		*currentName;
	gchar			*iconPath,
					*iconName;
	gint			i;
	iconManagerCat	*cat;
	iconManagerIcon	*icon;
	GtkWidget		*img;
	GtkIconFactory	*iconFactory;
	GtkIconSet		*stockIcon; 
	struct _iconManLoad
	{
		gchar	*iconName;
		glong	status;
	} iconManStatus[] =
	{
		{"away.png", BUDDY_STATUS_AWAY},
		{"dnd.png", BUDDY_STATUS_DND},
		{"ffc.png", BUDDY_STATUS_FFC},
		{"invisible.png", BUDDY_STATUS_INV},
		{"invisible_set.png", BUDDY_INV_SET},
		{"na.png", BUDDY_STATUS_NA},
		{"occ.png", BUDDY_STATUS_OCC},
		{"offline.png", BUDDY_STATUS_OFFLINE},		
		{"online.png", BUDDY_STATUS_ONLINE}
	};
	
	
	// eventually clear an old structure
	if (icons)
		clearStatusIcons();
	
	iconFactory = gtk_icon_factory_new();
	
	
	dir = g_dir_open(base, 0, NULL);

	while(currentName = g_dir_read_name(dir))
	{
		iconPath = g_strdup_printf("%s/%s", base, currentName);
		
		if (g_file_test(iconPath, G_FILE_TEST_IS_DIR))
		{
			cat = (iconManagerCat*)g_malloc0(sizeof(iconManagerCat));
			cat->protocolID = g_strdup(currentName);
			
			iconDir = g_dir_open(iconPath, 0, NULL);
			while(currentName = g_dir_read_name(iconDir))
			{
				for (i=0; i<9; i++)
				{
					if (!strcmp(currentName, iconManStatus[i].iconName))
					{
						iconName = g_strdup_printf("%s/%s", iconPath, currentName);
						icon = (iconManagerIcon*)g_malloc0(sizeof(iconManagerIcon));
						icon->identifier = iconManStatus[i].status;
						icon->pb = gdk_pixbuf_new_from_file(iconName, NULL);
						cat->icons = g_list_append(cat->icons, icon);
						g_free(iconName);
						
						// also add an entry to the icon factory for the menu
						iconName = g_strdup_printf("icqnd-status-%s-%d", cat->protocolID, iconManStatus[i].status);
						stockIcon = gtk_icon_set_new_from_pixbuf(icon->pb);
						gtk_icon_factory_add(iconFactory, iconName, stockIcon); 
						g_free(iconName);
					}
				}				
			}
			
			g_dir_close(iconDir);
			
			icons = g_list_append(icons, cat);
			
		}
		
		g_free(iconPath);
	}
	
	gtk_icon_factory_add_default(iconFactory);

	g_dir_close(dir);
}



GdkPixbuf* iconManager::getStatusPixbuf(gchar* protoID, glong status, gboolean inv)
{
	GList	*search;
	iconManagerCat	*cat;
	iconManagerIcon	*icon;
	
	if (inv)
		status = BUDDY_STATUS_INV;		
	
	search = icons;
	
	while(search)
	{
		cat = (iconManagerCat*)search->data;
		if (!g_ascii_strcasecmp(cat->protocolID, protoID))
			break;
		
		search = search->next;
	}
	
	if (!search)
		cat = (iconManagerCat*)icons->data;
	
	search = cat->icons;
	while(search)
	{
		icon = (iconManagerIcon*)search->data;
		if (icon->identifier == status)
			return icon->pb;
		
		search = search->next;
	}
	
	return NULL;
}



GdkPixbuf* iconManager::getUserStatusPixbuf(IMUserDaemon* user)
{
	gchar	*proto;
	
	if (!user || !user->owner || !((IMOwnerDaemon*)user->owner)->protocol)
		return NULL;
	
	if (user->info->pluginID == LICQ_PPID)
	{
		if (isdigit(user->info->licqID[0]))
			proto = "icq";
		else
			proto = "aim";
	} else
		proto = ((IMOwnerDaemon*)user->owner)->protocol->licqName;
	
		return getStatusPixbuf(proto,
							   user->info->status,
							   user->info->isInvisible);
	
	return NULL;
}



GdkPixbuf* iconManager::getEventPixbuf(gint ev)
{
	GList			*search;
	iconManagerIcon	*ico;
	
	if (ev == -1)
		return NULL;
	
	search = eventIcons;
	
	while(search)
	{
		ico = (iconManagerIcon*)search->data;
		if (ico->identifier == ev)
			return ico->pb;
		
		search = search->next;
	}
	
	if (eventIcons)
		return ((iconManagerIcon*)g_list_last(eventIcons)->data)->pb;
	else
		return NULL;
}



void iconManager::loadIcons(gchar *base, stockIconType* icons, GList **iconList)
{
	GtkIconFactory	*iconFactory; // the icon factory containing all icons
	GtkIconSet		*icon; 	      // an icon set (we use one pic for every size)
	GdkPixbuf		*pixbuf;
	gchar			*fileName;
	guint			i;
	iconManagerIcon	*newIcon;
	
	iconFactory = gtk_icon_factory_new();

				
	// load all icons into stock
	i=0;
	while(icons[i].fileName)
	{
		fileName = g_strdup_printf("%s/%s", base, icons[i].fileName);
		if (g_file_test(fileName, G_FILE_TEST_EXISTS))
		{
			pixbuf = gdk_pixbuf_new_from_file(fileName, NULL);
			icon = gtk_icon_set_new_from_pixbuf(pixbuf);
			gtk_icon_factory_add(iconFactory, icons[i].stockID, icon); 
			
			if (iconList)
			{
				newIcon = (iconManagerIcon*)g_malloc0(sizeof(iconManagerIcon));
				newIcon->pb = pixbuf;
				newIcon->identifier = icons[i].id;
				
				*iconList = g_list_append(*iconList, newIcon);
			}
			
		}
		else
			fprintf(stderr, "image file not found: %s\n", fileName);
		
		g_free(fileName);
		
		i++;
	}
	
	gtk_icon_factory_add_default(iconFactory);
}




void iconManager::loadEventIcons(gchar *base)
{	
	stockIconType icons[] = {	
		{ICON_CHAT, "chat.png", USER_EVENT_CHAT},
		{ICON_FILE, "file.png", USER_EVENT_FILE},
		{ICON_MESSAGE, "message.png", USER_EVENT_MESSAGE},
		{ICON_URL, "url.png", USER_EVENT_URL},		
		{ICON_HISTORY, "history.png", 0},
		{ICON_INFO, "info.png", 0},
		{ICON_SMILEYS, "smiley.png", 0},
		{ICON_SYSMSG, "sysmsg.png", 0},
		{NULL, NULL}		
		};

		
		if (eventIcons)
		{
			g_list_foreach(eventIcons, (GFunc)g_free, NULL);
			eventIcons = NULL;
		}
		
		
		loadIcons(base, icons, &eventIcons);
}


void iconManager::loadExtendedIcons(gchar *base)
{
	stockIconType icons[] = {	
		{ICON_BIRTHDAY, "birthday.gif", 0},
		{ICON_TYPING1, "typing1.png", 0},
		{ICON_TYPING2, "typing2.png", 0},
		{ICON_TYPING3, "typing3.png", 0},	
		{NULL, NULL},		
		};	
		
		loadIcons(base, icons, NULL);
	
}



void iconManager::loadSmileys(gchar *base)
{
	FILE	*tokensFile;
	gchar	*tokensName;
	gchar	*buf,
			*smileyName;
	gulong	size,
			pos,
			count,
			smileyPos;
	guint	i;
	GString	*smileyToken,
			*imageName,
			*smileyCount;
	gboolean hasToken;
	GtkWidget	*img;
	GList	*search,
			*genuineIcons;

	
	// open the rules file
	tokensName = g_strdup_printf("%s/tokens.conf", base);
	tokensFile = fopen(tokensName, "r");
	if (!tokensFile)
	{
		fprintf(stderr, "settings::loadSmileys(): Could not open smiley tokens file %s!\n", tokensName);
		g_free(tokensName);
		return;
	}
	
	fseek(tokensFile, 0, SEEK_END);
	size = ftell(tokensFile);
	rewind(tokensFile);

	buf = (gchar*)g_malloc0(size);
	fread(buf, 1, size, tokensFile);
	
	fclose(tokensFile);
	g_free(tokensName);

	// read the file content -> create the smileys tree and load the images	
	pos = 0;
	smileyCount = g_string_new("");
	hasToken = FALSE;
	
	// get the smiley count
	while(buf[pos] != '\n')
		smileyCount = g_string_append_c(smileyCount, buf[pos++]);

	count = strtoul(smileyCount->str, NULL, 10);
	g_string_free(smileyCount, TRUE);
	
	// is the count valid
	if (!count)
	{
		fprintf(stderr, "settings::loadSmileys(): Corrupted tokens.conf file. The beginning of the file must be the smiley count.\n");
		g_free(buf);
		fclose(tokensFile);
		return;
	}
		
	smileyToken = g_string_new("");
	imageName = g_string_new("");
	images.smileys = (iconManagerSmiley*)g_malloc0(sizeof(iconManagerSmiley)*(count+1));
	smileyPos = 0;
	genuineIcons = NULL;
	pos++;
	
	while(pos<size && smileyPos<count)
	{
		// smiley tokens and image names are separated by tabs
		if (buf[pos] == '\t')
			hasToken = TRUE;
		
		// a line is over -> we should have a new pair
		if (buf[pos] == '\n')
		{
			// is the line valid? if not -> return
			if (!strlen(smileyToken->str) || !strlen(imageName->str))
			{
				fprintf(stderr, "settings::loadSmileys(): Corrupted tokens.conf file. Format must be\n		smileytoken[TAB]image name[RETURN]!");
				g_free(tokensName);
				g_free(buf);
				g_string_free(smileyToken, TRUE);
				g_string_free(imageName, TRUE);
				fclose(tokensFile);
				return;
			}
			
			images.smileys[smileyPos].token = g_strdup(smileyToken->str);
			
			smileyName = g_strdup_printf("%s/%s", base, imageName->str);
			images.smileys[smileyPos].icon = gdk_pixbuf_new_from_file(smileyName, NULL);
			
			search = genuineIcons;
			while(search)
			{
				if (!strcmp(smileyName, (gchar*)search->data))
					break;
				search = search->next;
			}
			
			if (!search)
				genuineIcons = g_list_append(genuineIcons, g_strdup(smileyName));
			
			images.smileys[smileyPos].isFirstIcon = (search == NULL);

			g_free(smileyName);
			
			smileyToken = g_string_assign(smileyToken, "");
			imageName = g_string_assign(imageName, "");
			smileyPos++;
			hasToken = FALSE;
			pos++;
		}
		
		if (!hasToken)
			smileyToken = g_string_append_c(smileyToken, buf[pos]);
		else
			if (buf[pos] != '\t' && buf[pos] != '\n')
				imageName = g_string_append_c(imageName, buf[pos]);
		
		pos++;
	}
	
	g_free(buf);
	g_string_free(smileyToken, TRUE);
	g_string_free(imageName, TRUE);
	
	g_list_foreach(genuineIcons, (GFunc)g_free, NULL);
	g_list_free(genuineIcons);
	
	
		
	// create a tree containing the smiley information
	// to avoid looking for every character of a smiley token on every text
	// character a tree is generated.
	// if we have eg. the smileys ":) :( :P", the first node only contains
	// one element: ":" and three branches: ")", "(", "P". This way programs
	// have only to look for the ":" and only THEN look for other possibilities
	// for this smiley.

	
	smileyTree = (smileysNode*)g_malloc0(sizeof(smileysNode));
	
	smileyTree->summary = g_string_new("");
	for (i=0;i<count;i++)
		smileyTree->similies = g_list_append(smileyTree->similies, GUINT_TO_POINTER(i));
	
	// create the smileys tree
	createSmileyTree(smileyTree, 0);
}


void iconManager::clearSmileys()
{
	guint		i;
	smileysNode	*node;
	
	// clear the images
	i = 0;
	while(images.smileys[i].token)
	{
		g_free(images.smileys[i].token);
		g_object_unref(G_OBJECT(images.smileys[i].icon));
		
		i++;
	}
	
	g_free(images.smileys);
	
	
	// clear the smileys tree
	clearSmileysTree(smileyTree);
	smileyTree = NULL;
	
}



void iconManager::clearSmileysTree(smileysNode *node)
{
	GList	*children;
	
	if (node->children)
	{
		children = node->children;
		while(children)
		{
			clearSmileysTree((smileysNode*)children->data);
			children = children->next;
		}
		
		g_list_free(node->similies);
		g_list_free(node->children);
		g_string_free(node->summary, TRUE);
		g_free(node);
	}
}



void iconManager::createSmileyTree(smileysNode *node, guchar pos)
{
	smileysNode *node2;
	GList		*search,
				*search2;
	guint		index;
	
	
	search = node->similies;
	
	while(search)
	{
		index = GPOINTER_TO_UINT(search->data);
		
		if (strlen(images.smileys[index].token)>pos)
		{					
			search2 = node->children;
	
			while(search2)
			{
				node2 = (smileysNode*)search2->data;
			
				if (node2->character == images.smileys[index].token[pos])
					break;
				
				search2 = search2->next;
			}
			
			if (!search2)
			{
				node2 = (smileysNode*)g_malloc0(sizeof(smileysNode));
				node2->summary = g_string_new("");
				node2->character = images.smileys[index].token[pos];
				if (pos+1 == strlen(images.smileys[index].token))
					node2->smileysIndex = index;
				else
					node2->smileysIndex = -1;
				
				g_string_append_c(node->summary, images.smileys[index].token[pos]);
				node->children = g_list_append(node->children, node2);
				
				node2->similies = g_list_append(node2->similies, GUINT_TO_POINTER(index));
				
			} else
				node2->similies = g_list_append(node2->similies, GUINT_TO_POINTER(index)); 
		}
		
		search = search->next;
	}
	
	
	search = node->children;
	while(search)
	{
		node2 = (smileysNode*)search->data;
		
		createSmileyTree(node2, pos+1);
		search = search->next;
	}
	
	// add cross references so the user doesn't need to type a "nose" when typing a smiley
	search = node->children;
	while(search)
	{
		node2 = (smileysNode*)search->data;

		if (node2->character == '-')
			g_list_concat(node->children, node2->children); 			
		
		// createSmileyTree(node2, pos+1);
		search = search->next;
	}
}



void iconManager::cb_iconThemesChanged(gchar* sname, GList *values, iconManager* self)
{
	settingsProperty	*prop;
	gchar				*base;

	while(values)
	{
		prop = (settingsProperty*)values->data;
		
		if (!strcmp(prop->name, "EventTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s/events", SHARE_DIR, g_value_get_string(prop->val));
			self->loadEventIcons(base);
			g_free(base);
		} else
		if (!strcmp(prop->name, "SmileyTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s/smileys", SHARE_DIR, g_value_get_string(prop->val));
			self->loadSmileys(base);
			g_free(base);	
		} else
		if (!strcmp(prop->name, "StatusTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s/status", SHARE_DIR, g_value_get_string(prop->val));
			self->loadStatusIcons(base);
			g_free(base);
		} else
		if (!strcmp(prop->name, "ExtendedTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s/extended", SHARE_DIR, g_value_get_string(prop->val));
			self->loadExtendedIcons(base);
			g_free(base);
		}
		
		values = values->next;
	}
}



iconManager* i_getIcons()
{
	if (!iconMan)
		iconMan = new iconManager();
	
	return iconMan;
}
