/***************************************************************************
 *            imbuddy.cpp
 *
 *  Mon Jan 24 22:04:43 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "imbuddy.h"


IMBuddy::IMBuddy(IMUserDaemon *_user)
{
	if (_user)
	{
		// copy user reference and set callback function
		user = _user;
		user->dowry = this;
		user->setCallback((IMEventCallback)this->cb_buddyCallback, this);
	} else
		user = NULL;
		
	cEntries   = NULL;	
	openEvents = NULL;
	
	if (user)
		settings_getSettings()->installListener("appearance", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);
}


IMBuddy::~IMBuddy()
{
	GList	*eventsSearch;
	
	// delete all windows
	eventsSearch = g_list_copy(openEvents);
	while(eventsSearch)
	{
		((basicWindow*)eventsSearch->data)->destroyWindow();
		delete (basicWindow*)eventsSearch->data;
		
		eventsSearch = eventsSearch->next;
	}
	g_list_free(eventsSearch);
	g_list_free(openEvents);
	
	if (user)
		delete user;
	
	settings_getSettings()->removeListenerByData(this);
}



basicWindow* IMBuddy::getOpenedEvent(userEventType evType)
{
	GList *search;

	
	search = openEvents;
	while(search)
	{
		if (((basicWindow*)search->data)->wType == evType)
		{
			return (basicWindow*)search->data;
		}
		search = search->next;
	}	
	
	return NULL;
}


	
gboolean IMBuddy::openEvent(userEventType evType, gboolean isIncoming)
{
	basicWindow	   *newEvent;
	IMEventManager *newMan;
	gboolean	   openWindow,
				   setConvURL;
	
	// FIXME: this is somehow dirty
	if (evType == UE_URL)
	{
		evType = UE_MESSAGE;
		setConvURL = TRUE;
	} else
		setConvURL = FALSE;
	
	newEvent = NULL;
	
	if ( (newEvent = getOpenedEvent(evType)))
		newEvent->focusWindow();

	
	
	if (!newEvent)
	{
		openWindow = TRUE;
	
		switch(evType)
		{
			case UE_MESSAGE:
				newEvent = (basicWindow*)new conversationWindow();
				newMan = (IMEventManager*)new IMMessageManager(user->info);			
				break;
		
			case UE_FILETRANSFER:
				newEvent = (basicWindow*)new fileTransferWindow(FALSE);
				newMan = (IMEventManager*)new IMFileTransferManager(user->info);
				openWindow = FALSE;
				break;

			case UE_FILETRANSFERREQUEST:
				newEvent = (basicWindow*)new fileTransferWindow(TRUE);
				newMan = (IMEventManager*)new IMFileTransferManager(user->info);
				newEvent->manager = newMan;		
				((fileTransferWindow*)newEvent)->createRequestDialog(NULL);		
				openWindow = FALSE;
				break;


			case UE_CHATREQUEST:
				newEvent = (basicWindow*)new chatWindow(TRUE);
				newMan = (IMEventManager*)new IMChatManager(user->info);
				newEvent->manager = newMan;	
				((chatWindow*)newEvent)->createRequestDialog(NULL);
				openWindow = FALSE;
				break;
		
		
			case UE_CHAT:
				newEvent = (basicWindow*)new chatWindow(FALSE);
				newMan = (IMEventManager*)new IMChatManager(user->info);
				openWindow = FALSE;
				break;
		
		
			case UE_USERINFO:
				newEvent = (basicWindow*)new userInfoWindow();
				newMan = new IMUserUpdateManager(user->info);
				break;
		
		
			case UE_HISTORY:			
				newEvent = (basicWindow*)new historyWindow();	
				newMan = new IMHistoryManager(user->info);
				break;

			case UE_AUTHORIZATION:
			case UE_AUTHORIZATIONREQUEST:
			case UE_AUTHORIZATIONGRANT:
				if (isIncoming && evType == UE_AUTHORIZATIONGRANT)
				{
					newEvent = (basicWindow*)new simpleMessageWindow(UE_AUTHORIZATIONGRANT, tr("<small>The user accepted your request to add him to his contact list</small>"), tr("<b><i>%s</i> authorized you</b>"));
					newMan = new IMAuthorizationManager(user->info);
				} else
				{
					newEvent = (basicWindow*)new authorizeWindow(evType);
					newMan = new IMAuthorizationManager(user->info);
				}
				break;		
		
			case UE_AUTHORIZATIONREFUSE:
				newEvent = (basicWindow*)new simpleMessageWindow(UE_AUTHORIZATIONREFUSE, tr("<small>The user rejected your request to add him to your contact list</small>"), tr("<b><i>%s</i> rejected your request</b>"));
				newMan = new IMAuthorizationManager(user->info);
				break;
		
			case UE_AUTORESPONSESET:
				newEvent = (basicWindow*)new autoResponseWindow(0);
				newMan = new IMAutoResponseManager(user->info, user->owner == NULL);
				break;
		
		
			case UE_AUTORESPONSEGET:
				newEvent = (basicWindow*)new fetchResponseWindow();
				newMan = new IMAutoResponseManager(user->info, FALSE);
				break;
		
			case UE_ENCRYPTION:
				newEvent = (basicWindow*)new securityWindow();
				newMan = new IMSecurityManager(user->info);
				break;
						
			case UE_ADDEDTOLIST:
				newEvent = (basicWindow*)new simpleMessageWindow(UE_ADDEDTOLIST, tr("<small>The user has added you to his contact list</small>"), tr("<b><i>%s</i> added you</b>"));
				newMan = new IMMiscManager(user->info);
				break;
			
			case UE_EDITUSERMODES:
				newEvent = (basicWindow*)new modesWindow();
				newMan = new IMEventManager(user->info);
				break;


			default:
				fprintf(stderr, "IMBuddy::openEvent: unknown new action!\n");
				return FALSE;
		}
	
		// set the newMan->setCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);callback and create the window if neccessary
		newEvent->manager = newMan;
		newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);
		newMan->setCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
		user->addManager(newMan);
		if (openWindow)
			newEvent->createWindow();
		user->initializeManagerForUser(newMan);
	
		openEvents = g_list_append(openEvents, newEvent);
	};
	
	
	// post processing
	switch(evType)
	{
		case UE_MESSAGE:
			if (isIncoming)
				break;
			if (setConvURL)
				((conversationWindow*)newEvent)->enableURLMode();
			else			
				((conversationWindow*)newEvent)->disableURLMode();
			break;
			
		default:
			break;
	}
	
	return TRUE;
}



gboolean IMBuddy::closeEvent(userEventType evType)
{
	return TRUE;	
}






void IMBuddy::updateContactListStatus(gboolean updateAlias)
{
	GList			 *search;
	contactListUser  *entry;
	
	search = cEntries;
	while(search)
	{
		entry = (contactListUser*)search->data;		

		entry->setStatus(user->info->status, 
						 i_getIcons()->getUserStatusPixbuf(user));
		//uu_getStatusSymbol(user->info->status, user->info->isInvisible));
		
		if (updateAlias)
			entry->setName(user->info->formattedAlias);

		search = search->next;
	}
	
	
}


void IMBuddy::updateContactListEvents(basicEventInfo* ev, gboolean moveToTop)
{
	GdkPixbuf	*toFlash;
	GList		*search;
	
	toFlash = getEventSymbol((basicEventInfo*)ev);		
		
	search = cEntries;
	while(search)
	{
		((contactListUser*)search->data)->setFlashing(toFlash);
		if (moveToTop)
			((contactListUser*)search->data)->moveToTop();
					
		search = search->next;
	}	
}



void IMBuddy::changeStateIcons(gulong toChange, gboolean add)
{
	GList	*search;
	gulong	newState;
	
	search = cEntries;
	
	while(search)
	{
		newState = ((contactListUser*)search->data)->getStateMask();
		if (!add)
			newState = (newState | toChange)^toChange;
		else
			newState = newState | toChange;		
		
		((contactListUser*)search->data)->setStateMask(newState);
					
		search = search->next;
	}	
}



void IMBuddy::putUserIntoGroups(contactListEntry* root)
{
	GList				*search;
	contactListEntry	*entry,
						*newEntry;

	
	search = user->groups;
	while(search)
	{
		entry = root->findEntryByID(((groupInfo*)search->data)->icqndID, CE_GROUP);
				
		if (entry)
		{
			// doOpen = ((contactListGroup*)entry)->getGroupOpened();
			
			newEntry = new contactListUser(
							user->info->licqID,
							user->info->formattedAlias,
							i_getIcons()->getUserStatusPixbuf(user),
			//				uu_getStatusSymbol(user->info->status, user->info->isInvisible),
							user->info->status,
							user,
							cEntries);
					
					
			entry->addChild((contactListEntry*)newEntry);
			/*		
			group = (groupInfo*)entry->data;
			((contactListGroup*)entry)->setGroupOpened(doOpen);*/
			cEntries = g_list_append(cEntries, newEntry);
					
		} else
			fprintf(stderr, "IMOwner::putUserIntoGroups(): No group was found for %s!\n", user->info->formattedAlias); 				
								
		search = search->next;
	}	
}



GdkPixbuf* IMBuddy::getEventSymbol(basicEventInfo *info)
{
	gint last;


	if (!info)
	{
		info = user->getEvent(EO_LAST | EO_NORMAL | EO_SYSTEM);
		if (info)
			last = info->type;
		else
			last = -1;
	}
	else
		last = info->type;
	
	return i_getIcons()->getEventPixbuf(last);	
}





// =============================================================================

gboolean IMBuddy::cb_buddyCallback(userEventType type, 
					               gint command, 
								   gint lastResult, gpointer info, 
								   IMBuddy* self)
{
	GList		*search;
		
	switch(command)
	{
		// some window wants another window to be opened
		case EC_FROMMANAGER:
			self->openEvent((userEventType)lastResult, FALSE);
			break;
		
		// status was updated (FIXME: analyze this command further)
		case UC_UPDATEUSER:
			self->updateContactListStatus(lastResult == USER_UPDATE_ALL || lastResult == USER_UPDATE_GENERAL);

			// update the typing icon
			if (lastResult == USER_UPDATE_TYPING)
			{
				if (self->user->info->typing == 2)
					self->changeStateIcons(1, TRUE);
				else
					self->changeStateIcons(1, FALSE);
			}
			// break;
			break;
			
		case UC_AUTORESPONSE_CHECKED:
			search = self->cEntries;
			while(search)
			{
				((contactListUser*)search->data)->blinkEntry();
				search = search->next;
			}
			break;
		
		case UC_UPDATE_ADDSTATE:
		case UC_UPDATE_REMOVESTATE:
			self->changeStateIcons(lastResult, command == UC_UPDATE_ADDSTATE);
			break;			
		
					
		// a new event was received or finished
		case UC_EVENT_FINISHED:
		case UC_EVENT_RECEIVED:
			self->updateContactListEvents((basicEventInfo*)info, command == UC_EVENT_RECEIVED);
			break;
			
			
		// a window was destroyed
		case WC_DESTROYED:
			search = self->openEvents;
			while(search)
			{
				if (search->data == info)
				{
					self->openEvents = g_list_remove(self->openEvents, search->data);
					fprintf(stderr, "IMBuddy::cb_buddyCallback: event window destroyed (%d left)!\n", g_list_length(self->openEvents));
					break;
				}
				
				search = search->next;
			}
			break;
	}	
	
	return FALSE;
}



void IMBuddy::cb_settingsChangedCallback(gchar* sname, GList *values, IMBuddy* self)
{
	gboolean 			redrawWindows;
	settingsProperty	*prop;
	GList				*search;
	basicWindow			*wnd;
	
	redrawWindows = FALSE;
	
	while(values)
	{
		prop = (settingsProperty*)values->data;
		
		if (!strcmp(prop->name, "EventTheme"))
		{
			redrawWindows = TRUE;
			break;
		}
		
		values = values->next;
	}
	
	// redraw all relevant areas
	if (redrawWindows)
	{
		search = self->openEvents;
		while(search)
		{
			wnd = (basicWindow*)search->data;
			
			if (wnd->content)
				gtk_widget_queue_draw(wnd->content);
			
			search = search->next;
		}
	}
}




// ==================== IMOwner ================================================

IMOwner::IMOwner():
IMBuddy::IMBuddy(NULL)
{
	GList				*rawProtos,
						*groups,
						*search;
	IMBuddy				*newProto;
	IMGroupManager		*groupMan;
	gulong				openedGroups,
						i;
	gchar				*defaultEncoding;
	gboolean			blinkEvents;
	
	setupDefaultSettings();
	
	// setup logging
	logDaemon = new IMLogDaemon();
	
	
	groupMan = new IMGroupManager();
	groups = groupMan->loadGroups();
	IO_setGroupManager(groupMan);
	
	
	// set the opened groups
	settings_getSettings()->getProperties("contactlist", 
										  "lastOpenedGroups", &openedGroups,
										  "blinkEvents", &blinkEvents,
										  NULL);
										  
	settings_getSettings()->getProperties("conversations",
										  "standardEncoding", &defaultEncoding,
										  NULL);
	i=0;
	groups = groupMan->groups;
	while(groups)
	{
		((groupInfo*)groups->data)->opened = (openedGroups & (1<<i))!=0;
		i++;
		groups = groups->next;
	}
	
	eventCount = 0;
	ownerEventCount = 0;
	
	// connect to the general signal source
	IO_getGeneralSource()->addCallback((IMEventCallback)this->cb_generalCallback, this);
	
	// load all plugins
	IO_getPluginsList();
	
	// get all protocol owners
	rawProtos = IO_getOwnerList();

	while(rawProtos)
	{
		newProto = new IMBuddy((IMUserDaemon*)rawProtos->data);
		newProto->user->setCallback((IMEventCallback)this->cb_ownerCallback, this);
		newProto->user->dowry = newProto;
		newProto->user->setCharset(defaultEncoding, FALSE);

		((IMOwnerDaemon*)newProto->user)->loadContacts(CLT_NORMAL | CLT_VISIBLE);
				
		rawProtos = rawProtos->next;
	}
	
	// create main window
	window = new mainWindow((mwMenuClickFunction)this->cb_menuCallback, G_CALLBACK(this->cb_quitWindowCallback), this);	

	setWindowTitle();

	
	// check for saved owner events
	search = IO_getOwnerList();
	while(search)
	{
		((IMUserDaemon*)search->data)->dispatchPendingEvents();
		
		search = search->next;
	}
	
	
	// build the contact list and autoconnect or call the new user wizard
	contactList_setBlinkFlashing(blinkEvents);
	
	if (g_list_length(IO_getOwnerList())>0)
	{
		rebuildContactList(TRUE);
		autoConnect();	
	}
	else
	{
		window->setLockUserMenus(TRUE);
		openEvent(UE_REGISTEROWNER, 0);
	}		
	
	settings_getSettings()->installListener("appearance", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);
		
	errorCode = 0;
}



void IMOwner::setWindowTitle()
{
	GString			*title;
	gchar			*tmpTitle;
	GList			*search;
	IMOwnerDaemon	*owner;

	search = IO_getOwnerList();

	title = g_string_new("");
	
	while(search)
	{
		owner = (IMOwnerDaemon*)search->data;
		
		tmpTitle = uu_getFormattedBuddyName(owner->info, FALSE);
		g_string_append_printf(title, "%s, ", tmpTitle);
		g_free(tmpTitle);
		
		search = search->next;
	}	

	
	if (strlen(title->str)>0)
		title = g_string_set_size(title, strlen(title->str)-2);
	
	tmpTitle = g_strdup(title->str);
	g_string_printf(title, "Licq (%s)", tmpTitle);
	g_free(tmpTitle);
	
	window->setTitle(title->str);
	window->setPendingEventNotify(eventCount != 0 || ownerEventCount != 0);
	
	g_string_free(title, TRUE);
}




gboolean IMOwner::setupDefaultSettings()
{
	settings	*config;
	GdkColor	log_errorCol,
				log_warningCol,
				log_packetCol,
				log_defaultCol,
				conv_messageCol,
				conv_typingCol;
	gchar		*encoding;
	
	// create the settings and set it as default
	config = new settings("licq_qnd-gui.conf");
	settings_setSettings(config);	
	
	// defaults for the logging window
	GDK_COLOR_SET(&log_errorCol, 0xFFFF, 0x0000, 0x0000);
	GDK_COLOR_SET(&log_warningCol, 0x8888, 0x0000, 0x0000);
	GDK_COLOR_SET(&log_packetCol, 0x4444, 0x4444, 0x4444);
	GDK_COLOR_SET(&log_defaultCol, 0x0000, 0x0000, 0x0000);
	
	config->installProperties("logwindow",
							  "errorColor", ICQND_TYPE_COLOR, &log_errorCol,
							  "warningColor", ICQND_TYPE_COLOR, &log_warningCol,
							  "packetColor", ICQND_TYPE_COLOR, &log_packetCol,
							  "defaultColor", ICQND_TYPE_COLOR, &log_defaultCol,
							  NULL);
	
	// defaults for the main window
	config->installProperties("mainwindow",
							  "width", G_TYPE_ULONG, 100,
							  "height", G_TYPE_ULONG, 300,
							  "positionX", G_TYPE_ULONG, 0,
							  "positionY", G_TYPE_ULONG, 0,
							  NULL);

	
	// defaults for the startup
	config->installProperties("startup",
							  "status", G_TYPE_ULONG, 0,
							  NULL);
	
	
	GDK_COLOR_SET(&conv_messageCol, 0x0000, 0x0000, 0xFFFF);
	GDK_COLOR_SET(&conv_typingCol, 0xF300, 0xF500, 0x2500);
	
	
	// defaults for the appearance
	config->installProperties("appearance",
							  "containerMode", G_TYPE_ULONG, CM_MESSAGEWINDOW,
							  "showTrayIcon", G_TYPE_BOOLEAN, TRUE,
							  // defaults for the typing notification
							  "tabMessageColor", ICQND_TYPE_COLOR, &conv_messageCol, 
							  "tabTypingColor", ICQND_TYPE_COLOR, &conv_typingCol,	
							  // defaults for the themes
					  		   "EventTheme", G_TYPE_STRING, "default",
					  		   "SmileyTheme", G_TYPE_STRING, "default",
					  		   "StatusTheme", G_TYPE_STRING, "default",
					  		   "ExtendedTheme", G_TYPE_STRING, "default",
					  		   NULL);
	
	
	// defaults for conversations
	config->installProperties("conversations",
							  "standardEncoding", G_TYPE_STRING, "UTF-8",
							  "sendTyping", G_TYPE_BOOLEAN, TRUE,
							  "showSmileys", G_TYPE_BOOLEAN, TRUE,
							  "showURIs", G_TYPE_BOOLEAN, TRUE,
							  "showHistory", G_TYPE_BOOLEAN, TRUE,
							  "historyCount", G_TYPE_ULONG, 5,
							  "colorsSet", G_TYPE_BOOLEAN, FALSE,
							  "foregroundColor", ICQND_TYPE_COLOR, NULL,
							  "backgroundColor", ICQND_TYPE_COLOR, NULL,
							  // defaults for the toolbar
							  "remoteToolbarRules", G_TYPE_STRING, "1,2,20,3,18,4,5,6",
							  "localToolbarRules", G_TYPE_STRING, "7,8,9,10,18,11,12",
							  "buttonbarRules", G_TYPE_STRING, "17,13,14,20,16,15",
							  "remoteToolbarSpacing", G_TYPE_ULONG, 2,
							  "localToolbarSpacing", G_TYPE_ULONG, 4,
							  "buttonbarSpacing", G_TYPE_ULONG, 0,
							  "showRemoteToolbar", G_TYPE_BOOLEAN, TRUE,
							  "showLocalToolbar", G_TYPE_BOOLEAN, TRUE,
							  "showButtonbar", G_TYPE_BOOLEAN, TRUE,	
							  "messageTag", G_TYPE_STRING, "%t %a: ",
							  "dontApplyColors", G_TYPE_BOOLEAN, FALSE,					  
							  NULL); 
							  							  
	
	// defaults for file transfer
	config->installProperties("filetransfer",
							  "lastDirectory", 	G_TYPE_STRING, getenv("HOME"),
							  NULL);
							  
	// defaults for the contact list
	config->installProperties("contactlist",
							  "showOfflineUsers", G_TYPE_BOOLEAN, TRUE,
							  "showGroups", G_TYPE_BOOLEAN, TRUE,
							  "showEmptyGroups", G_TYPE_BOOLEAN, TRUE,
							  "lastOpenedGroups", G_TYPE_ULONG, 0, 
							  "blinkEvents", G_TYPE_BOOLEAN, TRUE,
							  "showTooltips", G_TYPE_BOOLEAN, TRUE,
							  "tooltipsMask", G_TYPE_ULONG, 0xFFFFFFFF,
							  NULL);
							  
							  
	// set the values that require a fast access
	config->getProperties("conversations",
						  "standardEncoding", &encoding,
						  NULL);
	config->fastAccess.preferredCharset = g_strdup(encoding);
	
	
	// setup the icons
	i_getIcons();
	
	
	return TRUE;
}




IMOwner::~IMOwner()
{
	settings	*config;
	
	// save and delete the config
	config = settings_getSettings();
	
	config->writeSettings();
	delete config;
	
	
	// unregister us from the logging manager
	delete logDaemon;
	
	// delete the main window
	delete window;
}



gboolean IMOwner::openEvent(userEventType evType, guint status)
{
	basicWindow	*newEvent;
	IMEventManager *newMan;
	IMBuddy *buddy;
	IMOwnerDaemon	*owner;
	GList *search;
	
	
	// set a new autoresponse
	if (evType == UE_AUTORESPONSESET)
	{
		
		if (!IO_getOwnerList())
			return FALSE;
		
		buddy = (IMBuddy*)((IMUserDaemon*)IO_getOwnerList()->data)->dowry;
	
		newEvent = buddy->getOpenedEvent(evType);	
		
		if (newEvent)
		{
			((autoResponseWindow*)newEvent)->updateStatus(status);
			newEvent->focusWindow();
		} else
		{
			newEvent = (basicWindow*)new autoResponseWindow(status);
			newMan = new IMAutoResponseManager(buddy->user->info, TRUE);
			newMan->setCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
			newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);

			newEvent->manager = newMan;
			buddy->user->addManager(newMan);				
			newEvent->createWindow();
			buddy->openEvents = g_list_append(buddy->openEvents, newEvent);
		}	

		return TRUE; 		
	} 
	/*
	if (evType == UE_USERINFO)
	{
		if (!protocols)
			return FALSE;
		
		buddy = (IMBuddy*)protocols->data;
		
		newEvent = buddy->getOpenedEvent(evType);
		
		if (!newEvent)
		{
			newEvent = (basicWindow*)new userInfoWindow();
			newMan = new IMUserUpdateManager(buddy->user->info);
			newMan->setCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
			newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);

			newEvent->manager = newMan;
			buddy->user->addManager(newMan);
			newEvent->createWindow();
			buddy->openEvents = g_list_append(buddy->openEvents, newEvent);
		} else
		newEvent->focusWindow();
		
		return TRUE;
	} else */
	// open whitepages window if licq user exists
	if (evType == UE_WHITEPAGES)
	{
		search = IO_getOwnerList();
		while(search)
		{
			owner = (IMOwnerDaemon*)search->data;
			
			if (owner->info->pluginID == LICQ_PPID)
				break;
			
			search = search->next;
		}			
		
		if (search)
		{
			newEvent = ((IMBuddy*)owner->dowry)->getOpenedEvent(UE_WHITEPAGES);
			if (newEvent)
				newEvent->focusWindow();
			else
			{
				newEvent = (basicWindow*)new searchForUserDialog();
				newMan = new IMUserSearchManager(owner->info);
				newMan->setCallback((IMEventCallback)newEvent->cb_eventCallback, newEvent);
				newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);

				newEvent->manager = newMan;
				owner->addManager(newMan);
				newEvent->createWindow();
				((IMBuddy*)owner->dowry)->openEvents = g_list_append(((IMBuddy*)owner->dowry)->openEvents, newEvent);
			}
		} else
		fprintf(stderr, "IMOwner::openEvent(): Whitepages only available in icq protocol\n");

		return TRUE;		
	}

	
	// other general events
	newEvent = getOpenedEvent(evType);
	
	if (newEvent)
	{
		newEvent->focusWindow();
		return TRUE;
	}
		
	switch(evType)
	{			
		case UE_AUTHORIZATIONGRANT:
			newEvent = (basicWindow*)new IDActionWindow(UE_AUTHORIZATIONGRANT, (IDActionCallback)this->cb_idActionCallback, this, "<small>Please enter the user id (and the protocol) for the user that you want to send an authorization</small>", TRUE, NULL, FALSE);
			break;
		
		case UE_AUTHORIZATIONREQUEST:
			newEvent = (basicWindow*)new IDActionWindow(UE_AUTHORIZATIONREQUEST, (IDActionCallback)this->cb_idActionCallback, this, "<small>Please enter the user id (and the protocol) for the user that you want to send an authorization request</small>", TRUE, NULL, FALSE);
			break;
		
		case UE_ADDUSER:
			newEvent = (basicWindow*)new IDActionWindow(UE_ADDUSER, (IDActionCallback)this->cb_idActionCallback, this, "<small>Please enter the user id (and the protocol) for the user that you want to add to your contact list</small>", FALSE, "_Alert user", TRUE);
			break;
		
		case UE_EDITGROUPS:
			newEvent = (basicWindow*)new groupsWindow((groupsWindowCallback)this->cb_groupWindowCallback, this);
			break;
		
		case UE_REGISTEROWNER:
			newEvent = new newOwnerWindow("<big><b>Add a New User Account to Licq</b></big>");
			break;
		
		case UE_MANAGEOWNERS:
			newEvent = new ownerManagerWindow();
			break;
		
		case UE_LOGMESSAGES:
			newEvent = new logWindow(logDaemon);
			break;
		
		case UE_SETOWNERPROPERTIES:
			u_showAlertMessage("Sorry", "Not implemented yet", GTK_STOCK_DIALOG_WARNING);
			break;
		
		case UE_OPTIONS:
			newEvent = new optionsWindow();
			break;

		
		default:
			// IMBuddy::openEvent(evType, FALSE); -> THIS WILL CRASH
			break;
	}
	
	if (newEvent)
	{
		newEvent->setDestroyCallback((IMEventCallback)this->cb_buddyCallback, this);
		newEvent->createWindow();
		openEvents = g_list_append(openEvents, newEvent);
	}
	
	return TRUE;
}



IMBuddy* IMOwner::getBuddyForLastGlobalEvent(gboolean favourSystemEvents, basicEventInfo** info)
{
	IMBuddy			*focus;
	IMUserDaemon	*tmpFocus, 
					*findFocus;
	GList			*search;
	basicEventInfo	*tmpEvent, 
					*basicEvent;
	time_t	 		t;

	
	t = 0;
	findFocus = NULL;
	search = IO_getOwnerList();
	
	while(search)
	{
		if ((tmpFocus = ((IMOwnerDaemon*)search->data)->getLastEvent(&tmpEvent, TRUE)) && tmpEvent->time > t)
		{					
			findFocus = tmpFocus;
			basicEvent = tmpEvent;
		}
				
		search = search->next;
	}
			
	// a recent event was found
	if (findFocus)
	{
		focus = (IMBuddy*)findFocus->dowry;
				
		// a user and no buddy around it? this must be a new user -> create a buddy
		if (!focus)
		{
			focus = new IMBuddy(findFocus);
			findFocus->dowry = focus;
		}	
		
		if (info)
			*info = basicEvent;
		
		return focus;
	}
	
	return NULL;
}



void IMOwner::setStatus(guint status, gboolean promptAR)
{
	GList			*owners;
	gboolean		showConnect;
	IMOwnerDaemon	*owner;
	
	
	owners = IO_getOwnerList();
	
	showConnect = FALSE;
	while(owners)
	{
		owner = (IMOwnerDaemon*)owners->data;
		
		if (status == BUDDY_STATUS_INV)
		{
			if (!owner->info->isInvisible)
				owner->setStatus(BUDDY_STATUS_ONLINE);			
			owner->setInvisible(!owner->info->isInvisible);
		} else
			owner->setStatus(status);
		
		// eventually start the connection animation on the proto button
		if (owner->info->status == BUDDY_STATUS_OFFLINE && status != BUDDY_STATUS_OFFLINE)
			window->updateStatusButton(owner, BUDDY_STATUS_CONNECT, FALSE);
		else
			window->updateStatusButton(owner, owner->info->status, owner->info->isInvisible);
		
		if (owner->info->status == BUDDY_STATUS_OFFLINE && status != BUDDY_STATUS_OFFLINE)
			showConnect = TRUE;
		
		owners = owners->next;
	}
	
	// check whether the auto response window should be prompted
	if (promptAR && 
		status != BUDDY_STATUS_ONLINE && 
		status != BUDDY_STATUS_OFFLINE &&
		status != BUDDY_STATUS_INV)
		openEvent(UE_AUTORESPONSESET, status);
	

	if (showConnect && status != BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_CONNECT, FALSE);
	else
	if (showConnect && status == BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_OFFLINE, FALSE);		
}



/*

void IMOwner::setStatus(guint status)
{
	GList 	*search;
	gboolean someoneOffline;
	
	// ask all protocols to change the status
	search = protocols;
	
	someoneOffline = FALSE;
	
	while(search)
	{
		if ( ((IMBuddy*)search->data)->user->info->status == BUDDY_STATUS_OFFLINE)
			someoneOffline = TRUE;
	
		((IMOwnerDaemon*)((IMBuddy*)search->data)->user)->setStatus(status);
					
		search = search->next;
	}
	
	if (someoneOffline && status != BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_CONNECT, FALSE);
	else
	if (someoneOffline && status == BUDDY_STATUS_OFFLINE)
		window->updateStatus(BUDDY_STATUS_OFFLINE, FALSE);		
}


void IMOwner::setInvisible(gboolean setting)
{
	GList 		*search;
	gboolean 	goOnline;
	// ask all protocols to change the status
	search = protocols;
	
	goOnline = FALSE;
	
	while(search)
	{
		if ( ((IMBuddy*)search->data)->user->info->status == BUDDY_STATUS_OFFLINE)
		{
			goOnline = TRUE;
			((IMOwnerDaemon*)((IMBuddy*)search->data)->user)->setStatus(BUDDY_STATUS_ONLINE);
		}

		((IMOwnerDaemon*)((IMBuddy*)search->data)->user)->setInvisible(setting);		
					
		search = search->next;
	}
	
	if (goOnline)
		window->updateStatus(BUDDY_STATUS_CONNECT, FALSE);			
} */



gboolean IMOwner::autoConnect()
{
	settings	*config;
	gulong 		status;
	
	config = settings_getSettings();
	
	config->getProperties("startup",
						  "status", &status,
						  NULL);

	switch (status)
	{
		case 0: break;
		case 1: setStatus(BUDDY_STATUS_ONLINE); break;
		case 2: setStatus(BUDDY_STATUS_AWAY); break;
		case 3: setStatus(BUDDY_STATUS_NA); break;
		case 4: setStatus(BUDDY_STATUS_OCC); break;
		case 5: setStatus(BUDDY_STATUS_DND); break;
		case 6: setStatus(BUDDY_STATUS_FFC); break;
		default: fprintf(stderr, "IMOwner::autoConnect: Invalid auto online id: %d.\n", status); break;
	}

	return TRUE;
}



void IMOwner::rebuildContactList(gboolean dispatchEvents)
{
	GList	*search,
			*searchOwners,
			*searchGroups;
	IMOwnerDaemon* owner;
	IMUserDaemon* user;
	IMBuddy* buddy;
	groupInfo	*group;
	contactListEntry	*groupEntry;
	contactListGroup	*tmpGroup;
	basicEventInfo	*basicEvent;
	
	// clear the old contact list
	if (window->contacts->children)
	{
		// save the old opening state
		searchGroups = window->contacts->getChildrenOfType(CE_GROUP);
		search = searchGroups;
		while(search)
		{
			tmpGroup = (contactListGroup*)search->data;
			group = (groupInfo*)tmpGroup->data;
			// does this group still exist?
			if (IO_getGroupManager()->getGroupByIcqndID(tmpGroup->id))
				group->opened = tmpGroup->getGroupOpened();
			search = search->next;
		}
		g_list_free(searchGroups);
		
		search = g_list_copy(window->contacts->children);
		while(search)
		{
			delete (contactListEntry*)search->data;
		
			search = search->next;
		}
	}
	
	if (!IO_getOwnerList())
		return;
	
	
	// build the groups
	search = IO_getGroupManager()->groups;
	while(search)
	{
		group = (groupInfo*)search->data;
		groupEntry = new contactListGroup(group->icqndID, group->name, group);
		((contactListGroup*)groupEntry)->setGroupOpened(group->opened);
		window->contacts->addChild(groupEntry);
		search = search->next;
	}
	
	
	
	// rebuild the contact list
	searchOwners = IO_getOwnerList();
	while(searchOwners)
	{
		owner = (IMOwnerDaemon*)searchOwners->data;	
		
		fprintf(stderr, "IMOwner::rebuildContactList(): Rebuild list for %s\n", owner->info->alias);
		
		search = owner->users;
		while(search)
		{
			user = (IMUserDaemon*)search->data;
			
			if (!user->dowry)
				buddy = new IMBuddy(user);
			else
				buddy = (IMBuddy*)user->dowry;
			
			g_list_free(buddy->cEntries);
			buddy->cEntries = NULL;
			
			buddy->putUserIntoGroups(window->contacts);
			
			// update events if there are some
			if (dispatchEvents)
				user->dispatchPendingEvents();
			
			basicEvent = user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);

			if (basicEvent)
				buddy->updateContactListEvents(basicEvent, TRUE);
			
			search = search->next;
		}
		
		searchOwners = searchOwners->next;
	}	
}



void IMOwner::updateAllUsers()
{
	gint i;
	
	updateTime = time(NULL);
	
	for (i=0;i<2;i++)
		updateOneUser(window->contacts);
}



gboolean IMOwner::updateOneUser(contactListEntry* pos)
{
	IMBuddy				*buddy;
	contactListEntry	*entry;
	GList				*usearch,
						*esearch;
	IMUserUpdateManager *uman;
	
	usearch = pos->children;
	
	while(usearch)
	{
		entry = (contactListEntry*)usearch->data;
		
		if (entry->type == CE_USER)
		{
			buddy = (IMBuddy*)((IMUserDaemon*)entry->data)->dowry;
			if (buddy->user->info->lastUpdated < updateTime)
			{
				esearch = buddy->cEntries;
				while(esearch)
				{
					entry = (contactListEntry*)esearch->data;
					entry->setHighlighted(TRUE);
					esearch = esearch->next;
				}
				
				buddy->user->info->lastUpdated = updateTime;
				uman = new IMUserUpdateManager(buddy->user->info);
				uman->setCallback((IMEventCallback)this->cb_massEventCallback, this);
				buddy->user->addManager(uman);
				uman->updateUserInfo(buddy->user->info->keepAlias? buddy->user->info->alias: NULL);
				return TRUE;
			}
		} else
		if (updateOneUser(entry))
			return TRUE;

		usearch = usearch->next;
	}
	
	return FALSE;
}





gboolean IMOwner::cb_ownerCallback(userEventType type, 
								   gint command, 
								   gint lastResult, gpointer info, 
								   IMOwner* self)
{
	GList					*search,
							*search2;
	basicEventInfo			*lastEv;
	contactListEntry		*entry;
	IMBuddy					*newBuddy;
	IMAutoResponseManager	*aman;
	IMUserDaemon			*user;
	IMOwnerDaemon			*owner;
	gchar					*txt;
	gint					newStatus;
	gboolean				newInvisible;

	
	// evaluate the command
	switch (command)
	{
		case EC_FROMMANAGER:
			if (info)
				((IMBuddy*)((IMUserDaemon*)info)->dowry)->openEvent((userEventType)lastResult, 0);
			else
				self->openEvent((userEventType)lastResult, FALSE);
			break;
		
		case UC_UPDATEMODES:
			user = (IMUserDaemon*)info;
			newBuddy = (IMBuddy*)user->dowry;
			search = newBuddy->cEntries;
			while(search)
			{
				delete (contactListEntry*)search->data;
				search = search->next;
			}
			g_list_free(newBuddy->cEntries);
			newBuddy->cEntries = NULL;
			newBuddy->putUserIntoGroups(self->window->contacts);
			
			// overtake old events
			lastEv = newBuddy->user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);

			if (lastEv)
				newBuddy->updateContactListEvents(lastEv, TRUE);
			break;
			
		
		case OC_NEWUSER:
		case OC_LOADUSER:
			user = (IMUserDaemon*)info;
		
			// this is somehow complicated: normally a buddy is wrapped around
			// a user to take over callback stuff, open events ...
			// however, when there was a system message ("a user added you")
			// the buddy is already created by OC_UNKNOWNUSER, so don't create
			// it again!
			if (!user->dowry)
				newBuddy = new IMBuddy(user);
			else
				newBuddy = (IMBuddy*)user->dowry;
			
			newBuddy->putUserIntoGroups(self->window->contacts);			
			// check for old events that were not deleted last time
			newBuddy->user->dispatchPendingEvents();
			
			// self->window->updateStatus(newBuddy->user->info->status);			
			break;
			
/*			
		case OC_NEWGROUP:
			if (!(newGroup = (contactListGroup*)self->window->contacts->findEntryByID((gchar*)info, CE_GROUP)))
			{		
				newGroup = new contactListGroup((gchar*)info, 
											    NULL);
				if (lastResult != 0)
					newGroup->setDefault(TRUE);
		
				self->window->contacts->addChild(newGroup);
			}
			break;
	*/	
		
		case OC_REMOVEUSER:
			user = (IMUserDaemon*)info;
			if (!user)
				break;
			
			newBuddy = (IMBuddy*)user->dowry;
			if (!newBuddy)
				break;
			
			search = newBuddy->cEntries;
			
			// delete that user from the contact list
			while(search)
			{
				entry = (contactListEntry*)search->data;
				delete entry;
				
				search = search->next;
			}	
			
			// decrease the group count
			search = IO_getGroupManager()->groups;
			while(search)
			{
				search2 = user->groups;
				
				while(search2)
				{
					if (search2->data == ((groupInfo*)search->data)->icqndID)
						((groupInfo*)search->data)->numUsers--;						
					
					search2 = search2->next;
				}
				
				search = search->next;
			}

			fprintf(stderr, "IMOwner::cb_ownerCallback(): Removed user %s from contact list\n", user->info->alias);				
			
			delete newBuddy;
			break;
		
		
		case OC_UPDATEUSER:
			break;
		
		
		case OC_SETSTATE:
		case UC_UPDATEUSER:
			owner = (IMOwnerDaemon*)info;
			self->window->updateStatusButton(owner, owner->info->status, owner->info->isInvisible);

			// try to make a good "average" status
			newStatus = BUDDY_STATUS_OFFLINE;
			newInvisible = FALSE;
		
			search = IO_getOwnerList();
			while(search)
			{
				owner = (IMOwnerDaemon*)search->data;
				
				if (owner->info->status != BUDDY_STATUS_OFFLINE)
				{	
					// if no status is defined set the best one
					if (newStatus == BUDDY_STATUS_OFFLINE)
						newStatus = owner->info->status;
					else
					// more protos are online with different statuses. Take "online" and quit
					if (newStatus != owner->info->status)
					{
						newStatus = BUDDY_STATUS_ONLINE;
						break;
					}
				}
				
				search = search->next;
			}				
		
			self->window->updateStatus(newStatus, FALSE);
			/*
			user = (IMUserDaemon*)info;
			// licq sends a logoff before logging on -> ignore that
			if (!(user->info->status == BUDDY_STATUS_OFFLINE && self->window->isTrayAnimation()))
				self->window->updateStatus(user->info->status, user->info->isInvisible);
			*/
			break;	
		
		case OC_LOGON_RATE_ERROR:
			self->window->updateStatus(BUDDY_STATUS_OFFLINE, FALSE);
			u_showAlertMessage(tr("Rate Error"), tr("You have logged on too often in a short time, so the server has blocked you for some mimutes. Please try again later."), GTK_STOCK_DIALOG_WARNING);
			break;
		
		case OC_EVENT_DELETED:
		case OC_EVENT_ADDED:
			

			if (command == OC_EVENT_ADDED)
			{
				// only blink an event if there is no more recent system event
				if (self->ownerEventCount == 0 || ((basicEventInfo*)info)->isOwnerEvent)
					self->window->setTrayBlinkEvent((basicEventInfo*)info);
				
				if (((basicEventInfo*)info)->isOwnerEvent)
					self->ownerEventCount++;
				else
					self->eventCount++;
			} else
			
			{
				// blink the last recent event
				lastEv = NULL;
				self->getBuddyForLastGlobalEvent(TRUE, &lastEv);
				self->window->setTrayBlinkEvent(lastEv);
				
				if ( ((basicEventInfo*)info)->isOwnerEvent)
					self->ownerEventCount--;
				else
					self->eventCount--;
			} 
						
			if (self->ownerEventCount == 0 && self->eventCount == 0)
				self->window->setPendingEventNotify(FALSE);
			else
				self->window->setPendingEventNotify(TRUE);
			
			if (self->ownerEventCount>0)
				self->window->updateSystemInfo(self->ownerEventCount, tr("<b>System message</b>"), tr("<b>s</b>"));
			else
				self->window->updateSystemInfo(self->eventCount, tr("Message"), tr("s"));
			
			break;
		
			
		// the auto response was set for the first protocol -> set it for all protocols
		case UC_SETAUTORESPONSE:
			if (g_list_length(IO_getOwnerList())>1)
			{
				txt = NULL;
				search = IO_getOwnerList();
				while(search)
				{
					user = (IMUserDaemon*)search->data;
				
					aman = new IMAutoResponseManager(user->info, TRUE);
					user->addManager(aman);
				
					if (!txt)
						txt = aman->getCustomResponse();
					else
						aman->setCustomResponse(txt, FALSE);
				
					delete aman;
				
					search = search->next;
				}
				
				g_free(txt);
			}
			break;
						
		
		default:
			fprintf(stderr, "IMOwner::cb_ownerCallback: unknown new event (No: %d)!\n", command);
			break;
	}
		
	
	return TRUE;
}



void IMOwner::cb_menuCallback(GtkWidget* item, gint menu, gint selection, IMOwner* self)
{
	IMBuddy				*focus;
	IMOwnerDaemon		*ownerDaemon;
	contactListEntry	*focusedEntry;
	basicEventInfo		*basicEvent;
	gint	 			last;
	gboolean			cViewState;
	gchar				*txt;

	switch(menu)
	{
		// a double click on the notify box
		case MENU_NOTIFYBOX:
			// find the most recent event and open it
			if (focus = self->getBuddyForLastGlobalEvent(TRUE, &basicEvent))
				focus->openEvent(self->getWindowIDForEvent(basicEvent->type), FALSE);				
			break;
			
			
		// a double click on the user name
		case MENU_DOUBLECLICK:
			// FIXME: search is already done in cb_contactListActivated (mainWindow)
			focusedEntry = (contactListEntry*)self->window->contacts->getFocusedEntry();
			focus = (IMBuddy*)((IMUserDaemon*)focusedEntry->data)->dowry;

			basicEvent = focus->user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
			if (basicEvent)
				last = basicEvent->type;
			else
				last = -1;
		
			// open the appropriate event for the last event (or open message window)
			focus->openEvent(self->getWindowIDForEvent(last), TRUE);
			break;
		
			
		// the status menu		
		case MENU_STATUS:
			// the owner was stored in the parent menu by mainWindow::addStatusButton
			ownerDaemon = (IMOwnerDaemon*)g_object_get_data(G_OBJECT(gtk_widget_get_parent(item)), "icqnd-status-owner");
		
			// if an ownerDaemon is set the status change should only affect one protocol
			if (ownerDaemon)
			{
				if (selection != BUDDY_STATUS_INV)
					ownerDaemon->setStatus(selection);
				else
					ownerDaemon->setInvisible(!ownerDaemon->info->isInvisible);
				
				if (ownerDaemon->info->status == BUDDY_STATUS_OFFLINE && selection != BUDDY_STATUS_OFFLINE)
					self->window->updateStatusButton(ownerDaemon, BUDDY_STATUS_CONNECT, ownerDaemon->info->isInvisible);
				
				((IMBuddy*)ownerDaemon->dowry)->openEvent(UE_AUTORESPONSESET, FALSE);
			} else
				self->setStatus(selection, TRUE);
			break;
			
			
		// the user menu
		case MENU_USER:
			// get the selected entry
			focusedEntry = (contactListEntry*)self->window->contacts->getFocusedEntry();
			if (!focusedEntry)
				return;
			
			focus = (IMBuddy*)((IMUserDaemon*)focusedEntry->data)->dowry;
			
			if (!focus || !focus->user)
				return;
			
			switch(selection)
			{
				// open the last event
				case MENU_OPENEVENT:
				basicEvent = focus->user->getEvent(EO_LAST | EO_SYSTEM | EO_NORMAL);
				if (basicEvent)
					focus->openEvent(self->getWindowIDForEvent(basicEvent->type), TRUE);
				break;
				
				// remove a user
				case MENU_REMOVEUSER:
					txt = g_strdup_printf(tr("Do you really want to remove %s from your contact list?\n\nThis will delete the history for this user as well"), focus->user->info->alias);
					if (u_showYesNoDialog(tr("Delete a user?"), txt, GTK_STOCK_DIALOG_WARNING, FALSE) == GTK_RESPONSE_YES)
						((IMOwnerDaemon*)focus->user->owner)->removeUserFromList(focus->user);
					break;
				
				// features that have their own windows
				default:
					focus->openEvent((userEventType)selection, FALSE);
					break;
			}
			break;
			
			
		// the main menu elements ("System", "User")
		case MENU_SYSTEM:
			switch(selection)
			{
				// show offline users
				case MENU_SHOWOFFLINEUSERS:
					cViewState = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));	
					settings_getSettings()->setProperties(TRUE, "contactlist",
														  "showOfflineUsers", cViewState,
														  NULL);
					break;
					
				// show groups
				case MENU_SHOWGROUPS:
					cViewState = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));	
					settings_getSettings()->setProperties(TRUE, "contactlist",
														  "showGroups", cViewState,
														  NULL);		
					break;

				// show empty groups
				case MENU_SHOWEMPTYGROUPS:
					cViewState = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));	
					settings_getSettings()->setProperties(TRUE, "contactlist",
														  "showEmptyGroups", cViewState,
														  NULL);	
					break;
				
				case MENU_QUIT:
					self->window->destroyWindow();
					break;				
				
				// update all users
				case MENU_UPDATEALL:
					self->updateAllUsers();
					break;		
					
				// features that have their own window
				default:
					self->openEvent((userEventType)selection, 0);
					break;
				
			}
			break;
	}		
}



void IMOwner::cb_idActionCallback(userEventType type, gint command, gchar* id, guint pluginID, gchar* comment, gboolean toggled, IMOwner* self)
{	
	IMUserDaemon	*userDaemon;
	IMOwnerDaemon	*owner;
	IMEventManager	*man;
	GList			*owners;
	
	
	// remove the window from the list of opened events
	self->openEvents = g_list_remove(self->openEvents, self->getOpenedEvent(type));
	
	if (command != REQUEST_ACCEPTED)
		return;
	
	// find the appropriate owner
	owners = IO_getOwnerList();
	while(owners)
	{
		owner = (IMOwnerDaemon*)owners->data;
		if (owner->info->pluginID == pluginID)
			break;
		owners = owners->next;
	}
	
	if (!owners)
	{
		fprintf(stderr, "IMOwner::cb_idActionCallback: FATAL: couldn't find owner to do a user action!\n");
		return;
	}
	
	
	if (type == UE_ADDUSER)
	{
		owner->addUserToList(id, toggled);
		return;
	}
	
	
	// find our user and create one if neccessary
	userDaemon = owner->findUserByID(id);
	
	if (!userDaemon)
		// okay, the user doesn't exist yet. create a new user
		userDaemon = owner->addUserTemporally(id);

	
	// we have our user now -> execute the appropriate event
	switch(type)
	{
		case UE_AUTHORIZATIONGRANT:
			man = new IMAuthorizationManager(userDaemon->info);
			userDaemon->addManager(man);
			((IMAuthorizationManager*)man)->answerRemoteRequest(TRUE, comment);
			delete man;
			break;
			
		case UE_AUTHORIZATIONREQUEST:
			man = new IMAuthorizationManager(userDaemon->info);
			userDaemon->addManager(man);
			((IMAuthorizationManager*)man)->sendRequest(comment, NULL);
			delete man;
			break;
					
		default:
			fprintf(stderr, "IMOwner::cb_idActionCallback: unknown event to execute on %s (%d)\n", id, type);
			break;
	}
}



gboolean IMOwner::cb_massEventCallback(userEventType type, gint command, gint lastResult, gpointer info, IMOwner* self)
{
	generalEventInfo	*generalEvent;
	contactListEntry	*entry;
	IMBuddy				*buddy;
	GList				*search;
	
	
	if (command != UC_EVENT_FINISHED)
		return TRUE;
	
	generalEvent = (generalEventInfo*)info;

	entry = self->window->contacts->findEntryByID(generalEvent->basic->userID, CE_USER);
	buddy = (IMBuddy*)entry->data;
	
	search = buddy->cEntries;
	while(search)
	{
		((contactListEntry*)search->data)->setHighlighted(FALSE);
		search = search->next;
	}
	
	buddy->user->removeManagerByCallback((gpointer)self->cb_massEventCallback);
	self->updateOneUser(self->window->contacts);
	
	return TRUE;
}



gboolean IMOwner::cb_quitWindowCallback(IMOwner* self)
{
	settings	*config;
	GList		*search,
				*groups;
	gulong		num, i;
	contactListGroup	*contactsGroup;
	
	config = settings_getSettings();
	
	
	// save the opened groups
	num = 0;
	i=0;
	groups = self->window->contacts->getChildrenOfType(CE_GROUP);
	search = groups;
	while(search)
	{	
		contactsGroup = (contactListGroup*)search->data;
		if (contactsGroup->getGroupOpened())
			num = num | (1<<i);
		
		i++;
		search = search->next;
	}
	g_list_free(groups);
	
	
	config->setProperties(FALSE, "contactlist", 
						  "lastOpenedGroups", num,
						  NULL);
	
	return FALSE;
}



void IMOwner::cb_groupWindowCallback(gint command, IMOwner* self)
{
	GList	*searchOwners,
			*search;
	IMUserDaemon* user;
	
	// updating was done -> rebuild EVERYTHING!
	if (command == 1)
	{	
		searchOwners = IO_getOwnerList();
		
		while(searchOwners)
		{
			search = ((IMOwnerDaemon*)searchOwners->data)->users;
			while(search)
			{
				user = (IMUserDaemon*)search->data;
				// FIXME: don't update that much just for the groups
				user->editUserInformation(TRUE, USER_UPDATE_BASIC);
				search = search->next;
			}
			
			searchOwners = searchOwners->next;
		}
		
		self->rebuildContactList(FALSE);
	}	
}



gboolean IMOwner::cb_generalCallback(userEventType type, gint command, gint lastResult, gpointer data, IMOwner* self)
{
	IMOwnerDaemon	*owner;
	IMBuddy			*newProto;
	GList			*ownerSearch,
					*usersSearch;
	gchar			*charset;
	
	switch(command)
	{	
		case EC_FROMMANAGER:
			self->openEvent((userEventType)lastResult, 0);
			break;		
		
		// a new owner was added / removed
		case PC_NEWOWNER:
		case PC_REMOVEOWNER:
			owner = (IMOwnerDaemon*)data;
			if (command == PC_NEWOWNER)
			{
				// wrap a new IMBuddy around that owner
				
				newProto = new IMBuddy((IMUserDaemon*)owner);
				newProto->user->setCallback((IMEventCallback)self->cb_ownerCallback, self);
				newProto->user->dowry = newProto;
				
				settings_getSettings()->getProperties("conversations",
										  			  "standardEncoding", &charset,
										 			  NULL);				
				newProto->user->setCharset(charset, FALSE);

				owner->loadContacts(CLT_NORMAL | CLT_VISIBLE);
		
				owner->dispatchPendingEvents();
		
				// dispatch events of all new users
				usersSearch = owner->users;
				while(usersSearch)
				{
					((IMUserDaemon*)usersSearch->data)->dispatchPendingEvents();
					usersSearch = usersSearch->next;
				}
				
				// if there a now 2 owners active that first one has to be displayed
				if (g_list_length(IO_getOwnerList())==2)
					self->window->addStatusButton((IMOwnerDaemon*)IO_getOwnerList()->data);
				
				if (g_list_length(IO_getOwnerList())>1)
					self->window->addStatusButton((IMOwnerDaemon*)data);
				
			} else
			{					
				// delete all children
				usersSearch = owner->users;
				while(usersSearch)
				{
					if (((IMUserDaemon*)usersSearch->data)->dowry)
						delete (IMBuddy*)((IMUserDaemon*)usersSearch->data)->dowry;
					
					usersSearch = usersSearch->next;
				}
				
				delete (IMBuddy*)owner->dowry;
				
				self->window->removeStatusButton((IMOwnerDaemon*)data);
				
				// if there is only one owner active don't show any buttons
				if (g_list_length(IO_getOwnerList()) == 1)
					self->window->removeStatusButton((IMOwnerDaemon*)IO_getOwnerList()->data);
			}
	
			// reload all groups
			IO_getGroupManager()->checkForNewGroups();
	
			ownerSearch = IO_getOwnerList();
			while(ownerSearch)
			{
				usersSearch = ((IMOwnerDaemon*)ownerSearch->data)->users;
				while(usersSearch)
				{
					((IMUserDaemon*)usersSearch->data)->editUserInformation(TRUE, USER_UPDATE_BASIC);
					usersSearch = usersSearch->next;
				}
				ownerSearch = ownerSearch->next;
			}

			// reload the contact list
			self->rebuildContactList(FALSE);
	
			// rebuild the window title
			self->setWindowTitle();
	
			
			// enable / disable the buttons
			self->window->setLockUserMenus(IO_getOwnerList() == NULL);
			
			break;

			
	}	

	return TRUE;	
}




userEventType IMOwner::getWindowIDForEvent(gint eventType)
{
	switch(eventType)
	{
		case -1:
		case USER_EVENT_MESSAGE:
			return UE_MESSAGE;
				
		case USER_EVENT_FILE:
			return UE_FILETRANSFER;
				
		case USER_EVENT_CHAT:
			return UE_CHAT;
				
		case USER_EVENT_URL:
			return UE_URL;
				
		case USER_EVENT_AUTH_REQUEST:
			return UE_AUTHORIZATION;
		
		case USER_EVENT_AUTH_ACCEPT:
			return UE_AUTHORIZATIONGRANT;
		
		case USER_EVENT_AUTH_REFUSE:
			return UE_AUTHORIZATIONREFUSE;
		
		case USER_EVENT_ADDED_TO_LIST:
			return UE_ADDEDTOLIST;
	
		default:
			return UE_MESSAGE;
	}		
}
