/***************************************************************************
 *            mainwindow.cpp
 *
 *  Tue Jan 25 22:22:32 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "mainwindow.h"

mainWindow::mainWindow(mwMenuClickFunction mcf, GCallback quitfunc, gpointer cbdata)
{
	GtkWidget	*frame3,
				*hbox1,
				*nhbox,
				*vbox,
				*viewPort,
				*tmp;
	gulong		width, height, posX, posY;
	settings	*config;
	gboolean	showTray;
	GList		*search;

	
	config = settings_getSettings();
	
	
	/* set callback data */
	menuCallBack = mcf;
	menuCallBackData = cbdata;
	
	
	/* set up the state icons */
	tmp = gtk_button_new();
	stateIcons = NULL;
	stateIcons = g_list_append(stateIcons, gtk_widget_render_icon(tmp, 
																  ICON_TYPING1,
																  GTK_ICON_SIZE_SMALL_TOOLBAR,
																  NULL));
	gtk_widget_destroy(tmp);																  
	
	
	config->getProperties("contactlist",
						  "showTooltips", &showTooltips,
						  "tooltipsMask", &tooltipsMask,
						  NULL);
	
	
	
	/* Create the contact list */
	contacts = new contactList(stateIcons);
	// load the last view mode
	updateContactListViewMode();
	// normal button press
	g_signal_connect(contacts->tv, "button_press_event", G_CALLBACK(this->cb_contactListClicked), this);
	// double click or RETURN
	g_signal_connect(contacts->tv, "row-activated", G_CALLBACK(this->cb_contactListActivated), this);
	
	// add tooltip events
	if (showTooltips)
	{
		g_signal_connect_swapped(contacts->tv, "leave-notify-event", G_CALLBACK(this->cb_tooltipLeaveNotify), this);
		g_signal_connect(contacts->tv, "motion-notify-event", G_CALLBACK(this->cb_tooltipMoveNotify), this);
	}
		
	// create all the menus
	createMenus();


	// ** create system notify box **
	notifyText = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(notifyText), 0.0f, 0.5f);
	gtk_widget_set_size_request(notifyText, 100, -1);
	
	// add a viewport so the text can be truncated
	viewPort = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewPort), GTK_SHADOW_NONE);
	gtk_container_add(GTK_CONTAINER(viewPort), notifyText);
	gtk_widget_add_events(viewPort, GDK_BUTTON1_MOTION_MASK);
	g_signal_connect(viewPort, "button_press_event", G_CALLBACK(this->cb_notifyBoxClicked), this);

	nhbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(nhbox), viewPort, TRUE, TRUE, 0);
	
	frame3 = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame3), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame3), nhbox);
	

	

	// create the status bar
	statusButtonText = gtk_label_new("");
	statusButtonIcon = gtk_image_new();
	hbox1 = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox1), statusButtonIcon, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), statusButtonText, FALSE, TRUE, 0);
	statusButton = gtk_button_new();
	gtk_button_set_relief(GTK_BUTTON(statusButton), GTK_RELIEF_NONE);
	gtk_container_add(GTK_CONTAINER(statusButton), hbox1);
	g_signal_connect(G_OBJECT(statusButton), "clicked", G_CALLBACK(this->cb_menuClicked), statusMenu);


	statusBox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(statusBox), statusButton, TRUE, TRUE, 0);

	
	
	// ** create main window **
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), mainMenu, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), contacts->listWidget, TRUE , TRUE, 3);
	gtk_box_pack_start(GTK_BOX(vbox), frame3, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), statusBox, FALSE, TRUE, 0);
		
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	// gtk_window_set_title(GTK_WINDOW(window), title);


	g_signal_connect_swapped(G_OBJECT(window), "delete-event", G_CALLBACK(quitfunc), cbdata);
	g_signal_connect_swapped(G_OBJECT(window), "delete-event", G_CALLBACK(this->cb_destroyWindow), this);
	g_signal_connect_swapped(G_OBJECT(window), "visibility-notify-event", G_CALLBACK(this->cb_visibilityChange), this);
	/* g_signal_connect(G_OBJECT(wnd), "configure-event", G_CALLBACK(MWpositionChange), this); */
	gtk_widget_add_events(window, GDK_VISIBILITY_NOTIFY_MASK | GDK_STRUCTURE_MASK);
	
	
	config->getProperties("mainwindow",
						  "width", &width,
						  "height", &height,
						  "positionX", &posX,
						  "positionY", &posY,
						  NULL);
	
	gtk_window_set_default_size(GTK_WINDOW(window), width, height);
	gtk_window_move(GTK_WINDOW(window), posX, posY);
										
	gtk_container_add(GTK_CONTAINER(window), vbox);
	
	
	
	// show window!
	gtk_widget_show_all(window);
	
	
	config->getProperties("appearance",
						  "showTrayIcon", &showTray,
						  NULL);
						  	
	
	// set up tray icon
	if (showTray)
	{
		tray = new trayIcon();
		if (!tray->installTrayIcon(gtk_image_new()))
		{
			delete tray;
			tray = NULL;
		} else
			g_signal_connect_swapped(tray->eventWidget, "button_press_event", G_CALLBACK(this->cb_trayClicked), this);
	}
	else
		tray = NULL;
	
	// connect to the settings callback
	settings_getSettings()->installListener("contactlist", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);	
	settings_getSettings()->installListener("appearance", (settingChangedFunc)this->cb_settingsChangedCallback, this, FALSE);
	
	trayConnectSource = 0;
	trayBlinkSource = 0;
	eventPixbuf = NULL;
	title = NULL;
	statusButtons = NULL;
	tooltipWindow = NULL;
	tooltipSource = 0;
	lastTooltipEntry = NULL;
	
	// add all owner buttons
	search = IO_getOwnerList();
	if (g_list_length(search)>1)
		while(search)
		{
			addStatusButton((IMOwnerDaemon*)search->data);
			search = search->next;
		}
			
	updateStatus(BUDDY_STATUS_OFFLINE, FALSE);
}




mainWindow::~mainWindow()
{
	if (tray)
	{
		tray->removeTrayIcon();
		delete tray;
	}
}



void mainWindow::setTitle(gchar* _title)
{
	if (title)
		g_free(title);
	
	title = g_strdup(_title);
	gtk_window_set_title(GTK_WINDOW(window), title);	
}


void mainWindow::setPendingEventNotify(gboolean state)
{
	GString	*newTitle;
	
	if (!title)
		return;
	
	if (state)
		newTitle = g_string_new("*");
	else
		newTitle = g_string_new("");
	
	newTitle = g_string_append(newTitle, title);
	
	gtk_window_set_title(GTK_WINDOW(window), newTitle->str);
	
	g_string_free(newTitle, TRUE);	
}



void mainWindow::updateSystemInfo(gint noe, gchar* evname, gchar* appendix)
{
	gchar	*tmp, *tmp2;
	
	if (noe == 0)
		gtk_label_set_markup(GTK_LABEL(notifyText), "");

	else
	{
		if (noe>1)
			tmp2 = appendix;
		else
			tmp2 = "";
		tmp = g_strdup_printf("%d %s%s", noe, evname, tmp2);
		gtk_label_set_markup(GTK_LABEL(notifyText), tmp);
		g_free(tmp);
	}
}



void mainWindow::updateStatus(guint status, gboolean isInvisible)
{
	gchar		*sName;

	if (status == BUDDY_STATUS_CONNECT)
	{
		startTrayAnimation();
		gtk_label_set_label(GTK_LABEL(statusButtonText), uu_getStatusDescription(status));
		return;
	}
	else
	if (trayConnectSource)
		stopTrayAnimation();
	
	
	lastStatusPixbuf = i_getIcons()->getStatusPixbuf("icq", status, isInvisible);
	
	
	if (isInvisible)
		sName = g_strdup_printf("(%s)", uu_getStatusDescription(status));
	else
		sName = g_strdup(uu_getStatusDescription(status));
	
	gtk_image_set_from_pixbuf(GTK_IMAGE(statusButtonIcon), lastStatusPixbuf);
	gtk_label_set_label(GTK_LABEL(statusButtonText), sName);
	if (tray)
		gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
	
	lastStatus = status;
	lastInvisible = isInvisible;
	
	g_free(sName);  
}


GtkMenu *mainWindow::addStatusButton(IMOwnerDaemon *owner)
{
	gint i;
	iconManager			*iconMan;
	gchar				*protoName;
	statusButtonType	*newButton;
	GtkMenu				*menu;
	GtkWidget			*hbox,
						*label,
						*img;
	GList				*children;
	
	u_menuItem statusMenuList[]=
	{
		{UMENU_NORMAL	, BUDDY_STATUS_ONLINE	, NULL		, tr_NOOP("Online")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_AWAY		, NULL		, tr_NOOP("Away")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_NA		, NULL		, tr_NOOP("Not Available")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_OCC		, NULL		, tr_NOOP("Busy")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_DND		, NULL		, tr_NOOP("Do Not Disturb")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_FFC		, NULL		, tr_NOOP("Free For Chat")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_OFFLINE	, NULL		, tr_NOOP("Offline")		, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, BUDDY_STATUS_INV		, NULL		, tr_NOOP("Invisible")		, NULL, TRUE, FALSE, 0, &menu_statusInvisible}
	};
	
	iconMan = i_getIcons();	
	
	// update the menu list icons
	if (owner)
	{
		if (owner->protocol->pluginID == LICQ_PPID)
			protoName = g_strdup("icq");		
		else
			protoName = g_ascii_strdown(owner->protocol->licqName, -1);
	} else
		protoName = g_strdup("icq");
		
	// add the right icons
	for (i=0; i<9; i++)
		if (statusMenuList[i].type != UMENU_SEPARATOR)
			statusMenuList[i].stockIcon = g_strdup_printf("icqnd-status-%s-%d", protoName, statusMenuList[i].id);
				
	menu = u_createMenuList(statusMenuList, 9, (GtkSignalFunc)this->cb_menuItemClicked, MENU_STATUS, this, TRUE);	


	// FIXME: this is not good, but how to separate the menus otherwise?
	g_object_set_data(G_OBJECT(menu), "icqnd-status-owner", owner);
		
	if (owner)
	{
		// create a new struc
		newButton = (statusButtonType*)g_malloc0(sizeof(statusButtonType));
	
		newButton->owner 			= owner;
		newButton->connectTimeout 	= 0;
		newButton->status 			= owner->info->status;
		newButton->statusMenu 		= menu;	
		newButton->statusImage 		= gtk_image_new_from_pixbuf(iconMan->getStatusPixbuf(protoName, 
																						 owner->info->status, 
																						 owner->info->isInvisible));
		
		
		// create the button
		newButton->button = gtk_button_new();
		g_signal_connect(newButton->button, "clicked", G_CALLBACK(this->cb_menuClicked), newButton->statusMenu);
		gtk_container_add(GTK_CONTAINER(newButton->button), newButton->statusImage);
		
		
		
		// create the menu item for the main menu
		// FIXME: change this as soon there is a multi user support for a protocol
		label = gtk_label_new_with_mnemonic(owner->protocol->name);
		gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
			
		img = gtk_image_new_from_stock(statusMenuList[0].stockIcon, GTK_ICON_SIZE_MENU);
		gtk_widget_set_size_request(img, 16, 16);
	
		hbox = gtk_hbox_new(FALSE, 2);
		gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 2);
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
			
		newButton->mainMenuEntry = gtk_menu_item_new();
		gtk_container_add(GTK_CONTAINER(newButton->mainMenuEntry), hbox);
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(newButton->mainMenuEntry), GTK_WIDGET(newButton->statusMenu));
		
		// check whether there has to be inserted a separator
		// (we do this by checking the count of menu items)
		children = gtk_container_get_children(GTK_CONTAINER(statusMenu));
		if (g_list_length(children) == 9)
			gtk_menu_shell_prepend(GTK_MENU_SHELL(statusMenu), gtk_separator_menu_item_new());
		g_list_free(children);
		
		gtk_menu_shell_prepend(GTK_MENU_SHELL(statusMenu), newButton->mainMenuEntry);
		gtk_widget_show_all(GTK_WIDGET(statusMenu));		
		
		
		
		gtk_box_pack_end(GTK_BOX(statusBox), newButton->button, FALSE, FALSE, 0);
		gtk_widget_show_all(statusBox);
			
		statusButtons = g_list_append(statusButtons, newButton);	
	} 
	

	// free our created stock id's
	for (i=0; i<9; i++)
		if (statusMenuList[i].type != UMENU_SEPARATOR)
			g_free(statusMenuList[i].stockIcon);	
	
	g_free(protoName);

	return menu;	
}



void mainWindow::removeStatusButton(IMOwnerDaemon *owner)
{
	GList				*search,
						*children;
	statusButtonType	*but;
	
	search = statusButtons;
	
	while(search)
	{
		but = (statusButtonType*)search->data;
		if (but->owner == owner)
		{
			gtk_container_remove(GTK_CONTAINER(statusBox), but->button);
			gtk_widget_destroy(GTK_WIDGET(but->statusMenu));	
			gtk_widget_destroy(but->mainMenuEntry);
			// check whether the separator has to be removed
			children = gtk_container_get_children(GTK_CONTAINER(statusMenu));
			if (g_list_length(children) == 10)
				gtk_widget_destroy(GTK_WIDGET(children->data));
			g_list_free(children);
			
			statusButtons = g_list_remove(statusButtons, but);
			
			g_free(but);
			return;
		}
		
		search = search->next;
	}
}



void mainWindow::updateStatusButton(IMOwnerDaemon *owner, glong status, gboolean invisible)
{
	GList				*search;
	statusButtonType	*but;
	gchar				*protoName;
	
	
	search = statusButtons;
	while(search)
	{
		but = (statusButtonType*)search->data;
		
		if (but->owner == owner)
			break;
		
		search = search->next;
	}
	
	if (!search)
		return;
		
	if (status == BUDDY_STATUS_CONNECT && !but->connectTimeout)
	{
		but->connectTimeout = g_timeout_add(300, (GSourceFunc)this->cb_statusButtonBlinking, but);
		return;
	}
	else
		if (but->connectTimeout)
		{
			g_source_remove(but->connectTimeout);
			but->connectTimeout = 0;
		}
		
		
	if (owner->info->pluginID == LICQ_PPID)
		protoName = "icq";
	else
		protoName = owner->protocol->licqName;
				
	gtk_image_set_from_pixbuf(GTK_IMAGE(but->statusImage), 
							  i_getIcons()->getStatusPixbuf(protoName,
															status,
															invisible));
	but->status = status;

}



void mainWindow::setLockUserMenus(gboolean lock)
{
	gtk_widget_set_sensitive(menu_users, !lock);
	gtk_widget_set_sensitive(menu_status, !lock);
	gtk_widget_set_sensitive(menu_editPersonalInfo, !lock);
	gtk_widget_set_sensitive(menu_reloadContacts, !lock);
	gtk_widget_set_sensitive(statusButton, !lock);	
}



void mainWindow::destroyWindow()
{
	cb_destroyWindow(this);
	// gtk_widget_destroy(window);
}



void mainWindow::trayClicked(GdkEventType type, guchar button)
{
/*	
	// a single click
	if (type == GDK_BUTTON_PRESS && button == 3)
			// right button was clicked -> show menu
			gtk_menu_popup(GTK_MENU(createMainMenu()), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
*/
	if (type == GDK_2BUTTON_PRESS && button == 1)
	{
		if (trayBlinkSource!=0)
			menuCallBack(NULL, MENU_NOTIFYBOX, 0, menuCallBackData);	
		else
		{
			// left button was clicked -> show / hide window
			if(GTK_WIDGET_VISIBLE(window))
			{
				// show window when it's iconified or obscured
				if ((gdk_window_get_state(window->window) & GDK_WINDOW_STATE_ICONIFIED) || isObscured)
					gtk_window_present(GTK_WINDOW(window));
				else
					gtk_window_iconify(GTK_WINDOW(window));
			}
			else
				gtk_window_present(GTK_WINDOW(window));					
		}
	}
}


void mainWindow::startTrayAnimation()
{
	if (!tray || trayConnectSource)
		return;
	
	trayIter = 0;
	trayConnectSource = g_timeout_add(150, (GSourceFunc)this->cb_trayAnimation, this);
}


void mainWindow::stopTrayAnimation()
{
	if (tray && trayConnectSource != 0)
	{
		g_source_remove(trayConnectSource);
		trayConnectSource = 0;
		// set the original pixbuf
		gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
		
	}
}


gboolean mainWindow::isTrayAnimation()
{
	return trayConnectSource != 0;
}


void mainWindow::setTrayBlinkEvent(basicEventInfo *ev)
{		
	if ((!ev && !trayBlinkSource) || !tray)
		return;
	
	if (!ev)
	{
		if (trayBlinkSource)
		{
			g_source_remove(trayBlinkSource);
			gtk_image_set_from_pixbuf(GTK_IMAGE(tray->iconWidget), lastStatusPixbuf);
			trayBlinkSource = 0;
		}
	} else
	{
		eventPixbuf = i_getIcons()->getEventPixbuf(ev->type);		
		
		if (!trayBlinkSource)
			trayBlinkSource = g_timeout_add(300, (GSourceFunc)this->cb_trayEventBlinking, this);
		
	}
}



void mainWindow::createMenus()
{
	GtkMenu	*viewMenu,
			*debuggingMenu,
			*usersMenu,
			*systemMenu;
	GtkWidget	*mainItem;
	gboolean	showGroups,
				showEmptyGroups,
				showOfflineUsers;

	
	// create the user menu
	// ====================
	u_menuItem buddyMenuList[]=
	{
		{UMENU_HEADLINE	, MENU_NONE				 , NULL, "", NULL, FALSE, FALSE, 0, &menu_userMenuLabel},
		{UMENU_NORMAL	, MENU_OPENEVENT		 , ICON_MESSAGE		, tr_NOOP("Open Event")			  , NULL, FALSE, FALSE, 0, &menu_openUserEvent},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_MESSAGE			 , ICON_MESSAGE		, tr_NOOP("Send Message")			  , NULL, TRUE, FALSE, 0, &menu_sendMessage},
		{UMENU_NORMAL	, UE_URL				 , ICON_URL			, tr_NOOP("Send URL")				  , NULL, TRUE, FALSE, 0, &menu_sendURL},
		{UMENU_NORMAL	, UE_FILETRANSFERREQUEST , ICON_FILE		, tr_NOOP("Send Files")				  , NULL, TRUE, FALSE, 0, &menu_sendFiles},
		{UMENU_NORMAL	, UE_CHATREQUEST		 , ICON_CHAT		, tr_NOOP("Request A Chat")			  , NULL, TRUE, FALSE, 0, &menu_sendChat},
		{UMENU_NORMAL	, UE_CONTACTSREQUEST	 , NULL				, tr_NOOP("Send Contacts")			  , NULL, FALSE, FALSE, 0, &menu_sendContacts},
		{UMENU_NORMAL	, UE_AUTHORIZATIONREQUEST, NULL				, tr_NOOP("Ask For Authorization")	  , NULL, TRUE, FALSE, 0, &menu_sendAuthReq},
		{UMENU_NORMAL	, UE_AUTHORIZATION		 , NULL				, tr_NOOP("Send Authorization")		  , NULL, TRUE, FALSE, 0, &menu_sendAuth},
		{UMENU_NORMAL	, UE_ENCRYPTION			 , NULL				, tr_NOOP("Encrypt Connection")		  , NULL, TRUE, FALSE, 0, &menu_sendEncryption},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTORESPONSEGET	 , NULL				, tr_NOOP("Check Auto Response")	  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTORESPONSESET	 , NULL				, tr_NOOP("Set Special Auto Response"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_EDITUSERMODES		 , NULL				, tr_NOOP("Edit Groups / Modes")	  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_USERINFO			 , ICON_INFO		, tr_NOOP("Users Information")		  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_HISTORY			 , ICON_HISTORY		, tr_NOOP("History")				  , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE				 , NULL				, NULL, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_REMOVEUSER		 , GTK_STOCK_CANCEL	, tr_NOOP("Remove User")			  , NULL, TRUE, FALSE, 0, NULL}
	};
	
	userMenu = u_createMenuList(buddyMenuList, 20, (GtkSignalFunc)this->cb_menuItemClicked, MENU_USER, this, TRUE);
	
	

	// create the main menu
	// ====================	
	
	// Menus of the second level. These menus will be sub menus of the first level menus
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  "showOfflineUsers", &showOfflineUsers,
										  NULL);

	u_menuItem viewMenuList[5]=
	{
		{UMENU_CHECK	, MENU_SHOWGROUPS		, NULL, tr_NOOP("Show Groups")			, NULL, TRUE		, showGroups	  , 0, &menu_showGroups},
		{UMENU_CHECK	, MENU_SHOWEMPTYGROUPS	, NULL, tr_NOOP("Show Empty Groups")	, NULL, showGroups	, showEmptyGroups , 0, &menu_showEmptyGroups},
		{UMENU_CHECK	, MENU_SHOWOFFLINEUSERS	, NULL, tr_NOOP("Show Offline Users")	, NULL, TRUE		, showOfflineUsers, 0, &menu_showOfflineUsers},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_RELOADLIST		, NULL, tr_NOOP("Reload Contact List")	, NULL, FALSE, FALSE, 0, &menu_reloadContacts}
	};		
		
	
	u_menuItem debuggingMenuList[2]=
	{
		{UMENU_NORMAL, UE_LOGMESSAGES	, GTK_STOCK_NETWORK, tr_NOOP("Network Window"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, MENU_NONE	, NULL, tr_NOOP("Debug Level")	, NULL, FALSE, FALSE, 0, NULL}	
	};
	
	
	// create the menus
	viewMenu = u_createMenuList(viewMenuList, 5, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, FALSE);
	debuggingMenu = u_createMenuList(debuggingMenuList, 2, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	statusMenu = addStatusButton(NULL);
	
	// Menus of the first level
	u_menuItem usersMenuList[13]=
	{
		{UMENU_NORMAL	, MENU_OPENALL			 , ICON_MESSAGE, tr_NOOP("Open All Messages")	 , NULL, FALSE, FALSE, 0, &menu_openAllEvents},
		{UMENU_SEPARATOR, MENU_NONE				 , NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_WHITEPAGES			 , GTK_STOCK_FIND, tr_NOOP("Find Users")			 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_ADDUSER			 , GTK_STOCK_ADD, tr_NOOP("Add User")			 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATIONGRANT	 , NULL, tr_NOOP("Authorize User")		 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_AUTHORIZATIONREQUEST, NULL, tr_NOOP("Request Authorization"), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE				 , ICON_CHAT, tr_NOOP("Set Random Chat Group"), NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE				 , ICON_CHAT, tr_NOOP("Start Random Chat")	 , NULL, FALSE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_EDITGROUPS			 , GTK_STOCK_INDEX, tr_NOOP("Edit Groups ...")		 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_UPDATEALL		 , NULL, tr_NOOP("Update All Users")	 , NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_CHECKINVISIBLEALL , NULL, tr_NOOP("Check Invisible Users"), NULL, FALSE, FALSE, 0, NULL}		
	};
	
	
	u_menuItem systemMenuList[12]=
	{
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Status")		, statusMenu	, TRUE, FALSE, 0, &menu_status},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("View")			, viewMenu		, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Debugging")	, debuggingMenu	, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_MANAGEOWNERS, NULL, tr_NOOP("Owner Manager..."), NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL	, UE_OPTIONS	, GTK_STOCK_PREFERENCES, tr_NOOP("Options...")	, NULL			, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Statistics")	, NULL			, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("Tips")			, NULL			, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_NONE	, NULL, tr_NOOP("About")		, NULL			, FALSE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, MENU_NONE, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL	, MENU_QUIT		, GTK_STOCK_QUIT, tr_NOOP("Exit")			, NULL			, TRUE, FALSE, 0, NULL}
	};

	
	usersMenu = u_createMenuList(usersMenuList, 13, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	systemMenu = u_createMenuList(systemMenuList, 12, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this, TRUE);
	
	
	// create the menu bar
	mainMenu = gtk_menu_bar_new();
	
	mainItem = gtk_menu_item_new_with_mnemonic("_System");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(mainItem), GTK_WIDGET(systemMenu));
	gtk_menu_shell_append(GTK_MENU_SHELL(mainMenu), mainItem);
	
	menu_users = gtk_menu_item_new_with_mnemonic("_Users");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_users), GTK_WIDGET(usersMenu));
	gtk_menu_shell_append(GTK_MENU_SHELL(mainMenu), menu_users);
}


/*
GtkWidget* mainWindow::createMainMenu()
{
	GtkMenu	*systemMenu,
			*userMenu,
			*helpMenu,
			*_statusMenu;
	gboolean showGroups,
			 showEmptyGroups,
			 showOfflineUsers; 
	GtkWidget	*menuBar;
	
	if (mainMenu)
		return mainMenu;
	
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  "showOfflineUsers", &showOfflineUsers,
										  NULL);	
		
	
	// order of a menu item
	// menu type (see utilities.h), radio group id, title, icon, id, enabled?, selected?, target (or NULL for none), submenu
	

	// menus of the 2nd level
	u_menuItem systemMenuList[5]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("User information")			, ICON_INFO		, UE_USERINFO		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Security / Password")  		, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Network window") 	  		, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Debugging")		  			, NULL			, UE_UNDEFINED		, TRUE, TRUE, NULL, NULL}
	};

	u_menuItem userMenuList[12]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Find user") 				, GTK_STOCK_FIND, USER_FIND			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Add user")	  			, GTK_STOCK_ADD	, USER_ADD			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorize user")			, NULL		  	, USER_AUTHORIZE	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Request authorization")	, NULL		  	, USER_GETAUTH		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL}	,
		{UMENU_NORMAL, 0, tr_NOOP("Open all messages")		, ICON_MESSAGE	, USER_OPENALL		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Random chat")  			, ICON_CHAT	  	, RANDOM_USERCHAT	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Random chat group") 		, ICON_CHAT	  	, RANDOM_CHATGROUP	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Update all users")		, NULL		  	, USER_UPDATEALL	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Register user")   		, NULL		  	, USER_REGISTER		, TRUE, TRUE, NULL, NULL}
	};

	
	
	u_menuItem groupMenuList[4]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Edit groups")	  		, NULL		, GROUP_EDIT		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_CHECK, 0, tr_NOOP("Show Groups")				, NULL			, GROUP_SHOW, TRUE, showGroups, NULL, NULL},
		{UMENU_CHECK, 0, tr_NOOP("Show empty groups")		, NULL			, GROUP_SHOWEMPTY, showGroups, showEmptyGroups, &showEmptyGroupsMenu, NULL}				
	};

	u_menuItem helpMenuList[3]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Tips")	  				, NULL		  	, HELP_TIPS			, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("About")		  			, NULL		  	, HELP_ABOUT		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Statistics")	  			, NULL		  	, HELP_STATS		, TRUE, TRUE, NULL, NULL}
	};

	// create menus
	systemMenu  = u_createMenuList(systemMenuList, 5, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SYSTEM, this);
	userMenu    = u_createMenuList(userMenuList,  12, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	groupMenu   = u_createMenuList(groupMenuList,  4, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	helpMenu    = u_createMenuList(helpMenuList,   3, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	_statusMenu = createStatusMenu();
	
	
	// create main menu
	u_menuItem mainMenuList[14]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("System")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, systemMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Users") 				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, userMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Groups")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, groupMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Status")				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, _statusMenu},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Mini mode") 			, NULL		  	, MAIN_MINIVIEW		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Reload contact list"), NULL		  	, MAIN_ACTUALIZE	, TRUE, TRUE, NULL, NULL},
		{UMENU_CHECK,  0, tr_NOOP("Show offline users")	, NULL		  	, MAIN_SHOWOFFLINE	, TRUE, showOfflineUsers, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Options ...")		, NULL		  	, MAIN_OPTIONS		, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Plugins ...") 		, NULL		  	, MAIN_PLUGINS		, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL ,NULL ,0 , FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Save settings") 		, NULL		  	, MAIN_SAVESETTINGS	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Help") 				, NULL		  	, MAIN_NONE			, TRUE, TRUE, NULL, helpMenu},
		{UMENU_NORMAL, 0, tr_NOOP("Exit")	  			, NULL		  	, MAIN_EXIT			, TRUE, TRUE, NULL, NULL}
	};
	
	mainMenu = u_createMenuList(mainMenuList, 14, (GtkSignalFunc)this->cb_menuItemClicked, MENU_MAIN, this);
	
	return mainMenu;
	
} 




GtkMenu* mainWindow::createStatusMenu()
{
	if (statusMenu)
		return statusMenu;
	
	u_menuItem statusMenuList[]=
	{
		{UMENU_NORMAL, BUDDY_STATUS_ONLINE	, ICON_ONLINE		, tr_NOOP("Online")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_AWAY	, ICON_AWAY			, tr_NOOP("Away")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_NA		, ICON_NA			, tr_NOOP("Not Available")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_OCC		, ICON_OCCUPIED		, tr_NOOP("Busy")			, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_DND		, ICON_DND			, tr_NOOP("Do Not Disturb")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_FFC		, ICON_CHAT			, tr_NOOP("Free For Chat")	, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_OFFLINE	, ICON_OFFLINE		, tr_NOOP("Offline")		, NULL, TRUE, FALSE, 0, NULL},
		{UMENU_SEPARATOR, UE_UNDEFINED, NULL, NULL, NULL, FALSE, FALSE, 0, NULL},
		{UMENU_NORMAL, BUDDY_STATUS_INV		, ICON_INVISIBLE	, tr_NOOP("Invisible")		, NULL, TRUE, FALSE, 0, NULL}
	};
	

	statusMenu = u_createMenuList(statusMenuList, 9, (GtkSignalFunc)this->cb_menuItemClicked, MENU_STATUS, this);	

	return statusMenu;
}



GtkMenu* mainWindow::createUserMenu()
{
	GtkMenu *sendMenu;
	if (userMenu)
		return userMenu;
	
	u_menuItem userMenuSendList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Message")				, ICON_MESSAGE	, UE_MESSAGE			 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("URL")					, ICON_URL		, UE_URL				 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Chat request")			, ICON_CHAT		, UE_CHATREQUEST		 , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Files")					, ICON_FILE		, UE_FILETRANSFERREQUEST , TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Contacts")				, ICON_INFO		, UE_CONTACTSREQUEST	 , TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE	, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorization request")	, ICON_INFO		, UE_AUTHORIZATIONREQUEST, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Authorization")			, ICON_INFO		, UE_AUTHORIZATIONGRANT	 , TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Encrypt connection")		, ICON_INFO		, UE_ENCRYPTION			 , TRUE, TRUE, NULL, NULL}
	};
	
	sendMenu = u_createMenuList(userMenuSendList, 10, (GtkSignalFunc)this->cb_menuItemClicked, MENU_SEND, this);
	
	
	
	u_menuItem userMenuList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Show messages")			, NULL			, POPUP_SHOWEVENTS, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Send event")				, NULL			, POPUP_SEND	, TRUE, TRUE, NULL, sendMenu},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE	, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Check auto response")	, NULL			, POPUP_ARESPONSEGET, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Special auto response")	, NULL			, POPUP_ARESPONSESET, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("User information")		, ICON_INFO		, POPUP_INFO	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("History")				, ICON_HISTORY	, POPUP_HISTORY	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Remove user")			, ICON_BLANK	, POPUP_DELETE	, TRUE, TRUE, NULL, NULL}
	};
	
	
	// userMenu = u_createMenuList(userMenuList, 8, (GtkSignalFunc)this->cb_menuItemClicked, MENU_POPUP, this);

	return userMenu;
}




GtkMenu* mainWindow::createGroupMenu()
{
	gboolean showGroups,
			 showEmptyGroups;
	
	if (groupMenu)
		return groupMenu;
	
	settings_getSettings()->getProperties("contactlist",
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  NULL);
	
	
	u_menuItem groupMenuList[]=
	{
		{UMENU_NORMAL, 0, tr_NOOP("Rename group")		, ICON_MESSAGE	, 0	, TRUE, TRUE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Edit group")			, ICON_URL		, 0	, TRUE, TRUE, NULL, NULL},
		{UMENU_SEPARATOR, 0, NULL, NULL, 0, FALSE, FALSE, NULL, NULL},
		{UMENU_NORMAL, 0, tr_NOOP("Remove group")		, ICON_FILE		, 0	, TRUE, TRUE, NULL, NULL}
	};
	
		
	groupMenu = u_createMenuList(groupMenuList, 4, (GtkSignalFunc)this->cb_menuItemClicked, MENU_POPUP, this);

	return groupMenu;	
	
}
*/




gboolean mainWindow::updateContactListViewMode()
{
	gboolean showOfflineUsers,
			 showGroups,
			 showEmptyGroups;
	
	gint	 newView;
	GList	 *groups,
			 *search;
	contactListGroup	*contactsGroup;
	groupInfo	*group;
	
	// save old group settings
	groups = contacts->getChildrenOfType(CE_GROUP);
	search = groups;
	while(search)
	{
		contactsGroup = (contactListGroup*)search->data;
		group = (groupInfo*)contactsGroup->data;
		group->opened = contactsGroup->getGroupOpened();
		
		search = search->next;
	}
	g_list_free(groups);
	
	settings_getSettings()->getProperties("contactlist", 
							  			  "showOfflineUsers", &showOfflineUsers,				
										  "showGroups", &showGroups,
										  "showEmptyGroups", &showEmptyGroups,
										  NULL);
								
	newView = 0;
	if (showGroups)
		newView = newView | CVIEW_GROUPS;
	if (!showEmptyGroups)
		newView = newView | CVIEW_NOEMPTYGROUPS;
	if (!showOfflineUsers)
		newView = newView | CVIEW_HIDEOFFLINE;
		
	contacts->deleteEntries();
	contacts->showEntries(newView);	

	// restore the opened groups
	groups = contacts->getChildrenOfType(CE_GROUP);
	search = groups;
	while(search)
	{
		contactsGroup = (contactListGroup*)search->data;
		contactsGroup->setGroupOpened(((groupInfo*)contactsGroup->data)->opened);
		search = search->next;
	}
	g_list_free(groups);
	
	return showGroups;
}





// =============================================================================

gboolean mainWindow::cb_contactListClicked(GtkTreeView* cList, GdkEventButton* event, mainWindow* mw)
{
	contactListEntry *focus;
	gulong			 capats;
		
	if (event->button != 3)
		return FALSE;
	
	focus = mw->contacts->getEntryAtCursor((gint)event->x, (gint)event->y);
	
	if (!focus)
		return FALSE;
	

 	if (focus->type == CE_USER)
	{
		// find out what menus to enable / disable
		capats = ((IMOwnerDaemon*)((IMUserDaemon*)focus->data)->owner)->protocol->functions;

		(capats & PP_SEND_MSG) != 0? gtk_widget_show(mw->menu_sendMessage): gtk_widget_hide(mw->menu_sendMessage);
		(capats & PP_SEND_URL) != 0? gtk_widget_show(mw->menu_sendURL): gtk_widget_hide(mw->menu_sendURL);
		(capats & PP_SEND_FILE) != 0? gtk_widget_show(mw->menu_sendFiles): gtk_widget_hide(mw->menu_sendFiles);
		(capats & PP_SEND_CHAT) != 0? gtk_widget_show(mw->menu_sendChat): gtk_widget_hide(mw->menu_sendChat);
		(capats & PP_SEND_CONTACT) != 0? gtk_widget_show(mw->menu_sendContacts): gtk_widget_hide(mw->menu_sendContacts);
		(capats & PP_SEND_AUTH) != 0? gtk_widget_show(mw->menu_sendAuth): gtk_widget_hide(mw->menu_sendAuth);
		(capats & PP_SEND_AUTHxREQ) != 0? gtk_widget_show(mw->menu_sendAuthReq): gtk_widget_hide(mw->menu_sendAuthReq);
		(capats & PP_SEND_SECURE) != 0? gtk_widget_show(mw->menu_sendEncryption): gtk_widget_hide(mw->menu_sendEncryption);
		
		gtk_widget_set_sensitive(mw->menu_openUserEvent, ((contactListUser*)focus)->isFlashing());
		gtk_label_set_label(GTK_LABEL(mw->menu_userMenuLabel), focus->name);
		gtk_menu_popup(mw->userMenu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
	}
	return FALSE;
}




void mainWindow::cb_contactListActivated(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, mainWindow* mw)
{
	contactListEntry *focus;
	
	focus = mw->contacts->getFocusedEntry();
	
	if (!focus)
		return;
	
	// a group was clicked -> collapse or expand it
	if (focus->type == CE_GROUP)
	{
		if (((contactListGroup*)focus)->getGroupOpened())
			((contactListGroup*)focus)->setGroupOpened(FALSE);
		else
			((contactListGroup*)focus)->setGroupOpened(TRUE);
	} else
	{
		mw->menuCallBack(NULL, MENU_DOUBLECLICK, 0, mw->menuCallBackData);		
	}
}



void mainWindow::cb_menuClicked(GtkWidget* w, GtkWidget* toshow)
{
	gtk_menu_popup(GTK_MENU(toshow), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());

}


void mainWindow::cb_menuItemClicked(GtkWidget* item, mainWindow* mw)
{
	guint menu, 
		  selection;
	
	menu = (guint)g_object_get_data(G_OBJECT(item), "ICQmenuMENU");
	selection = (guint)g_object_get_data(G_OBJECT(item), "ICQmenuID");
			
	if (selection != MENU_NONE && mw->menuCallBack)
		mw->menuCallBack(item, menu, selection, mw->menuCallBackData);
}



gboolean mainWindow::cb_destroyWindow(mainWindow* mw)
{
	gint width, height, x, y;
	
	gtk_window_get_position(GTK_WINDOW(mw->window), &x, &y);
	gtk_window_get_size(GTK_WINDOW(mw->window), &width, &height);
	
	settings_getSettings()->setProperties(FALSE, "mainwindow",
										  "width", width,
										  "height", height,
										  "positionX", x,
										  "positionY", y,
										  NULL);
	
	gtk_main_quit();
	
	gtk_widget_destroy(mw->window);
	
	return FALSE;
}



gboolean mainWindow::cb_visibilityChange(mainWindow* mw, GdkEventVisibility *event)
{
	mw->isObscured = (event->state == GDK_VISIBILITY_FULLY_OBSCURED);
	
	return FALSE;
}




gboolean mainWindow::cb_trayClicked(mainWindow* mw, GdkEventButton* event)
{
	mw->trayClicked(event->type, event->button);
	
	return FALSE;
}


gboolean mainWindow::cb_trayAnimation(mainWindow* mw)
{
	GdkPixbuf	*icon;
	
	mw->trayIter++;
	mw->trayIter = mw->trayIter % 2;
	
	if (mw->trayIter)
		icon = i_getIcons()->getStatusPixbuf("icq", BUDDY_STATUS_ONLINE, FALSE);
	else
		icon = i_getIcons()->getStatusPixbuf("icq", BUDDY_STATUS_OFFLINE, FALSE);
	
	gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), icon);
	
	return TRUE;
}



gboolean mainWindow::cb_trayEventBlinking(mainWindow* mw)
{
	// don't do anything if the connect animation is visible
	if (mw->trayConnectSource)
		return TRUE;
	
	mw->trayShowEventIcon = !mw->trayShowEventIcon;
	
	if (mw->trayShowEventIcon)
		gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), mw->eventPixbuf);
	else
		gtk_image_set_from_pixbuf(GTK_IMAGE(mw->tray->iconWidget), mw->lastStatusPixbuf);
	
	return TRUE;
}


gboolean mainWindow::cb_statusButtonBlinking(statusButtonType* but)
{
	gchar	*protoName;
	
	but->connectImg = !but->connectImg;
	
	protoName = ((IMOwnerDaemon*)but->owner)->protocol->licqName;
	if (!strcmp(protoName, "Licq"))
		protoName = "icq";
	
	if (but->connectImg)
		gtk_image_set_from_pixbuf(GTK_IMAGE(but->statusImage), i_getIcons()->getStatusPixbuf(protoName, BUDDY_STATUS_ONLINE, FALSE));
	else
		gtk_image_set_from_pixbuf(GTK_IMAGE(but->statusImage), i_getIcons()->getStatusPixbuf(protoName, BUDDY_STATUS_OFFLINE, FALSE));		

	return TRUE;	
}



gboolean mainWindow::cb_notifyBoxClicked(GtkWidget* box, GdkEventButton* event, mainWindow* mw)
{
	if (event->type == GDK_2BUTTON_PRESS && mw->menuCallBack)
		mw->menuCallBack(NULL, MENU_NOTIFYBOX, 0, mw->menuCallBackData);
	return FALSE;
}




void mainWindow::cb_settingsChangedCallback(gchar* sname, GList *values, mainWindow* self)
{
	settingsProperty *prop;
	gboolean		 changeCView;
	GList			 *children,
					 *search;
	IMUserDaemon	 *user;
	statusButtonType *but;
	contactListUser	 *contactUser;
	iconManager		 *iconMan;
	
	printf("change settings\n");
	
	if (!strcmp(sname, "appearance"))
	{
		while(values)
		{
			prop = (settingsProperty*)values->data;
			
			// was the status theme changed? update all icons / status button images ...
			if (!strcmp(prop->name, "StatusTheme"))
			{
				iconMan = i_getIcons();
		
				// update the contact list icons			
				children = self->contacts->getChildrenOfType(CE_USER);
			
				search = children;
				while(search)
				{
					contactUser = (contactListUser*)search->data;
					user = (IMUserDaemon*)contactUser->data;
				
					contactUser->setStatus(user->info->status, iconMan->getUserStatusPixbuf(user));
				
					search = search->next;
				}
				g_list_free(children);
			
				// update the status buttons
				search = self->statusButtons;
				while(search)
				{
					but = (statusButtonType*)search->data;
					self->updateStatusButton((IMOwnerDaemon*)but->owner, ((IMOwnerDaemon*)but->owner)->info->status, ((IMOwnerDaemon*)but->owner)->info->isInvisible);
				
					search = search->next;
				}
			
				self->updateStatus(self->lastStatus, self->lastInvisible);	

				break;				
			}
			
			values = values->next;
		}
		
	} else
	{
		changeCView = TRUE;
		
		while(values)
		{
			prop = (settingsProperty*)values->data;
	
			if (changeCView)
			{			
				if (!strcmp(prop->name, "showGroups") || 
					!strcmp(prop->name, "showEmptyGroups") ||
					!strcmp(prop->name, "showOfflineUsers")
				   )
				{				
					gtk_widget_set_sensitive(self->menu_showEmptyGroups, self->updateContactListViewMode());
				
					// set changeCView to false so the list won't be updated again
					// if there's another value that would change its look
					changeCView = FALSE;		
				}
	
			}
			
			if (!strcmp(prop->name, "showTooltips"))
			{
				if (g_value_get_boolean(prop->val))
				{
					g_signal_connect_swapped(self->contacts->tv, "leave-notify-event", G_CALLBACK(self->cb_tooltipLeaveNotify), self);
					g_signal_connect(self->contacts->tv, "motion-notify-event", G_CALLBACK(self->cb_tooltipMoveNotify), self);
				} else
				{
					g_signal_handlers_disconnect_by_func(self->contacts->tv, (GClosure*)self->cb_tooltipLeaveNotify, self);
					g_signal_handlers_disconnect_by_func(self->contacts->tv, (GClosure*)self->cb_tooltipMoveNotify, self);
					if (self->tooltipSource)
					{
						g_source_remove(self->tooltipSource);
						self->tooltipSource = 0;
					}
					if (self->tooltipWindow)
					{
						gtk_widget_destroy(self->tooltipWindow);
						self->tooltipWindow = NULL;
					}
				}
			} else
			if (!strcmp(prop->name, "tooltipsMask"))
				self->tooltipsMask = g_value_get_ulong(prop->val);
		
			values = values->next;
		}
	}
}



gboolean mainWindow::cb_tooltipLeaveNotify(mainWindow* self, GdkEventCrossing* event)
{
	GtkWidget		*newWindow;
	GList			*children;
	gint			posX, posY;
	
	if (self->tooltipWindow)
	{
		// here comes the tricky part. If a user presses CTRL while going to the tooltip
		// it gets managed so information can be copied
		if (event->state & GDK_CONTROL_MASK)
		{
			newWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			gtk_widget_set_app_paintable(newWindow, TRUE);
			gtk_widget_set_name(newWindow, "gtk-tooltips");
			gtk_window_set_resizable(GTK_WINDOW(newWindow), FALSE);			
			gtk_window_set_skip_taskbar_hint(GTK_WINDOW(newWindow), TRUE);
			gtk_container_set_border_width(GTK_CONTAINER(newWindow), 10);
			
			children = gtk_container_get_children(GTK_CONTAINER(self->tooltipWindow));
			gtk_widget_reparent(GTK_WIDGET(children->data), newWindow);
			g_list_free(children);
			
			gtk_window_get_position(GTK_WINDOW(self->tooltipWindow), &posX, &posY);
			gtk_window_move(GTK_WINDOW(newWindow), posX, posY);
			
			gtk_widget_show_all(newWindow);
		}	
		
		gtk_widget_destroy(self->tooltipWindow);
		self->tooltipWindow = NULL;
	}
	if (self->tooltipSource)
	{
		g_source_remove(self->tooltipSource);
		self->tooltipSource = 0;
	}
	

	//scr = gdk_screen_get_default();
	//gdk_display_get_pointer(gdk_display_get_default(), &scr, &posX, &posY, NULL);		
	
	
	self->lastTooltipEntry = NULL;
	
	return FALSE;
}




gboolean mainWindow::cb_tooltipPaint(mainWindow* self)
{
	GtkRequisition	req;
		
	gtk_widget_size_request(self->tooltipWindow, &req);
	gtk_paint_flat_box(self->tooltipWindow->style,
					   self->tooltipWindow->window,
					   GTK_STATE_NORMAL,
					   GTK_SHADOW_OUT,
					   NULL,
					   self->tooltipWindow,
					   "tooltip",
					   0, 0,
					   req.width, req.height);
		
	return FALSE;
}


void mainWindow::cb_tooltipRealize(mainWindow* self)
{
	gint 		posX, 
				posY, 
				maxX, 
				maxY;
	GdkScreen	*scr;
	GtkRequisition	req;
	
	// positioning cannot be done before the content is realized, because
	// the size of all children is not known. Do this after the realization of 
	// the window.
	
	scr = gdk_screen_get_default();
	gdk_display_get_pointer(gdk_display_get_default(), &scr, &posX, &posY, NULL);	
	
	// move the window to the right position
	maxX = gdk_screen_get_width(scr);
	maxY = gdk_screen_get_height(scr);	

	gtk_widget_size_request(self->tooltipWindow, &req);
	
	if (posX + req.width > maxX) 
		posX-= req.width+2;
	
	if (posY + req.height > maxY) 
		posY-= req.height+2;
		
	gtk_window_move(GTK_WINDOW(self->tooltipWindow),posX+1,posY+1);	
}



#define MW_MAKE_TT_ITEM(txt, c)											\
		tmp = g_strdup_printf(txt, c);									\
		label2 = gtk_label_new(tmp);									\
		gtk_label_set_selectable(GTK_LABEL(label2), TRUE);				\
		gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);				\
		gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);			\
		gtk_box_pack_start(GTK_BOX(vbox1), label2, FALSE, TRUE, 0);		\
		g_free(tmp);

gboolean mainWindow::cb_tooltipShowTimeout(mainWindow* self)
{
	GtkWidget	*hbox1,
				*vbox1,
				*vbox2,
				*label1,
				*label2,
				*img,
				*alignment1,
				*alignment2;
	gchar		*tmp;
	GString		*realName;
	contactListEntry	*entry;
	IMUserDaemon		*user;
	IMAutoResponseManager *aman;
	
	// create the tooltip window
	self->tooltipWindow = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_widget_set_app_paintable(self->tooltipWindow, TRUE);	
	gtk_container_set_border_width(GTK_CONTAINER(self->tooltipWindow), 10);
	g_signal_connect_swapped(self->tooltipWindow, "expose-event", G_CALLBACK(self->cb_tooltipPaint), self);	
	g_signal_connect_swapped(self->tooltipWindow, "realize", G_CALLBACK(self->cb_tooltipRealize), self);

	gtk_widget_ensure_style(self->tooltipWindow);

	gtk_widget_set_name(self->tooltipWindow, "gtk-tooltips");
	
	
	// clear the old content
	gtk_container_foreach(GTK_CONTAINER(self->tooltipWindow), (GtkCallback)(gtk_widget_destroy), NULL);


	// fill the new content
	if (self->lastTooltipEntry)
	{
		user = (IMUserDaemon*)self->lastTooltipEntry->data;
		
		vbox1 = gtk_vbox_new(FALSE, 2);
		
		// the status image
		img = gtk_image_new_from_pixbuf(i_getIcons()->getUserStatusPixbuf(user));
		alignment1 = gtk_alignment_new(0.0f, 0.0f, 0.0f, 0.0f);
		gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 5, 0, 0, 0);
		gtk_container_add(GTK_CONTAINER(alignment1), img);
		
		// the relevant information
		tmp = g_strdup_printf("<big><b>%s</b></big>", user->info->formattedAlias);
		label1 = gtk_label_new(tmp);
		gtk_label_set_selectable(GTK_LABEL(label1), TRUE);
		gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
		gtk_misc_set_alignment(GTK_MISC(label1), 0.0f, 0.0f);
		g_free(tmp);
		
		// insert the real name
		if (self->tooltipsMask & TT_REALNAME)
		{
			realName = g_string_new(user->info->firstname);
			if (strlen(realName->str))
				realName = g_string_append(realName, " ");
			realName = g_string_append(realName, user->info->lastname);
			MW_MAKE_TT_ITEM(tr("<b>Real Name:</b> %s"), realName->str);
			g_string_free(realName, TRUE);
		}
		
		// insert the status
		if (self->tooltipsMask & TT_STATUS)
		{
			MW_MAKE_TT_ITEM(tr("<b>Status:</b> %s"), uu_getStatusDescription(user->info->status));
		}
		
		// insert the user id		
		if (self->tooltipsMask & TT_ID)
		{
			MW_MAKE_TT_ITEM(tr("<b>User ID:</b> %s"), user->info->formattedID);
		}
			
		// insert the protocol name
		if (self->tooltipsMask & TT_PROTOCOL)
		{
			MW_MAKE_TT_ITEM(tr("<b>Protocol:</b> %s"), ((IMOwnerDaemon*)user->owner)->protocol->name);
		}
			
		// insert the ip address
		if (self->tooltipsMask & TT_IP)
		{
			MW_MAKE_TT_ITEM(tr("<b>IP Address:</b> %s"), user->info->formattedIP);
		}
			
		// insert the auto response
		if (self->tooltipsMask & TT_AUTORESPONSE && user->info->status != BUDDY_STATUS_OFFLINE && user->info->status != BUDDY_STATUS_ONLINE)
		{
			aman = new IMAutoResponseManager(user->info, TRUE);
			user->addManager(aman);
									
			label2 = gtk_label_new(tr("<b>Auto Response:</b>"));									
			gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);				
			gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);			
			gtk_box_pack_start(GTK_BOX(vbox1), label2, FALSE, TRUE, 0);

			tmp = aman->getCustomResponse();
			if (strlen(tmp))
			{
				label2 = gtk_label_new(tmp);
				gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);
				gtk_label_set_selectable(GTK_LABEL(label2), TRUE);
				gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
				gtk_label_set_line_wrap(GTK_LABEL(label2), TRUE);
				alignment2 = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
				gtk_alignment_set_padding(GTK_ALIGNMENT(alignment2), 0, 0, 20, 0);
				gtk_container_add(GTK_CONTAINER(alignment2), label2);
				gtk_box_pack_start(GTK_BOX(vbox1), alignment2, FALSE, TRUE, 0);
			}
			g_free(tmp);

			delete aman;
		}
		
		
		// pack the vbox and the icon into a hbox
		hbox1 = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(GTK_BOX(hbox1), alignment1, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(hbox1), vbox1, TRUE, TRUE, 0);
		
		// now add the headline
		vbox2 = gtk_vbox_new(FALSE, 6);
		gtk_box_pack_start(GTK_BOX(vbox2), label1, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox2), hbox1, TRUE, TRUE, 0);
		
	
		gtk_container_add(GTK_CONTAINER(self->tooltipWindow), vbox2);
	}

	
	// show the window
	gtk_widget_show_all(self->tooltipWindow);
	
	
	self->tooltipSource = 0;
	return FALSE;
}


gboolean mainWindow::cb_tooltipMoveNotify(GtkWidget* w, GdkEventMotion *event, mainWindow* self)
{
	contactListEntry	*entry;
	
	entry = self->contacts->getEntryAtCursor((gint)event->x, (gint)event->y);
	
	// no entry or we are not pointing to a user? destroy the tooltip
	if (!entry || entry->type != CE_USER)
	{
		if (self->tooltipWindow)
		{
			gtk_widget_destroy(self->tooltipWindow);
			self->tooltipWindow = NULL;
		}
		if (self->tooltipSource)
		{
			g_source_remove(self->tooltipSource);
			self->tooltipSource = 0;
		}
		self->lastTooltipEntry = NULL;
		
		return FALSE;
	}
	
	// a new user -> create the tooltip
	if (entry != self->lastTooltipEntry)
	{
		if (self->tooltipWindow)
		{
			gtk_widget_destroy(self->tooltipWindow);
			self->tooltipWindow = NULL;
		}
		if (self->tooltipSource)
		{
			g_source_remove(self->tooltipSource);
			self->tooltipSource = 0;
		}
		self->tooltipSource = g_timeout_add(1000, (GSourceFunc)self->cb_tooltipShowTimeout, self);
		self->lastTooltipEntry = entry;
	}

	return FALSE;	
}
