/***************************************************************************
 *            optionswindow.cpp
 *
 *  Fri Jul 29 20:01:44 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "optionswindow.h"


#define OW_CREATE_MAINVBOX(w)								\
	w = gtk_vbox_new(FALSE, 2);								\
	gtk_container_set_border_width(GTK_CONTAINER(w), 10);

#define OW_CREATE_SECTION(caption, vbox)								\
	label = gtk_label_new(tr(caption));						\
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);					\
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);				\
																		\
	vbox = gtk_vbox_new(FALSE, 2);										\
	alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);				\
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 12, 20, 0);	\
	gtk_container_add(GTK_CONTAINER(alignment), vbox);					
	
	
#define OW_CREATE_SECTION_T(caption, t, c, r)							\
	label = gtk_label_new(tr(caption));						\
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);					\
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);				\
																		\
	t = gtk_table_new(r, c, FALSE);										\
	alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);				\
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 8, 20, 0);	\
	gtk_container_add(GTK_CONTAINER(alignment), t);						


typedef struct toolbarElement
{
	guint	 num;
	gchar	 *title;
	gboolean oneTime;
};

toolbarElement toolbarElements[] =
{
	{1, tr_NOOP("User status"), TRUE},
	{2, tr_NOOP("User time"), TRUE},
	{0, "-", FALSE},
	{3, tr_NOOP("Set encoding"), FALSE},
	{4, tr_NOOP("Encrypt connection"), FALSE},
	{5, tr_NOOP("Show History"), FALSE},
	{6, tr_NOOP("Show user info"), FALSE},
	{7, tr_NOOP("Insert smileys"), FALSE},
	{8, tr_NOOP("Send Files"), FALSE},
	{9, tr_NOOP("Send URLs"), FALSE},
	{10, tr_NOOP("Start a Chat"), FALSE},
	{11, tr_NOOP("Set Text Color"), FALSE},
	{12, tr_NOOP("Set Background"), FALSE},
	{0, "-", FALSE},
	{13, tr_NOOP("Send via Server"), TRUE},
	{14, tr_NOOP("Send urgent"), TRUE},
	{15, tr_NOOP("Send the message"), TRUE},
	{16, tr_NOOP("Send to multiple"), TRUE},
	{0, "-", FALSE},
	{17, tr_NOOP("Send Animation"), TRUE}, 
	{18, tr_NOOP("Separator"), FALSE},
	{19, tr_NOOP("<- (Align left)"), FALSE},
	{20, tr_NOOP("-> (Align right)"), FALSE},
	{0, NULL, FALSE}
};



optionsWindow::optionsWindow()
{
	wType = UE_OPTIONS;
}


optionsWindow::~optionsWindow()
{
	GList	*search;
	
	search = optionsItems;
	while(search)
	{
		delete (optionsWindowItem*)search->data;
		
		search = search->next;
	}
	
	g_list_free(optionsItems);
}

		
GtkWidget* optionsWindow::createWindowContent()
{
	GtkCellRenderer		*renderer;
	GtkTreeViewColumn	*column;
	GtkWidget			*scrolled,
						*hbox,
						*vbox;
	GtkTreeIter			iter;
	GList				*search;
	optionsWindowItem	*currentItem;			
	
	// show a message
	u_showAlertMessage("Info", 	"IcQnD is still under active development. "
								"However, the options window isn't finished yet. "
								"The options that can be selected will also affect "
								"the program. There are tabs which are grayed out. "
								"These tabs are \"designed\" but not functional yet. "
								"Please stay patient, all settings will be available "
								"in the next release. If you look for a specific "
								"option that is not accessible in the options window "
								"now take a look in ~/.licq/licq_qnd-gui.conf.", GTK_STOCK_DIALOG_INFO);
	
	
	// initialize the items
	optionsItems = NULL;
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_mainWindow());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_msgWindow());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_extensions());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_onEvent());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_connections());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_plugins());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_start());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_aresponse());
	optionsItems = g_list_append(optionsItems, new optionsWindowItem_themes());
	
	
	
	// create the notebook
	notebook = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	
	
	// create the store for the options items
	itemsStore = gtk_list_store_new(2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	search = optionsItems;
	
	while(search)
	{
		currentItem = (optionsWindowItem*)search->data;
		
		gtk_list_store_append(itemsStore, &iter);
		gtk_list_store_set(itemsStore, &iter,
						   0, currentItem->titleImage,
						   1, currentItem->titleString,
						   -1);
		
		currentItem->parentContainer = gtk_vbox_new(FALSE, 0);
		
		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), currentItem->parentContainer, NULL);
		
		search = search->next;
	}
	
	
	// create a view for the store
	itemsView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(itemsStore));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(itemsView), FALSE);
	g_signal_connect_swapped(gtk_tree_view_get_selection(GTK_TREE_VIEW(itemsView)), "changed", G_CALLBACK(this->cb_optionsItemSelected), this);
	
	// create the cell renderers
	renderer = gtk_cell_renderer_pixbuf_new();
	column = gtk_tree_view_column_new_with_attributes("", renderer,
													  "pixbuf", 0,
													  NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(itemsView), column);
	
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("", renderer,
													  "text", 1,
													  NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(itemsView), column);
	
	// create a scrolled window around the view
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scrolled), itemsView);
	gtk_widget_set_size_request(scrolled, 120, -1);
	
	
	
	// add the item view + notebook in a hbox
	hbox = gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox), scrolled, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), notebook, TRUE, TRUE, 0);
	
	// create a vbox containing the hbox and the button bar
	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), createButtonbar(), FALSE, TRUE, 0);
	
	
	setWindowSize(600, 600);
	
	
	return vbox;
}



void optionsWindow::destroyWindowContent()
{

}



GtkWidget	*optionsWindow::createButtonbar()
{
	GtkWidget	*hbox,
				*closeButton;
	
	applyButton = gtk_button_new_from_stock(GTK_STOCK_APPLY);
	g_signal_connect_swapped(applyButton, "clicked", G_CALLBACK(this->cb_applyButtonClicked), this);
	
	closeButton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	g_signal_connect_swapped(closeButton, "clicked", G_CALLBACK(this->cb_closeButtonClicked), this);
	
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
	gtk_box_set_spacing(GTK_BOX(hbox), 2);
	
	gtk_box_pack_start(GTK_BOX(hbox), applyButton, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), closeButton, FALSE, TRUE, 0);
	
	return hbox;
}


void optionsWindow::cb_optionsItemSelected(optionsWindow* self)
{
	GtkTreePath	*path;
	gint		pos;
	optionsWindowItem *currentItem;
	
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(self->itemsView), &path, NULL);

	if (!path) 
		return;
	
	pos = gtk_tree_path_get_indices(path)[0];	
	currentItem = (optionsWindowItem*)g_list_nth_data(self->optionsItems, pos);
	
	if (!currentItem->tabWidget)
	{
		currentItem->tabWidget = currentItem->createTab();
		gtk_widget_show_all(currentItem->tabWidget);
		gtk_box_pack_start(GTK_BOX(currentItem->parentContainer), currentItem->tabWidget, TRUE, TRUE, 0);
	}
	
	currentItem->window = self->window;
	gtk_notebook_set_show_border(GTK_NOTEBOOK(self->notebook), !currentItem->hasOwnNotebook);
	
	gtk_notebook_set_current_page(GTK_NOTEBOOK(self->notebook), pos);
	
	gtk_tree_path_free(path);
}



void optionsWindow::cb_applyButtonClicked(optionsWindow* self)
{
	GList				*search;
	optionsWindowItem	*currentItem;

	search = self->optionsItems;
	while(search)
	{
		currentItem = (optionsWindowItem*)search->data;
		
		if (currentItem->tabWidget)
			currentItem->applyChanges();
		search = search->next;
	}		
	
	getLicqDaemon()->SaveConf();
}


void optionsWindow::cb_closeButtonClicked(optionsWindow* self)
{
	
}


// -----------------------------------------------------------------------------

optionsWindowItem::optionsWindowItem()
{
	titleImage = NULL;
	titleString = "dummy";
	tabWidget = NULL;
	parentContainer = NULL;
	hasOwnNotebook = FALSE;
}

optionsWindowItem::~optionsWindowItem()
{
	
}

GtkWidget *optionsWindowItem::createTab()
{
	GtkWidget	*label;
	
	label = gtk_label_new("This tab isn't finished yet. Work is in progress. If you are looking for a specific option and you do not find it in the other dialogs, please take a look at ~/.licq/licq_qnd-gui.conf. Stay patient!");
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	
	return label;
}

void optionsWindowItem::destroyTab()
{
	
}

gboolean optionsWindowItem::applyChanges()
{
	return TRUE;
}

// -----------------------------------------------------------------------------

optionsWindowItem_mainWindow::optionsWindowItem_mainWindow()
{
	titleImage = NULL;
	titleString = tr_NOOP("Main Window");
	hasOwnNotebook = TRUE;	
}


optionsWindowItem_mainWindow::~optionsWindowItem_mainWindow()
{
	
}


GtkWidget* optionsWindowItem_mainWindow::createTab()
{
	GtkWidget	*notebook;
	
	notebook = gtk_notebook_new();
	//gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createMainWindowTab(), gtk_label_new(tr("Main Window")));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createContactlistTab(), gtk_label_new(tr("Contact List")));
	//gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createTrayIconTab(), gtk_label_new(tr("Tray Icon")));
	
	printf("create tab\n");
	
	return notebook;
}



gboolean optionsWindowItem_mainWindow::applyChanges()
{
	gulong	tooltipsMask;
	
	// apply tooltip changes
	tooltipsMask = 0;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tipStatusCheckbox)))
		tooltipsMask|= TT_STATUS;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tipRealnameCheckbox)))
		tooltipsMask|= TT_REALNAME;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tipIPCheckbox)))
		tooltipsMask|= TT_IP;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tipARCheckbox)))
		tooltipsMask|= TT_AUTORESPONSE;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tipProtocolCheckbox)))
		tooltipsMask|= TT_PROTOCOL;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tipIDCheckbox)))
		tooltipsMask|= TT_ID;
	
	settings_getSettings()->setProperties(TRUE, "contactlist",
										  "showTooltips", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showTipsCheckbox)),
										  "tooltipsMask", tooltipsMask,
										  NULL);
	
	return TRUE;
}



GtkWidget* optionsWindowItem_mainWindow::createMainWindowTab()
{
	return gtk_label_new("Main window");
}




GtkWidget* optionsWindowItem_mainWindow::createContactlistTab()
{
	GtkWidget	*mainVBox,
				*vbox,
				*label,
				*alignment,
				*table;
	gboolean	showTooltips;
	gulong		tooltipsMask;
	
	settings_getSettings()->getProperties("contactlist",
										  "showTooltips", &showTooltips,
										  "tooltipsMask", &tooltipsMask,
										  NULL);
		
	
	OW_CREATE_MAINVBOX(mainVBox);	
	
	// contact list sorting
	OW_CREATE_SECTION(tr("<b>Sort order</b>"), vbox);
	
	sortOrderList = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(sortOrderList), tr("last Event"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(sortOrderList), tr("Number of Events"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(sortOrderList), tr("Alphabetic"));
	
	sortStatusCheckbox = gtk_check_button_new_with_mnemonic("Also sort the _Status of the Users");
	
	gtk_box_pack_start(GTK_BOX(vbox), sortOrderList, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), sortStatusCheckbox, FALSE, TRUE, 2);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	gtk_widget_set_sensitive(label, FALSE);
	gtk_widget_set_sensitive(alignment, FALSE);
	
	
	// events
	OW_CREATE_SECTION(tr("<b>Events</b>"), vbox);
	
	blinkByOnlineCheckbox = gtk_check_button_new_with_mnemonic(tr("Highlight user when he goes online"));
	blinkByARCheckbox = gtk_check_button_new_with_mnemonic(tr("Highlight user when he checks your _auto response"));
	blinkEventsCheckbox = gtk_check_button_new_with_mnemonic(tr("Blink events"));

	gtk_box_pack_start(GTK_BOX(vbox), blinkByOnlineCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), blinkByARCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), blinkEventsCheckbox, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);

	gtk_widget_set_sensitive(label, FALSE);
	gtk_widget_set_sensitive(alignment, FALSE);
	

	// colors an look
	OW_CREATE_SECTION(tr("<b>Colors &amp; Look</b>"), vbox);

	detailledModeCheckbox = gtk_check_button_new_with_mnemonic(tr("Show detailled information in contact list"));
	useColorsCheckbox = gtk_check_button_new_with_mnemonic(tr("Use different colors for system groups"));
	useRealNamesCheckbox = gtk_check_button_new_with_mnemonic(tr("Show real names when possible"));
	showExtendedCheckbox = gtk_check_button_new_with_mnemonic(tr("Show extended icons (birthday, typing ...)"));
	
	gtk_box_pack_start(GTK_BOX(vbox), detailledModeCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), useColorsCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), useRealNamesCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), showExtendedCheckbox, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	gtk_widget_set_sensitive(label, FALSE);
	gtk_widget_set_sensitive(alignment, FALSE);
	
	
	// tool tips
	OW_CREATE_SECTION_T(tr("<b>Tooltips</b>"), table, 2, 6);
	showTipsCheckbox = gtk_check_button_new_with_mnemonic(tr("Show tool tips"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showTipsCheckbox), showTooltips);
	g_signal_connect_swapped(showTipsCheckbox, "toggled", G_CALLBACK(this->cb_showTooltipsToggled), this);

	tipStatusCheckbox = gtk_check_button_new_with_mnemonic(tr("Status"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipStatusCheckbox), tooltipsMask & TT_STATUS);

	tipProtocolCheckbox = gtk_check_button_new_with_mnemonic(tr("Protocol"));	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipProtocolCheckbox), tooltipsMask & TT_PROTOCOL);
	
	tipRealnameCheckbox = gtk_check_button_new_with_mnemonic(tr("Real name"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipRealnameCheckbox), tooltipsMask & TT_REALNAME);

	tipMessageCountCheckbox = gtk_check_button_new_with_mnemonic(tr("Message count"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipMessageCountCheckbox), tooltipsMask & TT_MSGCOUNT);

	tipIPCheckbox = gtk_check_button_new_with_mnemonic(tr("IP Address"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipIPCheckbox), tooltipsMask & TT_IP);

	tipOnlineSinceCheckbox = gtk_check_button_new_with_mnemonic(tr("Online time"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipOnlineSinceCheckbox), tooltipsMask& TT_ONLINETIME);
	
	tipIDLETimeCheckbox	= gtk_check_button_new_with_mnemonic(tr("Idle time"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipIDLETimeCheckbox), tooltipsMask & TT_IDLETIME);

	tipARCheckbox = gtk_check_button_new_with_mnemonic(tr("Auto response"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipARCheckbox), tooltipsMask & TT_AUTORESPONSE);	
	
	tipIDCheckbox = gtk_check_button_new_with_mnemonic(tr("User ID"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tipIDCheckbox), tooltipsMask & TT_ID);
	
	gtk_table_attach(GTK_TABLE(table), showTipsCheckbox, 0, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 5);



	gtk_table_attach_defaults(GTK_TABLE(table), tipRealnameCheckbox, 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), tipIDCheckbox, 1, 2, 1, 2);

	gtk_table_attach_defaults(GTK_TABLE(table), tipStatusCheckbox, 0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(table), tipProtocolCheckbox, 1, 2, 2, 3);	

	gtk_table_attach_defaults(GTK_TABLE(table), tipIPCheckbox, 0, 1, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(table), tipMessageCountCheckbox, 1, 2, 3, 4);

	gtk_table_attach_defaults(GTK_TABLE(table), tipOnlineSinceCheckbox, 0, 1, 4, 5);
	gtk_table_attach_defaults(GTK_TABLE(table), tipIDLETimeCheckbox, 1, 2, 4, 5);

	gtk_table_attach_defaults(GTK_TABLE(table), tipARCheckbox, 0, 1, 5, 6);	
	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	
	// gray unfinished stuff out
	gtk_widget_set_sensitive(tipMessageCountCheckbox, FALSE);
	gtk_widget_set_sensitive(tipOnlineSinceCheckbox, FALSE);
	gtk_widget_set_sensitive(tipIDLETimeCheckbox, FALSE);
	
	cb_showTooltipsToggled(this);
	
	
	return mainVBox;
}



GtkWidget* optionsWindowItem_mainWindow::createTrayIconTab()
{
	return gtk_label_new("Tray icon");
}



void optionsWindowItem_mainWindow::cb_showTooltipsToggled(optionsWindowItem_mainWindow* self)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->showTipsCheckbox)))
	{
		gtk_widget_set_sensitive(self->tipStatusCheckbox, TRUE);
		gtk_widget_set_sensitive(self->tipRealnameCheckbox, TRUE);
		gtk_widget_set_sensitive(self->tipIPCheckbox, TRUE);
		gtk_widget_set_sensitive(self->tipARCheckbox, TRUE);
		gtk_widget_set_sensitive(self->tipProtocolCheckbox, TRUE);
		gtk_widget_set_sensitive(self->tipIDCheckbox, TRUE);
	} else
	{
		gtk_widget_set_sensitive(self->tipStatusCheckbox, FALSE);
		gtk_widget_set_sensitive(self->tipRealnameCheckbox, FALSE);
		gtk_widget_set_sensitive(self->tipIPCheckbox, FALSE);
		gtk_widget_set_sensitive(self->tipARCheckbox, FALSE);
		gtk_widget_set_sensitive(self->tipProtocolCheckbox, FALSE);		
		gtk_widget_set_sensitive(self->tipIDCheckbox, FALSE);
	}
}



// -----------------------------------------------------------------------------

optionsWindowItem_msgWindow::optionsWindowItem_msgWindow()
{
	titleImage = NULL;
	titleString = tr_NOOP("Conversation");	
	hasOwnNotebook = TRUE;
	currentbar = &remotebar;
}


optionsWindowItem_msgWindow::~optionsWindowItem_msgWindow()
{
	
}


GtkWidget *optionsWindowItem_msgWindow::createTab()
{
	GtkWidget	*notebook,
				*previewNotebook,
				*vbox,
				*preview,
				*alignment,
				*label;
	

	// create the tool bars
	settings_getSettings()->getProperties("conversations",
										  "remoteToolbarRules", &remotebar.rules,
										  "localToolbarRules", &localbar.rules,
										  "buttonbarRules", &buttonbar.rules,
										  "remoteToolbarSpacing", &remotebar.spacing,
										  "localToolbarSpacing", &localbar.spacing,
										  "buttonbarSpacing", &buttonbar.spacing,
										  "showRemoteToolbar", &remotebar.visible,
										  "showLocalToolbar", &localbar.visible,
										  "showButtonbar", &buttonbar.visible,
										  "messageTag", &messageTag,
										  "dontApplyColors", &dontApplyColors,
										  "standardEncoding", &standardEncoding,
										  "sendTyping", &sendTyping,
										  "showSmileys", &showSmileys,
										  "showURIs", &showURIs,
										  "showHistory", &showHistory,
										  "historyCount", &historyCount,
										  NULL);
										  
	settings_getSettings()->getProperties("appearance",
										  "colorReceive", &incomingCol,
										  "colorSend", &outgoingCol,
										  "tabMessageColor", &msgCol,
										  "tabTypingColor", &typeCol,
										  NULL);
	
	
	// create the preview container
	preview = createPreview();
	// create the three toolbars
	createPreviewToolbar(&remotebar);
	createPreviewToolbar(&localbar);
	createPreviewToolbar(&buttonbar);
	
	gtk_container_set_border_width(GTK_CONTAINER(preview), 5);							
	
	previewNotebook = gtk_notebook_new();
	gtk_notebook_append_page(GTK_NOTEBOOK(previewNotebook), preview, NULL);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(previewNotebook), FALSE);
	
	notebook = gtk_notebook_new();
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createGeneralTab(), gtk_label_new("General"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createColorsTab(), gtk_label_new("Colors / Format"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createToolbarsTab(), gtk_label_new("Toolbars"));
	
	label = gtk_label_new("<i>Preview of the message window</i>");
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
	alignment = gtk_alignment_new(0.0f, 0.0f, 0.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 5, 0, 0, 0);
	gtk_container_add(GTK_CONTAINER(alignment), label);
	
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), notebook, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), previewNotebook, TRUE, TRUE, 0);
	
	return vbox;
}


	
GtkWidget *optionsWindowItem_msgWindow::createPreview()
{
	GtkWidget	*paned,
				*vbox1,
				*vbox2,
				*frame1,
				*frame2;
	GtkTextBuffer	*buf;
	GtkTextTagTable	*textTags;
	GtkTextTag		*atag;
	
	// create the toolbar containers
	localbar.container = gtk_event_box_new();
	remotebar.container = gtk_event_box_new();
	buttonbar.container = gtk_event_box_new();
	
	// create the preview text views
	frame1 = u_createTextView(&remotePreviewView);
	gtk_widget_set_size_request(frame1, -1, 20);
	
	// we need to copy the contents of the text tag table, because the tags
	// are shared by all open windows. Changing a tag for the preview would
	// change every tag before clicking the "apply" button
	textTags = gtk_text_tag_table_new();
	atag = gtk_text_tag_new("HL");
	g_object_set(atag, 	"weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_tag_table_add(textTags, atag);		
	
	atag = gtk_text_tag_new("newline");
	g_object_set(atag, 	"size", 3*PANGO_SCALE, NULL);
	gtk_text_tag_table_add(textTags, atag);	

	atag = gtk_text_tag_new("timestamp");
	g_object_set(atag,  "size", 8*PANGO_SCALE,
						"weight", PANGO_WEIGHT_LIGHT,
						NULL);
	gtk_text_tag_table_add(textTags, atag);	
	
	atag = gtk_text_tag_new("incoming");
	g_object_set(atag, "foreground-gdk", &incomingCol,
					   NULL);
	gtk_text_tag_table_add(textTags, atag);
	
	atag = gtk_text_tag_new("outgoing");
	g_object_set(atag, "foreground-gdk", &outgoingCol, 
						NULL);
	gtk_text_tag_table_add(textTags, atag);	


	
	buf = gtk_text_buffer_new(textTags);
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(remotePreviewView), buf);
	g_object_unref(buf);
	g_object_unref(textTags);
	
	refreshRemoteViewPreview();
	
	frame2 = u_createTextView(&localPreviewView);
	gtk_widget_set_size_request(frame2, -1, 20);
	
	
	// create the box for the menu + entry
	vbox1 = gtk_vbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox1), localbar.container, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), frame2, TRUE, TRUE, 0);	
	
	
	// create divider
	paned = gtk_vpaned_new();
		
	gtk_paned_pack1(GTK_PANED(paned), frame1, TRUE, TRUE);
	gtk_paned_pack2(GTK_PANED(paned), vbox1, FALSE, TRUE);
		
	
	// create the main vertical box
	vbox2 = gtk_vbox_new(FALSE, 0);	

	gtk_box_pack_start(GTK_BOX(vbox2), remotebar.container, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), paned, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), buttonbar.container, FALSE, FALSE, 0);
	
	return vbox2;
}
	

void optionsWindowItem_msgWindow::createPreviewToolbar(editToolbarElements* tb)
{
	conversationWindow	*cwnd;
	GList				*children;
	GtkWidget			*bar,
						*alignment;
	
	children = gtk_container_get_children(GTK_CONTAINER(tb->container));
	if (children)				
		gtk_widget_destroy(GTK_WIDGET(children->data));
	g_list_free(children);
	
	if (tb->visible)
	{
		cwnd = new conversationWindow();
		bar = cwnd->createToolbar(tb->rules, tb->spacing, TRUE, &tb->list);
		delete cwnd;
		
		if (tb == &remotebar)
		{
			alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
			gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 5, 0, 0);
			gtk_container_add(GTK_CONTAINER(alignment), bar);

			gtk_container_add(GTK_CONTAINER(tb->container), alignment);			
		} else
		if (tb == &localbar)
		{
			gtk_container_set_border_width(GTK_CONTAINER(bar), 2);
			
			gtk_container_add(GTK_CONTAINER(tb->container), bar);
		} else
		{
			alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
			gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 3,0,0,0);
			gtk_container_add(GTK_CONTAINER(alignment), bar);

			gtk_container_add(GTK_CONTAINER(tb->container), alignment);			
		}
	}
}




GtkWidget* optionsWindowItem_msgWindow::createColorsTab()
{
	GtkWidget	*vbox,
				*mainVBox,
				*label,
				*label2,
				*alignment,
				*alignment2,
				*hbox2,
				*hbox3,
				*hbox4,
				*table,
				*msgHintsButton,
				*msgRefreshButton;
	
	OW_CREATE_MAINVBOX(mainVBox);
		
	
	OW_CREATE_SECTION(tr("<b>Colors</b>"), vbox);
	
	// color for sent messages
	label2 = gtk_label_new(tr("Color for sent messages"));
	sentColorButton = gtk_color_button_new();
	gtk_color_button_set_color(GTK_COLOR_BUTTON(sentColorButton), &outgoingCol);
	g_signal_connect(sentColorButton, "color-set", G_CALLBACK(this->cb_msgColorSelected), this);
	hbox2 = gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox2), sentColorButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), label2, FALSE, FALSE, 0);
	alignment2 = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment2), 5, 0, 0, 0);
	gtk_container_add(GTK_CONTAINER(alignment2), hbox2);
	
	// color for received messages
	label2 = gtk_label_new(tr("Color for received messages"));
	recColorButton = gtk_color_button_new();
	gtk_color_button_set_color(GTK_COLOR_BUTTON(recColorButton), &incomingCol);
	g_signal_connect(recColorButton, "color-set", G_CALLBACK(this->cb_msgColorSelected), this);

	hbox2 = gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox2), recColorButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), label2, FALSE, FALSE, 0);
	
	
	// color for received messages
	label2 = gtk_label_new(tr("Color for received messages (in another tab)"));
	msgColorButton = gtk_color_button_new();
	gtk_color_button_set_color(GTK_COLOR_BUTTON(msgColorButton), &msgCol);

	hbox3 = gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox3), msgColorButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox3), label2, FALSE, FALSE, 0);


	// color when typing
	label2 = gtk_label_new(tr("Tab color when a user is typing"));
	typeColorButton = gtk_color_button_new();
	gtk_color_button_set_color(GTK_COLOR_BUTTON(typeColorButton), &typeCol);

	hbox4 = gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox4), typeColorButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox4), label2, FALSE, FALSE, 0);


	
	// don't apply colors button
	dontApplyColorsButton = gtk_check_button_new_with_mnemonic(tr("Do not apply colors to the messages"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dontApplyColorsButton), dontApplyColors);
	g_signal_connect_swapped(dontApplyColorsButton, "clicked", G_CALLBACK(this->cb_dontApplyColorsButtonClicked), this);
	
		
	gtk_box_pack_start(GTK_BOX(vbox), alignment2, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox3, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox4, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), dontApplyColorsButton, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	
	
	OW_CREATE_SECTION_T(tr("<b>Font / Format</b>"), table, 4, 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 2);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	
	// message format
	label2 = gtk_label_new(tr("Message format:"));
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.5f);
	msgFormatEntry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(msgFormatEntry), messageTag);
	msgHintsButton = u_createStockImageButton(GTK_STOCK_HELP);
	g_signal_connect_swapped(msgHintsButton, "clicked", G_CALLBACK(this->cb_msgHelpButtonClicked), this);
	msgRefreshButton = u_createStockImageButton(GTK_STOCK_REFRESH);
	g_signal_connect_swapped(msgRefreshButton, "clicked", G_CALLBACK(this->cb_msgRefreshButtonClicked), this);
		
	gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), msgFormatEntry, 1, 2, 0, 1);
	gtk_table_attach(GTK_TABLE(table), msgRefreshButton, 2, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach(GTK_TABLE(table), msgHintsButton, 3, 4, 0, 1, GTK_FILL, GTK_FILL, 0, 0);


	// font
	label2 = gtk_label_new(tr("Message font:"));
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.5f);		

	msgFontButton = gtk_font_button_new();

	gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), msgFontButton, 1, 4, 1, 2);	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	return mainVBox;
}



void optionsWindowItem_msgWindow::refreshRemoteViewPreview()
{
	GtkTextIter	iter1,
				iter2;
	GtkTextBuffer	*buf;

	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(remotePreviewView));
	
	// clear the old content
	gtk_text_buffer_get_bounds(buf, &iter1, &iter2);
	gtk_text_buffer_delete(buf, &iter1, &iter2);
	
	refreshMessageFormatList(messageTag, "H-Jay", "Homer", "Simpson", buf, "outgoing");
	gtk_text_buffer_get_end_iter(buf, &iter2);
	gtk_text_buffer_insert_with_tags_by_name(buf, &iter2, tr("I know I should say something but I'm so unimaginative!\n"), -1, dontApplyColors? NULL : "outgoing", NULL);
	gtk_text_buffer_insert_with_tags_by_name(buf, &iter2, "\n", -1, "newline", dontApplyColors ? NULL: "outgoing", NULL);		
	
	refreshMessageFormatList(messageTag, "Mutch", "Marge", "Simpson", buf, "incoming");
	gtk_text_buffer_get_end_iter(buf, &iter2);
	gtk_text_buffer_insert_with_tags_by_name(buf, &iter2, tr("However ...!\n"), -1, dontApplyColors ? NULL: "incoming", NULL);
	gtk_text_buffer_insert_with_tags_by_name(buf, &iter2, "\n", -1, "newline",  dontApplyColors ? NULL: "incoming", NULL);		

	
}


void optionsWindowItem_msgWindow::refreshMessageFormatList(gchar* tag, gchar* alias, gchar* firstname, gchar* lastname, GtkTextBuffer* buf, gchar* dirTag)
{
	gchar	*lastPos,
			*currentPos,
			*toInsert,
			*currentTag;
	GtkTextIter	endIter;
	
	lastPos = tag;
	currentPos = tag;
	gtk_text_buffer_get_end_iter(buf, &endIter);
	
	while(currentPos = strchr(lastPos, '%'))
	{
		// insert non-token stuff
		if (lastPos != currentPos)
			gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, lastPos, currentPos - lastPos, "HL",  dirTag, NULL);

		toInsert = NULL;
		currentTag = "HL";
		
		// insert the appropriate tag
		switch(currentPos[1])
		{
			case 't': 
				toInsert = g_strdup("(12:00:00)");
				currentTag = "timestamp";
				break;
			case 'a': 
				toInsert = g_strdup(alias);
				break;
			case 'f':
				toInsert = g_strdup(firstname);
				break;
			case 'l':
				toInsert = g_strdup(lastname);
				break;
			case 'n':
				toInsert  = g_strdup_printf("%s %s", firstname, lastname);			
				break;
			case 'b':
				toInsert = g_strdup("\n");
				break;
			
			default:
				fprintf(stderr, "optionsWindowItem_msgWindow::refreshMessageFormatList: Unknown token '%c'\n", currentPos[1]);
				break;
		}
				
		gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, toInsert, -1, currentTag, dirTag, NULL);
		g_free(toInsert);		
		
		lastPos = currentPos+2;
	}
	
	// insert the rest
	gtk_text_buffer_insert_with_tags_by_name(buf, &endIter, lastPos, -1, "HL", dirTag, NULL);
}



GtkWidget* optionsWindowItem_msgWindow::createGeneralTab()
{
	GtkWidget	*label,
				*label2,
				*label3,
				*alignment,
				*vbox,
				*hbox,
				*mainVBox;
	charsetType	*charset;
	guint		i;
	gchar		*tmp;
	
	OW_CREATE_MAINVBOX(mainVBox)
	
	OW_CREATE_SECTION(tr("<b>Sending / Receiving</b>"), vbox);
	
	sendTypingCheckbox = gtk_check_button_new_with_mnemonic(tr("Inform user when you start _typing"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sendTypingCheckbox), sendTyping);

	showSmileysCheckbox = gtk_check_button_new_with_mnemonic(tr("Show _smiley images instead of tokens ;)"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showSmileysCheckbox), showSmileys);
	
	showURIsCheckbox = gtk_check_button_new_with_mnemonic(tr("Look for _URIs in the text"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showURIsCheckbox), showURIs);
	
	gtk_box_pack_start(GTK_BOX(vbox), sendTypingCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), showSmileysCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), showURIsCheckbox, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);

	
	OW_CREATE_SECTION(tr("<b>Misc settings</b>"), vbox);
	
	tabbedChatCheckbox = gtk_check_button_new_with_mnemonic(tr("_Arrange chat windows in tabs"));
	gtk_widget_set_sensitive(tabbedChatCheckbox, FALSE);

	// history check box / spin button
	showHistoryCheckbox = gtk_check_button_new_with_mnemonic(tr("_List the last "));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showHistoryCheckbox), showHistory);
	
	label2 = gtk_label_new_with_mnemonic(tr(" _history entries when opening a chat window"));
	historyCountBox = gtk_spin_button_new_with_range(1, 20, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(historyCountBox), historyCount);
	
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), showHistoryCheckbox, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), historyCountBox, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);	
	
	
	gtk_box_pack_start(GTK_BOX(vbox), tabbedChatCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	
	
	// localization
	OW_CREATE_SECTION(tr("<b>Localization</b>"), vbox);
	
	label2 = gtk_label_new(tr("<small>Different countries use different encodings (a special number is sent for non-latin characters like \"à,ß,ä\"). Some clients do not send the encoding. Please select an encoding to use in this case.</small>"));
	gtk_label_set_line_wrap(GTK_LABEL(label2), TRUE);
	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);
	gtk_widget_set_size_request(label2, 400, -1);
	
	
	encodingComboBox = gtk_combo_box_new_text();
	charset = charset_getCharsetList();
	i = 0;
	
	while(charset[i].name)
	{
		tmp = g_strdup_printf("%s (%s)", charset[i].name, charset[i].id);
		gtk_combo_box_append_text(GTK_COMBO_BOX(encodingComboBox), tmp);
		if (!strcmp(charset[i].id, standardEncoding))
			gtk_combo_box_set_active(GTK_COMBO_BOX(encodingComboBox), i);
		g_free(tmp);
		
		i++;
	}
	
	label3 = gtk_label_new(tr("Default encoding:"));
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), label3, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), encodingComboBox, TRUE, TRUE, 0);
	
	
	gtk_box_pack_start(GTK_BOX(vbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	return mainVBox;
}


GtkWidget* optionsWindowItem_msgWindow::createToolbarsTab()
{
	GtkWidget			*hbox,
						*alignment,
						*mainHBox,
						*scrolled,
						*scrolledUsed,
						*label1,
						*label2,
						*label3,
						*vbox1,
						*vbox2,
						*vbox3;

	guint				i;
	GtkTreeIter			iter;
	gchar				*numStr;
	GtkCellRenderer		*renderer;
	GtkTreeViewColumn	*column;
	GList				*usedElements,
						*search;
	gboolean			inUse;
	
	
	label1 = gtk_label_new(tr("<b>Possible Elements</b>"));
	gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label1), 0.0f, 0.0f);
	
	toolsStore = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_BOOLEAN);
	
	usedElements = g_list_copy(localbar.list);
	usedElements = g_list_concat(usedElements, g_list_copy(remotebar.list));
	usedElements = g_list_concat(usedElements, g_list_copy(buttonbar.list));
	
	// set up the tools list
	i=0;
	while(toolbarElements[i].title)
	{
		inUse = FALSE;
		
		// look whether this element is already in our list (and don't show it if it's a one time element)
		if (toolbarElements[i].oneTime)
		{
			search = usedElements;
			while(search)
			{
				if (GPOINTER_TO_UINT(search->data) == toolbarElements[i].num)
				{
					inUse = TRUE;
					break;
				}
				search = search->next;
			}
		}
		
		gtk_list_store_append(toolsStore, &iter);
		
		if (strlen(toolbarElements[i].title)>1)
		{
			numStr = g_strdup_printf("%d", toolbarElements[i].num);
			gtk_list_store_set(toolsStore, &iter,
							   0, numStr,
							   1, toolbarElements[i].title,
							   2, &toolbarElements[i],
							   3, inUse,
							   -1);
			g_free(numStr);
		}
		else
			gtk_list_store_set(toolsStore, &iter,
							   0, "",
							   1, "                 ",
							   2, NULL,
							   3, TRUE,
							   -1);
		i++;
	}
	
	g_list_free(usedElements);
	
	toolsView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(toolsStore));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(toolsView), FALSE);
	g_signal_connect_swapped(G_OBJECT(toolsView), "row-activated", G_CALLBACK(this->cb_addElementButtonClicked), this);

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("", renderer,
													  "text", 0,
													  NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(toolsView), column);
	
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("", renderer,
													  "text", 1,
													  "strikethrough", 3,
													  NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(toolsView), column);
	
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scrolled), toolsView);

	vbox1 = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox1), label1, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 0);




	label2 = gtk_label_new("<b>Toolbar to Edit</b>");
	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);
	
	editToolsSelection = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(editToolsSelection), tr("Remote Bar"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(editToolsSelection), tr("Local Bar"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(editToolsSelection), tr("Button Bar"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(editToolsSelection), 0);
	g_signal_connect_swapped(editToolsSelection, "changed", G_CALLBACK(this->cb_toolbarSelected), this);
		

	// create the stores containing the used elements
	localbar.store = createToolbarStore(localbar.list);
	remotebar.store = createToolbarStore(remotebar.list);
	buttonbar.store = createToolbarStore(buttonbar.list);

	
	// set up the view of used elements
	editToolsView = gtk_tree_view_new();
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(editToolsView), FALSE);
	g_signal_connect_swapped(G_OBJECT(editToolsView), "row-activated", G_CALLBACK(this->cb_removeElementButtonClicked), this);
	
	renderer = gtk_cell_renderer_text_new();
	editToolsColumn = gtk_tree_view_column_new_with_attributes("", renderer,
															   "text", 0,
															   NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(editToolsView), editToolsColumn);
	
	scrolledUsed = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledUsed), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledUsed), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scrolledUsed), editToolsView);
			
			
	// create the show toolbar checkbox / spacing spin button
	showToolbarCheckbox = gtk_check_button_new_with_mnemonic(tr("Toolbar is visible"));
	g_signal_connect_swapped(showToolbarCheckbox, "clicked", G_CALLBACK(this->cb_showToolbarBoxClicked), this);

	toolbarSpacingSpinButton = gtk_spin_button_new_with_range(0, 20, 1);
	label3 = gtk_label_new(tr("Element spacing: "));
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), label3, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), toolbarSpacingSpinButton, FALSE, TRUE, 0);
			
			
	vbox2 = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox2), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox2), editToolsSelection, FALSE, TRUE, 0);	
	gtk_box_pack_start(GTK_BOX(vbox2), showToolbarCheckbox, FALSE, TRUE, 0);
	// gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, TRUE, 0);	
	gtk_box_pack_start(GTK_BOX(vbox2), scrolledUsed, TRUE, TRUE, 0);
			
			
	// create the add / remove buttons
	addElementButton = u_createStockImageButton(GTK_STOCK_GO_FORWARD);
	g_signal_connect_swapped(addElementButton, "clicked", G_CALLBACK(this->cb_addElementButtonClicked), this);
	removeElementButton = u_createStockImageButton(GTK_STOCK_GO_BACK);
	g_signal_connect_swapped(removeElementButton, "clicked", G_CALLBACK(this->cb_removeElementButtonClicked), this);

	vbox3 = gtk_vbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(vbox3), addElementButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox3), removeElementButton, FALSE, FALSE, 0);
	alignment = gtk_alignment_new(0.0f, 0.5f, 0.0f, 0.0f);
	gtk_container_add(GTK_CONTAINER(alignment), vbox3);
			
			
		
	// add everything into a hbox
	mainHBox = gtk_hbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(mainHBox), 10);
	gtk_box_pack_start(GTK_BOX(mainHBox), vbox1, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(mainHBox), alignment, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(mainHBox), vbox2, TRUE, TRUE, 0);
	
	
	cb_toolbarSelected(this);
	
	return mainHBox;	
}


GtkListStore *optionsWindowItem_msgWindow::createToolbarStore(GList *el)
{
	GtkListStore	*store;
	guint			i;
	GList			*search;
	GtkTreeIter		iter;
	toolbarElement	*currentElement;
	
	store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
	
	currentElement = NULL;
	search = el;

	while(search)
	{
		i = 0;
		currentElement = NULL;
		while(toolbarElements[i].title)
		{
			if (GPOINTER_TO_UINT(search->data) == toolbarElements[i].num)
			{
				currentElement = &toolbarElements[i];
				break;
			}
			
			i++;
		}
		
		if (currentElement)
		{
			gtk_list_store_append(store, &iter);
			gtk_list_store_set(store, &iter,
						  	 0, currentElement->title,
						  	 1, currentElement,
						  	 -1);
		}
		
		search = search->next;
	}
	
	return store;
}


gchar* optionsWindowItem_msgWindow::refreshElementList(GList *elements)
{
	GString	*newList;
	GList	*search;
	
	newList = g_string_new("");
	
	search = elements;
	while(search)
	{
		g_string_append_printf(newList, "%d,", GPOINTER_TO_UINT(search->data));
		search = search->next;
	}
	
	newList = g_string_set_size(newList, strlen(newList->str)-1);
	
	return g_string_free(newList, FALSE);
}



gboolean optionsWindowItem_msgWindow::applyChanges()
{
	charsetType	*encoding;

	encoding = charset_getCharsetList();
			
	// set all conversation changes and let the connected windows be informed	
	settings_getSettings()->setProperties(TRUE, "conversations",
										  "remoteToolbarRules", remotebar.rules,
										  "localToolbarRules", localbar.rules,
										  "buttonbarRules", buttonbar.rules,
										  "remoteToolbarSpacing", remotebar.spacing,
										  "localToolbarSpacing", localbar.spacing,
										  "buttonbarSpacing", buttonbar.spacing,
										  "showRemoteToolbar", remotebar.visible,
										  "showLocalToolbar", localbar.visible,
										  "showButtonbar", buttonbar.visible,
										  "messageTag", (gchar*)gtk_entry_get_text(GTK_ENTRY(msgFormatEntry)),
										  "dontApplyColors", dontApplyColors,
										  "standardEncoding", encoding[gtk_combo_box_get_active(GTK_COMBO_BOX(encodingComboBox))].id,
										  "sendTyping", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sendTypingCheckbox)),
										  "showSmileys", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showSmileysCheckbox)),
										  "showURIs", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showURIsCheckbox)),
										  "showHistory", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showHistoryCheckbox)),
										  "historyCount", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(historyCountBox)),
										  NULL);
	
	gtk_color_button_get_color(GTK_COLOR_BUTTON(msgColorButton), &msgCol);
	gtk_color_button_get_color(GTK_COLOR_BUTTON(typeColorButton), &typeCol);
	
	settings_getSettings()->setProperties(TRUE, "appearance",
										  "colorReceive", &incomingCol,
										  "colorSend", &outgoingCol,
										  "tabMessageColor", &msgCol,
										  "tabTypingColor", &typeCol,
										  NULL);										  
		
	return TRUE;
}



void optionsWindowItem_msgWindow::cb_toolbarSelected(optionsWindowItem_msgWindow* self)
{
	guint	active;
	
	active = gtk_combo_box_get_active(GTK_COMBO_BOX(self->editToolsSelection));
	
	switch(active)
	{
		case 0: self->currentbar = &(self->remotebar); break;
		case 1: self->currentbar = &(self->localbar); break;
		case 2: self->currentbar = &(self->buttonbar); break;		
	}
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(self->editToolsView), GTK_TREE_MODEL(self->currentbar->store));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->showToolbarCheckbox), self->currentbar->visible);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(self->toolbarSpacingSpinButton), (gulong)self->currentbar->spacing);	
}


void optionsWindowItem_msgWindow::cb_addElementButtonClicked(optionsWindowItem_msgWindow* self)
{
	GtkTreePath		*path;
	toolbarElement	*element;
	GtkTreeIter		iter;
	gboolean		inUse;
	guint			pos;
	
	// get the element that should be added
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(self->toolsView), &path, NULL);
	
	if (!path)
		return;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(self->toolsStore), &iter, path);

	gtk_tree_model_get(GTK_TREE_MODEL(self->toolsStore), &iter,
					   2, &element,
					   3, &inUse,
					   -1);	
	
	gtk_tree_path_free(path);
	
	// is this just a separator?
	if (!element)
		return;
	
	// look whether this element is already in use
	if (element->oneTime && inUse)
	{
		u_showAlertMessage(tr("Could not add element"), tr("The element you selected is cancelled. This means that it can be only used one time and is already in use. Please remove the element from one of the other toolbars in order to insert it here"), GTK_STOCK_DIALOG_WARNING);
		return;
	}
	
	// mark the element as used if its a one-time element
	if (element->oneTime)
		gtk_list_store_set(self->toolsStore, &iter,
						   3, TRUE,
						   -1);
	
	
	// get the position of the cursor in the toolbar store	
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(self->editToolsView), &path, NULL);
	
	if (path)
	{
		self->currentbar->list = g_list_insert(self->currentbar->list, GUINT_TO_POINTER(element->num), gtk_tree_path_get_indices(path)[0]+1);
		pos = gtk_tree_path_get_indices(path)[0];
		gtk_tree_model_get_iter(GTK_TREE_MODEL(self->currentbar->store), &iter, path);
		gtk_list_store_insert_after(self->currentbar->store, &iter, &iter);
		gtk_tree_path_free(path);		
	} else
	{
		self->currentbar->list = g_list_prepend(self->currentbar->list, GUINT_TO_POINTER(element->num));
		gtk_list_store_prepend(self->currentbar->store, &iter);	
	}
	
	gtk_list_store_set(self->currentbar->store, &iter, 0, element->title, 1, element, -1);
	
	// recreate the preview
	g_free(self->currentbar->rules);
	self->currentbar->rules = self->refreshElementList(self->currentbar->list);
	self->createPreviewToolbar(self->currentbar);
	gtk_widget_show_all(self->currentbar->container);
	
	// highlight the element
	path = gtk_tree_model_get_path(GTK_TREE_MODEL(self->currentbar->store), &iter);
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(self->editToolsView), path, self->editToolsColumn, FALSE);
	gtk_tree_path_free(path);
}



void optionsWindowItem_msgWindow::cb_removeElementButtonClicked(optionsWindowItem_msgWindow* self)
{
	GtkTreePath		*path;
	GtkTreeIter 	iter,
					iter2;
	toolbarElement	*el1,
					*el2;
	
	// get the element that should be removed
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(self->editToolsView), &path, NULL);
	
	if (!path)
		return;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(self->currentbar->store), &iter, path);
	
	gtk_tree_path_free(path);

	
	gtk_tree_model_get(GTK_TREE_MODEL(self->currentbar->store), &iter, 1, &el1, -1);
	
	// check whether its a one time element and activate the element in the
	// store of available elements	
	if (el1->oneTime)
	{
		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(self->toolsStore), &iter2);

		do
		{
			gtk_tree_model_get(GTK_TREE_MODEL(self->toolsStore), &iter2, 2, &el2, -1);
			if (el2 && el2->num == el1->num)
			{
				gtk_list_store_set(self->toolsStore, &iter2, 3, FALSE, -1);
				break;
			}
		} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(self->toolsStore), &iter2));
	}

	
	// remove the element from the store ...
	gtk_list_store_remove(self->currentbar->store, &iter);
	
	// ... and from the list
	self->currentbar->list = g_list_remove(self->currentbar->list, GUINT_TO_POINTER(el1->num));
	
	// recreate the preview
	g_free(self->currentbar->rules);
	self->currentbar->rules = self->refreshElementList(self->currentbar->list);
	self->createPreviewToolbar(self->currentbar);
	gtk_widget_show_all(self->currentbar->container);
}



void optionsWindowItem_msgWindow::cb_showToolbarBoxClicked(optionsWindowItem_msgWindow* self)
{
	self->currentbar->visible = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->showToolbarCheckbox));
	self->createPreviewToolbar(self->currentbar);
	gtk_widget_show_all(self->currentbar->container);	
}



void optionsWindowItem_msgWindow::cb_msgColorSelected(GtkColorButton* but, optionsWindowItem_msgWindow* self)
{
	GtkTextTag	*tag;
	GtkTextBuffer	*buf;
	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->remotePreviewView));

	if (but == GTK_COLOR_BUTTON(self->recColorButton))
		gtk_color_button_get_color(but, &self->incomingCol);
	else
		gtk_color_button_get_color(but, &self->outgoingCol);
	
	tag = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), "incoming");
	g_object_set(tag, "foreground-gdk", &self->incomingCol, NULL);

	tag = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(buf), "outgoing");
	g_object_set(tag, "foreground-gdk", &self->outgoingCol, NULL);
	
}


void optionsWindowItem_msgWindow::cb_msgRefreshButtonClicked(optionsWindowItem_msgWindow* self)
{
	g_free(self->messageTag);
	self->messageTag = g_strdup(gtk_entry_get_text(GTK_ENTRY(self->msgFormatEntry)));
	self->refreshRemoteViewPreview();
}



void optionsWindowItem_msgWindow::cb_msgHelpButtonClicked(optionsWindowItem_msgWindow* self)
{
	GtkWidget 	*dialog, 
   				*label,
				*hbox,
				*icon;
										 
	label = gtk_label_new(tr("In the entry field you can set the format of the "
							 "string that appears at the beginning of the messages "
							 "that are displayed in the conversation window. It uses "
							 "tokens, that means short letters are replaced by certain information.\n\n"
							 "Allowed tokens:\n"
							 "   %a		Alias\n"
							 "   %f		first name\n"
							 "   %l		last name\n"
							 "   %n		first and last name\n"
							 "   %t		current time\n"
							 "   %b		line break\n\n"
							 "Something like \"%t from %n:%b\" would give\n"
							 "\"(12:00:00) from First Last:\nHello, this is a message!\"\n\n"
							 "Any letters that are not tokens will simply be copied to the text"));
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_widget_set_size_request(label, 300, -1);
	
	icon = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
	gtk_misc_set_alignment(GTK_MISC(icon), 0.5f, 0.0f);
   
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
	gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);

   


	dialog = gtk_dialog_new_with_buttons (	tr("Hints for the message beginning"),
											GTK_WINDOW(self->window),
											GTK_DIALOG_DESTROY_WITH_PARENT,
											GTK_STOCK_OK,
											GTK_RESPONSE_NONE,
											NULL);
								
	g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);								


	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), hbox);

	gtk_widget_show_all(dialog);
	
}


void optionsWindowItem_msgWindow::cb_dontApplyColorsButtonClicked(optionsWindowItem_msgWindow* self)
{
	self->dontApplyColors = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->dontApplyColorsButton));
	
	self->refreshRemoteViewPreview();
}




// -----------------------------------------------------------------------------

optionsWindowItem_extensions::optionsWindowItem_extensions()
{
	titleImage = NULL;
	titleString = tr_NOOP("Applications");	
}


optionsWindowItem_extensions::~optionsWindowItem_extensions()
{
	
}


// -----------------------------------------------------------------------------

optionsWindowItem_onEvent::optionsWindowItem_onEvent()
{
	titleImage = NULL;
	titleString = tr_NOOP("On Events");	
}


optionsWindowItem_onEvent::~optionsWindowItem_onEvent()
{
	
}


#define OW_CREATE_ONEVENT_PARAM(w, cap, row, pt)				\
	label2 = gtk_label_new(tr(cap));							\
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);		\
	w = gtk_entry_new();										\
	gtk_entry_set_text(GTK_ENTRY(w), oem->Parameter(pt));		\
	gtk_table_attach(GTK_TABLE(table), label2, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0); \
	gtk_table_attach_defaults(GTK_TABLE(table), w, 1, 2, row, row+1);
	



GtkWidget	*optionsWindowItem_onEvent::createTab()
{
	GtkWidget	*mainVBox,
				*vbox,
				*label,
				*label2,
				*alignment,
				*table;
	COnEventManager	*oem;				// FIXME: wrap this
	
	oem = getLicqDaemon()->OnEventManager();
	
	OW_CREATE_MAINVBOX(mainVBox);
	
	
	// general
	OW_CREATE_SECTION(tr("<b>General</b>"), vbox);
	
	enableCheckbox = gtk_check_button_new_with_mnemonic(tr("Enable execution of a command on events"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enableCheckbox), oem->CommandType() != ON_EVENT_IGNORE);
	g_signal_connect_swapped(enableCheckbox, "toggled", G_CALLBACK(this->cb_enableButtonToggled), this);

	oem->Lock();

	label2 = gtk_label_new(tr("Command:"));
	commandEntry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(commandEntry), oem->Command());
	generalBox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(generalBox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(generalBox), commandEntry, TRUE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(vbox), enableCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), generalBox, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	
	// parameters
	OW_CREATE_SECTION_T(tr("<b>Command parameters for events</b>"), table, 2, 7);
	
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
	
	OW_CREATE_ONEVENT_PARAM(paramMsgEntry, "Message:", 0, ON_EVENT_MSG);
	OW_CREATE_ONEVENT_PARAM(paramURLEntry, "URL:", 1, ON_EVENT_URL);
	OW_CREATE_ONEVENT_PARAM(paramChatEntry, "Chat Request:", 2, ON_EVENT_CHAT);
	OW_CREATE_ONEVENT_PARAM(paramFTEntry, "File Transfer:", 3, ON_EVENT_FILE);
	OW_CREATE_ONEVENT_PARAM(paramONEntry, "Online Notify:", 4, ON_EVENT_NOTIFY);
	OW_CREATE_ONEVENT_PARAM(paramSysMsgEntry, "System Message:", 5, ON_EVENT_SYSMSG);
	OW_CREATE_ONEVENT_PARAM(paramMsgSentEntry, "Sent Messages:", 6, ON_EVENT_MSGSENT);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	paramBox = alignment;
	 
	 
	 // also execute
	OW_CREATE_SECTION_T(tr("<b>Also accept execution</b>"), table, 2, 3);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	
	acceptAwayCheckbox = gtk_check_button_new_with_mnemonic(tr("in Away"));
	gtk_widget_set_sensitive(acceptAwayCheckbox, FALSE);
	acceptNACheckbox = gtk_check_button_new_with_mnemonic(tr("in Not Available"));
	gtk_widget_set_sensitive(acceptNACheckbox, FALSE);
	acceptOccCheckbox = gtk_check_button_new_with_mnemonic(tr("in Occupied"));
	gtk_widget_set_sensitive(acceptOccCheckbox, FALSE);
	acceptDnDCheckbox = gtk_check_button_new_with_mnemonic(tr("in Do Not Disturb"));
	gtk_widget_set_sensitive(acceptDnDCheckbox, FALSE);
	acceptLogonCheckbox = gtk_check_button_new_with_mnemonic(tr("when contact list user status is updated while logging on"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(acceptLogonCheckbox), getLicqDaemon()->AlwaysOnlineNotify());
	
	gtk_table_attach_defaults(GTK_TABLE(table), acceptAwayCheckbox, 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), acceptNACheckbox, 1, 2, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), acceptOccCheckbox, 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), acceptDnDCheckbox, 1, 2, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), acceptLogonCheckbox, 0, 2, 2, 3);

	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	acceptBox = alignment;
	
	oem->Unlock();
	
	// call the callback, because it might gray out things that are created after
	// this widget was made.
	cb_enableButtonToggled(this);
	
	return mainVBox;
}


void optionsWindowItem_onEvent::cb_enableButtonToggled(optionsWindowItem_onEvent* self)
{
	gboolean toggled;
	
	toggled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->enableCheckbox));
	
	gtk_widget_set_sensitive(self->generalBox, toggled);
	gtk_widget_set_sensitive(self->paramBox, toggled);
	gtk_widget_set_sensitive(self->acceptBox, toggled);
}



gboolean optionsWindowItem_onEvent::applyChanges()
{
	COnEventManager	*oem;
	const gchar		*paramMsg,
					*paramURL,
					*paramChat,
					*paramFT,
					*paramON,
					*paramSysMsg,
					*paramMsgSent;
	

	paramMsg = gtk_entry_get_text(GTK_ENTRY(paramMsgEntry));
	paramURL = gtk_entry_get_text(GTK_ENTRY(paramURLEntry));
	paramChat = gtk_entry_get_text(GTK_ENTRY(paramChatEntry));
	paramFT = gtk_entry_get_text(GTK_ENTRY(paramFTEntry));
	paramON = gtk_entry_get_text(GTK_ENTRY(paramONEntry));
	paramSysMsg = gtk_entry_get_text(GTK_ENTRY(paramSysMsgEntry));
	paramMsgSent = gtk_entry_get_text(GTK_ENTRY(paramMsgSentEntry));
	
	const gchar* parameters[8] =
	{
		paramMsg, paramURL, paramChat, paramFT, paramON, paramSysMsg, paramMsgSent
	};
	
	
	oem = getLicqDaemon()->OnEventManager();
	
	// set whether execution is enabled / disabled
	oem->SetCommandType(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enableCheckbox))? ON_EVENT_RUN : ON_EVENT_IGNORE);
	
	// set the parameters
	oem->SetParameters(gtk_entry_get_text(GTK_ENTRY(commandEntry)), parameters);
	
	// set the rest
	getLicqDaemon()->SetAlwaysOnlineNotify(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(acceptLogonCheckbox)));
	
	return TRUE;	
}

// -----------------------------------------------------------------------------


optionsWindowItem_connections::optionsWindowItem_connections()
{
	titleImage = NULL;
	titleString = tr_NOOP("Connections");	
	
	serverTabs = NULL;
}


optionsWindowItem_connections::~optionsWindowItem_connections()
{
	IO_getGeneralSource()->removeCallback((IMEventCallback)this->cb_eventCallback);
	
	g_list_foreach(serverTabs, (GFunc)g_free, NULL);
}



GtkWidget	*optionsWindowItem_connections::createTab()
{
	GtkWidget	*vbox,
				*alignment,
				*alignment2,
				*label,
				*label2,
				*label3,
				*mainVBox,
				*hbox,
				*table;
				
	
	OW_CREATE_MAINVBOX(mainVBox);
	
	// firewall
	OW_CREATE_SECTION(tr("<b>Firewall</b>"), vbox);
	fwEnableCheckbox = gtk_check_button_new_with_mnemonic(tr("I am behind a firewall"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fwEnableCheckbox), getLicqDaemon()->Firewall());
	g_signal_connect_swapped(fwEnableCheckbox, "toggled", G_CALLBACK(this->cb_fwEnableButtonClicked), this);
	
	fwDirectConnectionsCheckbox = gtk_check_button_new_with_mnemonic(tr("I can receive direct connections"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fwDirectConnectionsCheckbox), getLicqDaemon()->TCPEnabled());
	
	fwPortsFromSpinButton = gtk_spin_button_new_with_range(0, 0xFFFF, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(fwPortsFromSpinButton), (gushort)getLicqDaemon()->TCPPortsLow());

	fwPortsToSpinButton = gtk_spin_button_new_with_range(0, 0xFFFF, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(fwPortsToSpinButton), (gushort)getLicqDaemon()->TCPPortsHigh());
	
	label2 = gtk_label_new(tr("Use port range from "));
	label3 = gtk_label_new(tr(" to "));
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), fwPortsFromSpinButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label3, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), fwPortsToSpinButton, FALSE, FALSE, 0);
	
	label2 = gtk_label_new(tr("<small>&quot;0&quot; means that licq will decide which ports to use</small>"));
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);
	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
	
	// we need another vbox here to disable it if neccessary
	fwBox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(fwBox), fwDirectConnectionsCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(fwBox), hbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(fwBox), label2, FALSE, TRUE, 0);
	alignment2 = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment2), 0, 0, 20, 0);
	gtk_container_add(GTK_CONTAINER(alignment2), fwBox);
	
	gtk_box_pack_start(GTK_BOX(vbox), fwEnableCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), alignment2, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	
	// proxy
	OW_CREATE_SECTION(tr("<b>Proxy</b>"), vbox);

	pxUseCheckbox = gtk_check_button_new_with_mnemonic(tr("Use HTTPS proxy server"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxUseCheckbox), getLicqDaemon()->ProxyEnabled());
	g_signal_connect_swapped(pxUseCheckbox, "toggled", G_CALLBACK(this->cb_pxUseButtonClicked), this);

	pxUseAuthorizationCheckbox = gtk_check_button_new_with_mnemonic(tr("Use authorization"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxUseAuthorizationCheckbox), getLicqDaemon()->ProxyAuthEnabled());
	g_signal_connect_swapped(pxUseAuthorizationCheckbox, "toggled", G_CALLBACK(this->cb_useAuthorizationButtonClicked), this);

	label2 = gtk_label_new(tr("Proxy server:"));
	label3 = gtk_label_new(tr(" Port:"));
	pxServerEntry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(pxServerEntry), getLicqDaemon()->ProxyHost());
	
	pxPortSpinButton = gtk_spin_button_new_with_range(0, 0xFFFF, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pxPortSpinButton), (gushort)getLicqDaemon()->ProxyPort());
	
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), pxServerEntry, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label3, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), pxPortSpinButton, FALSE, FALSE, 0);
	px1Box = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(px1Box), 0, 0, 20, 0);
	gtk_container_add(GTK_CONTAINER(px1Box), hbox);
	
	label2 = gtk_label_new(tr("Username:"));
	label3 = gtk_label_new(tr("Password:"));
	pxUsernameEntry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(pxUsernameEntry), getLicqDaemon()->ProxyLogin());
	
	pxPasswordEntry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(pxPasswordEntry), getLicqDaemon()->ProxyPasswd());
	
	table = gtk_table_new(2, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
	gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), pxUsernameEntry, 1, 2, 0, 1);
	gtk_table_attach(GTK_TABLE(table), label3, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), pxPasswordEntry, 1, 2, 1, 2);
	px2Box = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(px2Box), 0, 0, 20, 0);
	gtk_container_add(GTK_CONTAINER(px2Box), table);


	gtk_box_pack_start(GTK_BOX(vbox), pxUseCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), px1Box, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), pxUseAuthorizationCheckbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), px2Box, FALSE, TRUE, 0);
	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);


	// keep connection
	OW_CREATE_SECTION(tr("<b>Keep connection</b>"), vbox);
	kcIDClashCheckbox = gtk_check_button_new_with_mnemonic(tr("Keep the connection after connection clash"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(kcIDClashCheckbox), getLicqDaemon()->ReconnectAfterUinClash());
	gtk_box_pack_start(GTK_BOX(vbox), kcIDClashCheckbox, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);


	// connections
	OW_CREATE_SECTION(tr("<b>Server connections</b>"), vbox);
	gtk_box_pack_start(GTK_BOX(vbox), createPluginsConnectionTabs(), TRUE, TRUE, 0);
	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, TRUE, TRUE, 0);
	
	
	// add a callback to keep the connection tabs updated
	IO_getGeneralSource()->addCallback((IMEventCallback)this->cb_eventCallback, this);
	
	
	// update the sensibility of the widgets
	cb_fwEnableButtonClicked(this);
	cb_useAuthorizationButtonClicked(this);
	cb_pxUseButtonClicked(this);
	
	
	return mainVBox;
}


GtkWidget *optionsWindowItem_connections::createPluginsConnectionTabs()
{
	GList			*search;
	IMPluginDaemon	*proto;
	
	
	notebook = gtk_notebook_new();
		
	search = IO_getPluginsList();
	while(search)
	{
		proto = (IMPluginDaemon*)search->data;
		
		if (proto->isProtoPlugin && proto->isLoaded)
			appendPluginsConnectionTab(proto);
		
		search = search->next;
	}
	
	return notebook;
}


void optionsWindowItem_connections::appendPluginsConnectionTab(IMPluginDaemon *proto)
{
	GtkWidget		*vbox,
					*label1,
					*label2,
					*table;
	editServerConTab *con;	
	
	
	label1 = gtk_label_new(tr("Server:"));
	label2 = gtk_label_new(tr("Port:"));
			
	con = (editServerConTab*)g_malloc0(sizeof(editServerConTab));
	con->server = gtk_entry_new();
	if (proto->getServerName())
		gtk_entry_set_text(GTK_ENTRY(con->server), proto->getServerName());
	
	con->port = gtk_spin_button_new_with_range(0, 0xFFFF, 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(con->port), (gushort)proto->getServerPort());
	
	con->proto = proto;
			
	table = gtk_table_new(3, 2, FALSE);			
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
	gtk_table_attach(GTK_TABLE(table), label1, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), con->server, 1, 2, 0, 1);
	gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), con->port, 1, 2, 1, 2);
	
	if (!proto->getServerName())
	{
		label1 = gtk_label_new("<small>The server settings for this plugin cannot be set, because protocol plugin support for licq isn't finished yet. Sorry!</small>");
		gtk_label_set_line_wrap(GTK_LABEL(label1), TRUE);
		gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
		gtk_misc_set_alignment(GTK_MISC(label1), 0.0f, 0.0f);
		
		gtk_table_attach_defaults(GTK_TABLE(table), label1, 0, 2, 2, 3);
		
		gtk_widget_set_sensitive(con->server, FALSE);
		gtk_widget_set_sensitive(con->port, FALSE);
	}		
	
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, TRUE, 0);
	gtk_widget_show_all(vbox);
			
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, gtk_label_new(proto->name));
	g_object_set_data(G_OBJECT(vbox), "icqnd-options-con", proto);
	serverTabs = g_list_append(serverTabs, con);
}



gboolean optionsWindowItem_connections::applyChanges()
{
	GList				*search;
	editServerConTab	*serverTab;
	
	// firewall settings
	getLicqDaemon()->SetFirewall(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fwEnableCheckbox)));
	getLicqDaemon()->SetTCPEnabled(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fwDirectConnectionsCheckbox)));
	getLicqDaemon()->SetTCPPorts(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fwPortsFromSpinButton)),
							  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fwPortsToSpinButton)));
	
	
	// proxy settings
	getLicqDaemon()->SetProxyEnabled(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pxUseCheckbox)));
	getLicqDaemon()->SetProxyHost(gtk_entry_get_text(GTK_ENTRY(pxServerEntry)));
	getLicqDaemon()->SetProxyPort(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pxPortSpinButton)));
	getLicqDaemon()->SetProxyAuthEnabled(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pxUseAuthorizationCheckbox)));
	getLicqDaemon()->SetProxyLogin(gtk_entry_get_text(GTK_ENTRY(pxUsernameEntry)));
	getLicqDaemon()->SetProxyPasswd(gtk_entry_get_text(GTK_ENTRY(pxPasswordEntry)));

	// keep connection settings
	getLicqDaemon()->setReconnectAfterUinClash(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(kcIDClashCheckbox)));

	// server settings
	search = serverTabs;
	while(search)
	{
		serverTab = (editServerConTab*)search->data;
		serverTab->proto->setServerName((gchar*)gtk_entry_get_text(GTK_ENTRY(serverTab->server)));
		serverTab->proto->setServerPort(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(serverTab->port)));

		search = search->next;
	}
	
	return TRUE;	
}


void optionsWindowItem_connections::cb_fwEnableButtonClicked(optionsWindowItem_connections* self)
{
	gboolean toggled;
	
	toggled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->fwEnableCheckbox));
	
	gtk_widget_set_sensitive(self->fwBox, toggled);
}



void optionsWindowItem_connections::cb_pxUseButtonClicked(optionsWindowItem_connections* self)
{
	gboolean toggled;
	
	toggled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->pxUseCheckbox));
	
	gtk_widget_set_sensitive(self->px1Box, toggled);
	gtk_widget_set_sensitive(self->pxUseAuthorizationCheckbox, toggled);	

	if (toggled)
		cb_useAuthorizationButtonClicked(self);
	else
		gtk_widget_set_sensitive(self->px2Box, FALSE);
}



void optionsWindowItem_connections::cb_useAuthorizationButtonClicked(optionsWindowItem_connections* self)
{
	gboolean toggled;
	
	toggled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->pxUseAuthorizationCheckbox));	

	gtk_widget_set_sensitive(self->px2Box, toggled);	
}



gboolean optionsWindowItem_connections::cb_eventCallback(userEventType type, gint command, gint lastResult, gpointer info, optionsWindowItem_connections* self)
{
	GList				*search;
	gint				page;
	editServerConTab	*tab;
	
	switch(command)
	{
		case PC_NEWPLUGIN:
			self->appendPluginsConnectionTab((IMPluginDaemon*)info);
			break;
		
		case PC_REMOVEPLUGIN:
			page = 0;
			search = self->serverTabs;
			while(search)
			{
				tab = (editServerConTab*)search->data;
				
				if (tab->proto == info)
				{
					gtk_notebook_remove_page(GTK_NOTEBOOK(self->notebook), page);
					self->serverTabs = g_list_remove(self->serverTabs, tab);
					g_free(tab);
					break;
				}
				
				page++;
				search = search->next;
			}
			break;
			
	}
	
	return TRUE;	
}



// -----------------------------------------------------------------------------

optionsWindowItem_plugins::optionsWindowItem_plugins()
{
	titleImage = NULL;
	titleString = tr_NOOP("Plugins");	
}


optionsWindowItem_plugins::~optionsWindowItem_plugins()
{
	// remove us from the general signal source
	IO_getGeneralSource()->removeCallback((IMEventCallback)this->cb_eventCallback);

}



GtkWidget	*optionsWindowItem_plugins::createTab()
{
	GtkWidget	*label,
				*vbox,
				*alignment,
				*mainVBox;
	gchar		*id;
	
	GtkListStore	*store;
	GtkTreeIter		iter;
	IMPluginDaemon	*plugin;
	GList			*search;
	
	OW_CREATE_MAINVBOX(mainVBox);
	
	OW_CREATE_SECTION(tr("<b>Standard Plugins</b>"), vbox);
	
	gtk_box_pack_start(GTK_BOX(vbox), createPluginsList(&standardStore), TRUE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, TRUE, TRUE, 0);



	OW_CREATE_SECTION(tr("<b>Protocol Plugins</b>"), vbox);
	
	gtk_box_pack_start(GTK_BOX(vbox), createPluginsList(&protoStore), TRUE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, TRUE, TRUE, 0);
	
	
	// fill the plugin list
	search = IO_getPluginsList();
	while(search)
	{
		plugin = (IMPluginDaemon*)search->data;
		
		if (plugin->pluginID == LICQ_PPID)
		{
			search = search->next;
			continue;
		}
		
		if (plugin->isProtoPlugin)
			store = protoStore;
		else
			store = standardStore;
		
		if (plugin->isLoaded)
			id = g_strdup_printf("%d", plugin->id);
		else
			id = g_strdup("*");
		
		gtk_list_store_append(store, &iter);
		gtk_list_store_set(store, &iter,
						   0, id,
						   1, plugin->name,
						   2, plugin->version,
						   3, plugin->isLoaded,
						   4, plugin->isEnabled,
						   5, plugin->description,
						   6, plugin,
						   -1);
		
		g_free(id);
						   
		
		search = search->next;
	}
	
	// attach a listener to the general signal source
	IO_getGeneralSource()->addCallback((IMEventCallback)this->cb_eventCallback, this);
	
	return mainVBox;
}


GtkWidget	*optionsWindowItem_plugins::createPluginsList(GtkListStore **store)
{
	GtkCellRenderer		*renderer;
	GtkWidget			*view,
						*scrolled;
	GtkTreeViewColumn	*column;
	
	//								id			name			 version		loaded			enabled			descr			pointer
	*store = gtk_list_store_new(7, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_POINTER);

	view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(*store));

	// id column
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(tr("id"), renderer,
													  "text", 0, NULL);	
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);

	// name column
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(tr("Name"), renderer,
													  "text", 1, NULL);	
	gtk_tree_view_column_set_expand(column, TRUE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);

	// version column
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(tr("Version"), renderer,
													  "text", 2, NULL);	
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
	
	// loaded column
	renderer = gtk_cell_renderer_toggle_new();
	column = gtk_tree_view_column_new_with_attributes(tr("Loaded"), renderer,
													  "active", 3, NULL);	
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
	g_object_set_data(G_OBJECT(renderer), "icqnd-plugins-store", *store);
	g_signal_connect(renderer, "toggled", G_CALLBACK(this->cb_loadedCheckboxToggled), this);	

	
	// enabled column
	renderer = gtk_cell_renderer_toggle_new();
	column = gtk_tree_view_column_new_with_attributes(tr("Enabled"), renderer,
													  "active", 4, NULL);	
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
	g_object_set_data(G_OBJECT(renderer), "icqnd-plugins-store", *store);
	g_signal_connect(renderer, "toggled", G_CALLBACK(this->cb_enabledCheckboxToggled), this);	


	// description column
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(tr("Description"), renderer,
													  "text", 5, NULL);	
	gtk_tree_view_column_set_expand(column, TRUE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
	
	
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scrolled), view);
	
	return scrolled;
}


void optionsWindowItem_plugins::cb_loadedCheckboxToggled(GtkCellRenderer* t, gchar* pathString, optionsWindowItem_plugins* self)
{
	GtkTreePath*	path;
	GtkTreeIter		iter;
	IMPluginDaemon 	*plugin;
	gboolean		checked;
	GtkListStore	*store;
	
	path = gtk_tree_path_new_from_string(pathString);

	store = GTK_LIST_STORE(g_object_get_data(G_OBJECT(t), "icqnd-plugins-store"));
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
					   3, &checked,
					   6, &plugin,
					   -1);	
	
	checked = !checked;
	
	if (checked)
	{
		if(!plugin->loadPlugin())
			u_showAlertMessage(tr("Couldn't load plugin"), tr("The plugin couldn't be loaded. Please check the network log window for more information!"), GTK_STOCK_DIALOG_ERROR);
	} else
		plugin->unloadPlugin();

		
	gtk_tree_path_free(path);
}



void optionsWindowItem_plugins::cb_enabledCheckboxToggled(GtkCellRenderer* t, gchar* pathString, optionsWindowItem_plugins* self)
{
	GtkTreePath*	path;
	GtkTreeIter		iter;
	IMPluginDaemon 	*plugin;
	gboolean		checked;
	GtkListStore	*store;
	
	path = gtk_tree_path_new_from_string(pathString);

	store = GTK_LIST_STORE(g_object_get_data(G_OBJECT(t), "icqnd-plugins-store"));
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
					   4, &checked,
					   6, &plugin,
					   -1);	
			
	gtk_tree_path_free(path);
}



gboolean optionsWindowItem_plugins::cb_eventCallback(userEventType type, gint command, gint lastResult, gpointer info, optionsWindowItem_plugins* self)
{
	GtkTreeIter		iter;
	GtkListStore	*store;
	IMPluginDaemon	*plugin,
					*pluginToUpdate;
	gchar			*id;
	
	if (command != PC_NEWPLUGIN && command != PC_REMOVEPLUGIN)
		return TRUE;
	
	pluginToUpdate = (IMPluginDaemon*)info;
		
	if (pluginToUpdate->isProtoPlugin)
		store = self->protoStore;
	else
		store = self->standardStore;
	
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);

	do
	{
		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
						   6, &plugin, 
						   -1);
		
		if (plugin == pluginToUpdate)
		{
			if (plugin->isLoaded)
				id = g_strdup_printf("%d", plugin->id);
			else
				id = g_strdup("*");
	
			gtk_list_store_set(store, &iter,
					  		 	0, id,
					   			1, plugin->name,
					   			2, plugin->version,
					   			3, plugin->isLoaded,
					   			4, plugin->isEnabled,
					   			5, plugin->description,
					   			6, plugin,
					   			-1);
	
			g_free(id);
			break;				
		}
		
	} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter));	
	
	return TRUE;
}


// -----------------------------------------------------------------------------

optionsWindowItem_start::optionsWindowItem_start()
{
	titleImage = NULL;
	titleString = tr_NOOP("Startup");
}


optionsWindowItem_start::~optionsWindowItem_start()
{
	
}



// -----------------------------------------------------------------------------

optionsWindowItem_aresponse::optionsWindowItem_aresponse()
{
	titleImage = NULL;
	titleString = tr_NOOP("Auto Responses");	
}


optionsWindowItem_aresponse::~optionsWindowItem_aresponse()
{
	
}



GtkWidget	*optionsWindowItem_aresponse::createTab()
{
	GtkWidget				*mainVBox,
							*label,
							*label2,
							*label3,
							*label4,
							*label5,
							*alignment,
							*vbox,
							*hbox,
							*table,
							*paned,
							*frame,
							*scrolled;
	guint					i;
	GtkTreeIter 			iter;
	GtkCellRenderer			*renderer;
	GtkListStore			*store;
	IMAutoResponseManager	*man;
	GList					*contents,
							*names,
							*searchc,
							*searchn;
	
	GtkWidget				*autoAwaySpin,
							*autoNASpin,
							*autoOfflineSpin;

	struct		_responseCat
	{
		gchar	*name;
		gulong	status;
	} cathegories[] =
	{
		{tr_NOOP("Away"), BUDDY_STATUS_AWAY},
		{tr_NOOP("Not Available"), BUDDY_STATUS_NA},
		{tr_NOOP("Occupied"), BUDDY_STATUS_OCC},
		{tr_NOOP("Do Not Disturb"), BUDDY_STATUS_DND},
		{tr_NOOP("Free For Chat"), BUDDY_STATUS_FFC}
	};

	lastStore = NULL;
	
	
	OW_CREATE_MAINVBOX(mainVBox);
	
	
	
	// edit autoresponses
	OW_CREATE_SECTION(tr("<b>Edit Auto responses</b>"), vbox);

	man = new IMAutoResponseManager(NULL, TRUE);
	
	// create all neccessary stores
	arTypeStore = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_ULONG, G_TYPE_POINTER);
	for (i=0; i<5; i++)
	{
		// create the sub stores
		contents = NULL;
		names = NULL;
		man->fetchTemplates(cathegories[i].status, &contents, &names);
		
		searchc = contents;
		searchn = names;
		store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
		while(searchc)
		{
			gtk_list_store_append(store, &iter);
			gtk_list_store_set(store, &iter,
							   0, searchn->data,
							   1, searchc->data,
							   -1);
			
			searchc = searchc->next;
			searchn = searchn->next;
		}
		
		g_list_foreach(names, (GFunc)g_free, NULL);
		g_list_foreach(contents, (GFunc)g_free, NULL);
		g_list_free(names);
		g_list_free(contents);
		
		gtk_list_store_append(arTypeStore, &iter);
		gtk_list_store_set(arTypeStore, &iter,
						   0, cathegories[i].name,
						   1, cathegories[i].status,
						   2, store,
						   -1);
	}
	delete man;
	
	// create the templates view
	arTemplatesView = gtk_tree_view_new();
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(arTemplatesView), TRUE);
	g_signal_connect_swapped(gtk_tree_view_get_selection(GTK_TREE_VIEW(arTemplatesView)), "changed", G_CALLBACK(this->cb_arTemplateChanged), this);

	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(renderer), "editable", TRUE, NULL);
	g_signal_connect(renderer, "edited", G_CALLBACK(this->cb_arTemplatesViewEdited), this);
	col = gtk_tree_view_column_new_with_attributes(tr("Preset Slot"), renderer, 
													"text", 0, 
													NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(arTemplatesView), col);
	
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scrolled), arTemplatesView);
	gtk_widget_set_size_request(scrolled, 100, -1);
	
	
	// create the templates entry
	frame = u_createTextView(&arTemplateEntryView);	
	
	// create the types view
	arTypeView = gtk_combo_box_new_with_model(GTK_TREE_MODEL(arTypeStore));
	g_signal_connect_swapped(arTypeView, "changed", G_CALLBACK(this->cb_arTypeChanged), this);
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(arTypeView), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(arTypeView), renderer, 
								   "text", 0, 
								   NULL);
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(arTypeStore), &iter);
	gtk_combo_box_set_active_iter(GTK_COMBO_BOX(arTypeView), &iter);
	
	
	label2 = gtk_label_new(tr("Status:"));
	
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), arTypeView, TRUE, TRUE, 0);
	

	// add the templates store and the text entry in a vpaned
	paned = gtk_hpaned_new();
	gtk_paned_pack1(GTK_PANED(paned), scrolled, FALSE, TRUE);
	gtk_paned_pack2(GTK_PANED(paned), frame, TRUE, TRUE);
	
	
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), paned, TRUE, TRUE, 0);	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, TRUE, TRUE, 0);
	
	
	
	
	// time outs
	OW_CREATE_SECTION_T(tr("<b>Automatic set auto response</b>"), table, 2, 4);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 2);
	
	label2 = gtk_label_new(tr("<small>After a certain period of time (in minutes) a special status can be set automatically if you are inactive. \"0\" means that this status should never be set.</small>"));
	gtk_label_set_line_wrap(GTK_LABEL(label2), TRUE);
	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);

	label3 = gtk_label_new(tr("Away:"));
	gtk_misc_set_alignment(GTK_MISC(label3), 0.0f, 0.0f);
	autoAwaySpin = gtk_spin_button_new_with_range(0, 0xFFFF, 1);
	
	label4 = gtk_label_new(tr("Not Available:"));
	gtk_misc_set_alignment(GTK_MISC(label4), 0.0f, 0.0f);
	autoNASpin = gtk_spin_button_new_with_range(0, 0xFFFF, 1);
	
	label5 = gtk_label_new(tr("Offline:"));
	gtk_misc_set_alignment(GTK_MISC(label5), 0.0f, 0.0f);
	autoOfflineSpin = gtk_spin_button_new_with_range(0, 0xFFFF, 1);


	gtk_table_attach(GTK_TABLE(table), label2, 0, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	
	gtk_table_attach(GTK_TABLE(table), label3, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);	
	gtk_table_attach_defaults(GTK_TABLE(table), autoAwaySpin, 1, 2, 1, 2);
	gtk_table_attach(GTK_TABLE(table), label4, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), autoNASpin, 1, 2, 2, 3);
	gtk_table_attach(GTK_TABLE(table), label5, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach_defaults(GTK_TABLE(table), autoOfflineSpin, 1, 2, 3, 4);	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, FALSE, TRUE, 0);
	
	gtk_widget_set_sensitive(label, FALSE);
	gtk_widget_set_sensitive(alignment, FALSE);
	
	
	return mainVBox;
}



gboolean optionsWindowItem_aresponse::applyChanges()
{
	GtkTreeIter				iter1,
							iter2;
	GList					*names,
							*contents;
	gulong					status;
	GtkListStore			*statusTemplates;
	gchar					*currentName,
							*currentContent;
	IMAutoResponseManager	*man;
	
	// save all auto responses
	// FIXME: is it really neccessary to rewrite ALL responses?
	
	// apply the latest change
	cb_arTemplateChanged(this);
	
	man = new IMAutoResponseManager(NULL, TRUE);
	
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(arTypeStore), &iter1))
	{
		do
		{
			gtk_tree_model_get(GTK_TREE_MODEL(arTypeStore),
							   &iter1,
							   1, &status,
							   2, &statusTemplates,
							   -1);
			
			names = NULL;
			contents = NULL;
			if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(statusTemplates), &iter2))
			{
				do
				{
					gtk_tree_model_get(GTK_TREE_MODEL(statusTemplates), &iter2,
									   0, &currentName,
									   1, &currentContent,
									   -1);
					names = g_list_append(names, currentName);
					contents = g_list_append(contents, currentContent);
					
				} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(statusTemplates), &iter2));
				
				man->writeTemplates(status, contents, names);
			}
			
			
		} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(arTypeStore), &iter1));
	}
	
	delete man;
	
	return TRUE;
}




void optionsWindowItem_aresponse::cb_arTypeChanged(optionsWindowItem_aresponse* self)
{
	GtkTreeIter	iter;
	GtkListStore	*store;
	GtkTreePath		*path;
	
	
	
	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(self->arTypeView), &iter);
	
	gtk_tree_model_get(GTK_TREE_MODEL(self->arTypeStore), &iter,
					   2, &store,
					   -1);

	gtk_tree_view_set_model(GTK_TREE_VIEW(self->arTemplatesView), GTK_TREE_MODEL(store));
	
	// select the first entry
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
	path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(self->arTemplatesView), path, self->col, FALSE);
	gtk_tree_path_free(path);
}



void optionsWindowItem_aresponse::cb_arTemplateChanged(optionsWindowItem_aresponse* self)
{
	gchar			*content;
	GtkTreePath		*path;
	GtkTextIter		titer1, titer2;
	GtkTextBuffer	*buf;

	
	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(self->arTemplateEntryView));

	// set the last edited entry
	if (self->lastStore)
	{
		gtk_text_buffer_get_bounds(buf, &titer1, &titer2);
		content = gtk_text_buffer_get_text(buf, &titer1, &titer2, FALSE);
		gtk_list_store_set(self->lastStore, &self->lastIter, 1, content, -1);
	}		

    
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(self->arTemplatesView), &path, NULL);
	
	if (!path)
		return;
	
	self->lastStore = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(self->arTemplatesView)));	
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(self->lastStore), &self->lastIter, path);
	gtk_tree_path_free(path);
	gtk_tree_model_get(GTK_TREE_MODEL(self->lastStore), &self->lastIter, 1, &content, -1);
	
	gtk_text_buffer_set_text(buf,
							 content,
							 -1);
		
}



void optionsWindowItem_aresponse::cb_arTemplatesViewEdited(GtkCellRendererText *cell, gchar *pathStr, gchar *newText, optionsWindowItem_aresponse* self)
{
	GtkTreePath		*path;
	GtkTreeIter		iter;
	GtkListStore	*store;
	
	path = gtk_tree_path_new_from_string(pathStr);
	
	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(self->arTemplatesView)));
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
	
	gtk_tree_path_free(path);
	
	gtk_list_store_set(store, &iter, 0, newText, -1);
}




// -----------------------------------------------------------------------------

optionsWindowItem_themes::optionsWindowItem_themes()
{
	titleImage = NULL;
	titleString = tr_NOOP("Themes");
}


optionsWindowItem_themes::~optionsWindowItem_themes()
{
	
}



gboolean optionsWindowItem_themes::applyChanges()
{
	GtkTreeIter			iter;
	themeDirectoryType	*themeType;
	gchar				*eventTheme,
						*smileyTheme,
						*statusTheme,
						*extendedTheme;	
	
	// get the status theme
	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(status.listView), &iter);
	gtk_tree_model_get(GTK_TREE_MODEL(status.store), &iter, 1, &themeType, -1);
	statusTheme = themeType->basename;

	// get the extended theme
	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(extended.listView), &iter);
	gtk_tree_model_get(GTK_TREE_MODEL(extended.store), &iter, 1, &themeType, -1);
	extendedTheme = themeType->basename;
	
	// get the events theme
	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(events.listView), &iter);
	gtk_tree_model_get(GTK_TREE_MODEL(events.store), &iter, 1, &themeType, -1);
	eventTheme = themeType->basename;

	// get the smileys theme
	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(emoticons.listView), &iter);
	gtk_tree_model_get(GTK_TREE_MODEL(emoticons.store), &iter, 1, &themeType, -1);
	smileyTheme = themeType->basename;
	
	// apply the changes
	settings_getSettings()->setProperties(TRUE, "appearance",
										  "EventTheme", eventTheme,
										  "SmileyTheme", smileyTheme,
										  "StatusTheme", statusTheme,
										  "ExtendedTheme", extendedTheme,
										  NULL);

	
	return TRUE;
}



GtkWidget	*optionsWindowItem_themes::createTab()
{
	GtkWidget	*mainVBox;
	gchar		*eventTheme,
				*smileyTheme,
				*statusTheme,
				*extendedTheme;
	
		
	createThemesTable();
	
	
	OW_CREATE_MAINVBOX(mainVBox);
	
	settings_getSettings()->getProperties("appearance", 
					  		   			  "EventTheme", &eventTheme,
					  		   			  "SmileyTheme", &smileyTheme,
					  		   			  "StatusTheme", &statusTheme,
					  		   			  "ExtendedTheme", &extendedTheme,
										  NULL);
	
	
	// status icons
	appendIconsSection(tr("<b>Status Icons</b>"), statusTheme, mainVBox, &status);
	appendIconsSection(tr("<b>Extended Icons</b>"), extendedTheme, mainVBox, &extended);
	appendIconsSection(tr("<b>Event Icons</b>"), eventTheme, mainVBox, &events);
	appendIconsSection(tr("<b>Smileys</b>"), smileyTheme, mainVBox, &emoticons);
	
	g_free(eventTheme);
	g_free(smileyTheme);
	g_free(statusTheme);
	g_free(extendedTheme);
	
	return mainVBox;
}



void optionsWindowItem_themes::appendIconsSection(gchar* cap, gchar* current, GtkWidget* mainVBox, themeTypeVars* vars)
{
	GtkWidget			*alignment,
						*label,
						*label2,
						*vbox,
						*hbox;
	GList				*search;
	themeDirectoryType 	*themeDir;
	GtkTreeIter			iter;
	GtkCellRenderer		*renderer;
	gint				currentSelection,
						i;
	
	OW_CREATE_SECTION(cap, vbox);
	
	
	// create the event box where the icons are stored in later
	vars->box = gtk_event_box_new();
	
	currentSelection = 0;
	i = 0;
	
	// create the themes store
	vars->store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
	search = vars->themes;
	while(search)
	{
		themeDir = (themeDirectoryType*)search->data;
		
		gtk_list_store_append(vars->store, &iter);
		gtk_list_store_set(vars->store, &iter,
						   0, themeDir->name,
						   1, themeDir,
						   -1);
		if (!strcmp(themeDir->basename, current))
			currentSelection = i;
		
		search = search->next;
		i++;
	}
	
	// create the theme selection list view
	vars->listView = gtk_combo_box_new_with_model(GTK_TREE_MODEL(vars->store));

	g_object_set_data(G_OBJECT(vars->listView), "icqnd-options-theme", vars);
	g_signal_connect(vars->listView, "changed", G_CALLBACK(this->cb_themesListChanged), this);
	gtk_combo_box_set_active(GTK_COMBO_BOX(vars->listView), currentSelection);
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(vars->listView), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(vars->listView), renderer, "text", 0, NULL);

	label2 = gtk_label_new(tr("Theme:"));
	
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vars->listView, TRUE, TRUE, 0);
	
	
	
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), vars->box, TRUE, TRUE, 0);
	
	
	gtk_box_pack_start(GTK_BOX(mainVBox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), alignment, TRUE, TRUE, 0);
}



void optionsWindowItem_themes::createThemesTable()
{
	GDir				*currentDir,
						*themeDir;
	gchar				*base,
						*themeName,
						*themeSecName,
						*themeTitle,
						*tmp,
						*strStart,
						*strEnd;
	const gchar			*currentName;
	const gchar			*currentThemeName;
	FILE				*authorFile;
	gulong				authorsSize;
	gchar				*buf;
	themeDirectoryType	*themeType;
	
	status.themes = NULL;
	extended.themes = NULL;
	emoticons.themes = NULL;
	events.themes = NULL;

	base = g_strdup_printf("%sicqnd", SHARE_DIR);

	// get a list of all theme directories	
	currentDir = g_dir_open(base, 0, NULL);
	
	
	while(currentThemeName = g_dir_read_name(currentDir))
	{
		themeName = g_strdup_printf("%s/%s", base, currentThemeName);
		
		// okay, this should be a theme if it's a directory		
		if (!g_file_test(themeName, G_FILE_TEST_IS_DIR))
		{
			g_free(themeName);
			continue;
		}
		
		
		themeTitle = NULL;
		
		// try to get the author information
		tmp = g_strdup_printf("%s/author", themeName);
		authorFile = fopen(tmp, "r");
		
		if (authorFile)
		{
			fseek(authorFile, 0, SEEK_END);
			authorsSize = ftell(authorFile);
			rewind(authorFile);

			buf = (gchar*)g_malloc0(authorsSize);
			fread(buf, 1, authorsSize, authorFile);	

			if ((strStart = strstr(buf, "theme:")) && (strEnd = strstr(strStart, "\n")))
			{
				strStart+=6;
				strStart = g_strstrip(strStart);
				themeTitle = (gchar*)g_malloc0(strEnd - strStart);
				memcpy(themeTitle, strStart, strEnd - strStart-1);
			}	
			
			g_free(buf);
			fclose(authorFile);
		}
		
		if (!themeTitle)
			themeTitle = g_strdup(currentName);
						
		
		themeDir = g_dir_open(themeName, 0, NULL);
		while(currentName = g_dir_read_name(themeDir))
		{
			themeSecName = g_strdup_printf("%s/%s", themeName, currentName);
			if (!g_file_test(themeSecName, G_FILE_TEST_IS_DIR))
			{
				g_free(themeSecName);
				continue;
			}
			
			
			// an events directory
			if (!strcmp(currentName, "events"))
				themeType = getDirFiles(themeSecName, themeTitle, &events.themes);
			else
			// a directory for extended icons				
			if (!strcmp(currentName, "extended"))
				themeType = getDirFiles(themeSecName, themeTitle, &extended.themes);
			else
			// a directory for status icons
			if (!strcmp(currentName, "status"))
				themeType = getStatusIconsFromDirectory(themeSecName, themeTitle);
			else
			// a directory for smileys
			if (!strcmp(currentName, "smileys"))
				themeType = getSmileysFromDirectory(themeSecName,themeTitle);
			
			// append the basename info
			themeType->basename = g_strdup(currentThemeName);
			
			g_free(themeSecName);
		}				
		
		g_dir_close(themeDir);
		
		g_free(themeName);
	}	
	
	g_dir_close(currentDir);
	g_free(themeTitle);
}


themeDirectoryType* optionsWindowItem_themes::getDirFiles(gchar* dirName, gchar* themeTitle, GList** toAdd)
{
	themeDirectoryType	*themeType;
	GDir				*dir;
	const gchar			*currentName;
	GList				*files;
	gchar				*imageFile;
	
	files = NULL;
	
	// get all file names	
	dir = g_dir_open(dirName, 0, NULL);
	
	while(currentName = g_dir_read_name(dir))
	{
		imageFile = g_strdup_printf("%s/%s", dirName, currentName);
		files = g_list_append(files, imageFile);
	}
	
	g_dir_close(dir);
	
	
	// append an entry to the glist
	themeType = (themeDirectoryType*)g_malloc0(sizeof(themeDirectoryType));
	
	themeType->name = g_strdup(themeTitle);
	themeType->directory = g_strdup(dirName);
	themeType->imageNames = files;
	
	*toAdd = g_list_append(*toAdd, themeType);
	
	return themeType;
}


themeDirectoryType* optionsWindowItem_themes::getSmileysFromDirectory(gchar* dirName, gchar* themeTitle)
{
	FILE	*tokensFile;
	gchar	*tokensName;
	gchar	*buf,
			*tmp;
	gulong	size,
			pos,
			count,
			smileyPos;
	GString	*smileyToken,
			*imageName,
			*smileyCount;
	gboolean hasToken;
	GList	*files;
	themeDirectoryType	*themeType;
	
	files = NULL;

	
	// open the rules file
	tokensName = g_strdup_printf("%s/tokens.conf", dirName);
	tokensFile = fopen(tokensName, "r");
	if (!tokensFile)
	{
		g_free(tokensName);
		return NULL;
	}
	
	fseek(tokensFile, 0, SEEK_END);
	size = ftell(tokensFile);
	rewind(tokensFile);

	buf = (gchar*)g_malloc0(size);
	fread(buf, 1, size, tokensFile);
	
	fclose(tokensFile);
	g_free(tokensName);

	// read the file content -> create the smileys tree and load the images	
	pos = 0;
	smileyCount = g_string_new("");
	hasToken = FALSE;
	
	// get the smiley count
	while(buf[pos] != '\n')
		smileyCount = g_string_append_c(smileyCount, buf[pos++]);

	count = strtoul(smileyCount->str, NULL, 10);
	g_string_free(smileyCount, TRUE);
	
	// is the count valid
	if (!count)
	{
		g_free(buf);
		fclose(tokensFile);
		return NULL;
	}
		
	smileyToken = g_string_new("");
	imageName = g_string_new("");
	smileyPos = 0;
	pos++;
	
	while(pos<size && smileyPos<count)
	{
		// smiley tokens and image names are separated by tabs
		if (buf[pos] == '\t')
			hasToken = TRUE;
		
		// a line is over -> we should have a new pair
		if (buf[pos] == '\n')
		{
			// is the line valid? if not -> return
			if (!strlen(smileyToken->str) || !strlen(imageName->str))
			{
				g_free(tokensName);
				g_free(buf);
				g_string_free(smileyToken, TRUE);
				g_string_free(imageName, TRUE);
				fclose(tokensFile);
				return NULL;
			}
			
			tmp = g_strdup_printf("%s/%s", dirName, imageName->str);
			files = g_list_append(files, tmp);

			smileyToken = g_string_assign(smileyToken, "");
			imageName = g_string_assign(imageName, "");
			smileyPos++;
			hasToken = FALSE;
			pos++;
		}
		
		if (!hasToken)
			smileyToken = g_string_append_c(smileyToken, buf[pos]);
		else
			if (buf[pos] != '\t' && buf[pos] != '\n')
				imageName = g_string_append_c(imageName, buf[pos]);
		
		pos++;
	}
	
	g_free(buf);
	g_string_free(smileyToken, TRUE);
	g_string_free(imageName, TRUE);

	
	// append an entry to the glist
	themeType = (themeDirectoryType*)g_malloc0(sizeof(themeDirectoryType));
	
	themeType->name = g_strdup(themeTitle);
	themeType->directory = g_strdup(dirName);
	themeType->imageNames = files;
	
	emoticons.themes = g_list_append(emoticons.themes, themeType);
	
	return themeType;
}



void optionsWindowItem_themes::openImages(GList *images, GtkWidget *container)
{
	GtkWidget	*img,
				*table;
	GList		*children,
				*search;
	guint		len,
				i;
	
	children = gtk_container_get_children(GTK_CONTAINER(container));
	if (children)
	{
		g_list_foreach(children, (GFunc)gtk_widget_destroy, NULL);
		g_list_free(children);
	}
	
	len = g_list_length(images);
	table = gtk_table_new(len/2+1, 10, FALSE);
	
	i=0;
	search = images;
	while(search)
	{
		img = gtk_image_new_from_file((gchar*)search->data);
		
		gtk_table_attach_defaults(GTK_TABLE(table), img, (i%10), (i%10)+1, (i/10), (i/10)+1);
		
		search = search->next;
		i++;
	}
	
	gtk_container_add(GTK_CONTAINER(container), table);
	
	gtk_widget_show_all(container);
}



themeDirectoryType* optionsWindowItem_themes::getStatusIconsFromDirectory(gchar* dirName, gchar* themeTitle)
{
	GDir				*dir,
						*iconDir;
	const gchar			*currentName;
	gchar				*iconPath,
						*iconName;
	gint				i;
	themeDirectoryType	*themeType;
	GList				*files;
	
	struct _iconManLoad
	{
		gchar	*iconName;
		glong	status;
	} iconManStatus[] =
	{
		{"away.png", BUDDY_STATUS_AWAY},
		{"dnd.png", BUDDY_STATUS_DND},
		{"ffc.png", BUDDY_STATUS_FFC},
		{"invisible.png", BUDDY_STATUS_INV},
		{"invisible_set.png", BUDDY_INV_SET},
		{"na.png", BUDDY_STATUS_NA},
		{"occ.png", BUDDY_STATUS_OCC},
		{"offline.png", BUDDY_STATUS_OFFLINE},		
		{"online.png", BUDDY_STATUS_ONLINE}
	};
	
	files = NULL;
	
	dir = g_dir_open(dirName, 0, NULL);

	while(currentName = g_dir_read_name(dir))
	{
		iconPath = g_strdup_printf("%s/%s", dirName, currentName);
		
		if (g_file_test(iconPath, G_FILE_TEST_IS_DIR))
		{		
			iconDir = g_dir_open(iconPath, 0, NULL);
			while(currentName = g_dir_read_name(iconDir))
			{
				for (i=0; i<9; i++)
				{
					if (!strcmp(currentName, iconManStatus[i].iconName))
					{
						iconName = g_strdup_printf("%s/%s", iconPath, currentName);
						files = g_list_append(files, iconName);
					}
				}				
			}
			
			g_dir_close(iconDir);
			
		}
		
		g_free(iconPath);
	}

	g_dir_close(dir);	
	
	
	// append an entry to the glist
	themeType = (themeDirectoryType*)g_malloc0(sizeof(themeDirectoryType));
	
	themeType->name = g_strdup(themeTitle);
	themeType->directory = g_strdup(dirName);
	themeType->imageNames = files;
	
	status.themes = g_list_append(status.themes, themeType);
	
	return themeType;
}



void optionsWindowItem_themes::cb_themesListChanged(GtkComboBox* w, optionsWindowItem_themes* self)
{
	GtkTreeIter			iter;
	themeTypeVars		*vars;
	themeDirectoryType	*theme;

	vars = (themeTypeVars*)g_object_get_data(G_OBJECT(w), "icqnd-options-theme");

	gtk_combo_box_get_active_iter(w, &iter);
	gtk_tree_model_get(GTK_TREE_MODEL(vars->store), &iter, 1, &theme, -1);
	
	self->openImages(theme->imageNames, vars->box);
}
