/***************************************************************************
 *            systemcharset.cpp
 *
 *  Sat Jul 24 01:16:54 2004
 *  Copyright  2004  User
 *  Email
 ****************************************************************************/

#include "systemcharset.h"
#include "licq_chat.h"	// for ENCODING tokens

static charsetType charsets[] =
{
	{ tr_NOOP("Unicode")			, "UTF-8"			, ENCODING_DEFAULT},
	{ tr_NOOP("Unicode-16")			, "ISO-10646-UCS-2"	, ENCODING_DEFAULT},
	{ tr_NOOP("Arabic")				, "ISO 8859-6"		, ENCODING_DEFAULT},
	{ tr_NOOP("Arabic")				, "CP 1256"			, ENCODING_ARABIC},
	{ tr_NOOP("Baltic")				, "ISO 8859-13"		, ENCODING_DEFAULT},
	{ tr_NOOP("Baltic")				, "CP 1257"			, ENCODING_BALTIC},
	{ tr_NOOP("Central European")	, "ISO 8859-2"		, ENCODING_DEFAULT},
	{ tr_NOOP("Central European")	, "CP 1250"			, ENCODING_EASTEUROPE},
	{ tr_NOOP("Chinese")			, "GBK"				, ENCODING_GB2312},
	{ tr_NOOP("Chinese Traditional"), "Big5"			, ENCODING_CHINESEBIG5},
	{ tr_NOOP("Cyrillic")			, "ISO 8859-5"		, ENCODING_DEFAULT},
	{ tr_NOOP("Cyrillic")			, "KOI8-R"			, ENCODING_DEFAULT},
	{ tr_NOOP("Cyrillic")			, "CP 1251"			, ENCODING_RUSSIAN},
	{ tr_NOOP("Esperanto")			, "ISO 8859-3"		, ENCODING_DEFAULT},
	{ tr_NOOP("Greek")				, "ISO 8859-7"		, ENCODING_DEFAULT},
	{ tr_NOOP("Greek")				, "CP 1253"			, ENCODING_GREEK},
	{ tr_NOOP("Hebrew")				, "ISO 8859-8-I"	, ENCODING_DEFAULT},
	{ tr_NOOP("Hebrew")				, "CP 1255"			, ENCODING_HEBREW},
	{ tr_NOOP("Japanese")			, "Shift-JIS"		, ENCODING_SHIFTJIS},
	{ tr_NOOP("Japanese")			, "JIS7"			, ENCODING_DEFAULT},
	{ tr_NOOP("Japanese")			, "eucJP"			, ENCODING_DEFAULT},
	{ tr_NOOP("Korean")				, "eucKR"			, ENCODING_DEFAULT},
	{ tr_NOOP("Western European")	, "ISO 8859-1"		, ENCODING_DEFAULT},
	{ tr_NOOP("Western European")	, "ISO 8859-15"		, ENCODING_DEFAULT},
	{ tr_NOOP("Western European")	, "CP 1252"			, ENCODING_ANSI},
	{ tr_NOOP("Tamil")				, "TSCII"			, ENCODING_DEFAULT},
	{ tr_NOOP("Thai")				, "TIS-620"			, ENCODING_THAI},
	{ tr_NOOP("Turkish")			, "ISO 8859-9"		, ENCODING_DEFAULT},
	{ tr_NOOP("Turkish")			, "CP 1254"			, ENCODING_TURKISH},
	{ tr_NOOP("Ukrainian")			, "KOI8-U"			, ENCODING_DEFAULT},
	{ NULL, NULL, 0}
};


static charsetType guessCharsets[] =
{
	{ tr_NOOP("Unicode")			, "UTF-8"			, ENCODING_DEFAULT},
	{ tr_NOOP("Unicode-16")			, "ISO-10646-UCS-2"	, ENCODING_DEFAULT},
	{ tr_NOOP("Western European")	, "ISO 8859-1"		, ENCODING_DEFAULT},
	{ tr_NOOP("Western European")	, "ISO 8859-15"		, ENCODING_DEFAULT},	
	{ tr_NOOP("Arabic")				, "ISO 8859-6"		, ENCODING_DEFAULT},
	{ tr_NOOP("Baltic")				, "ISO 8859-13"		, ENCODING_DEFAULT},
	{ tr_NOOP("Central European")	, "ISO 8859-2"		, ENCODING_DEFAULT},
	{ tr_NOOP("Cyrillic")			, "ISO 8859-5"		, ENCODING_DEFAULT},
	{ tr_NOOP("Turkish")			, "ISO 8859-9"		, ENCODING_DEFAULT},
	{ tr_NOOP("Esperanto")			, "ISO 8859-3"		, ENCODING_DEFAULT},
	{ tr_NOOP("Greek")				, "ISO 8859-7"		, ENCODING_DEFAULT},	
	{ tr_NOOP("Hebrew")				, "ISO 8859-8-I"	, ENCODING_DEFAULT},	
	{ tr_NOOP("Arabic")				, "CP 1256"			, ENCODING_ARABIC},
	{ tr_NOOP("Baltic")				, "CP 1257"			, ENCODING_BALTIC},
	{ tr_NOOP("Central European")	, "CP 1250"			, ENCODING_EASTEUROPE},
	{ tr_NOOP("Chinese")			, "GBK"				, ENCODING_GB2312},
	{ tr_NOOP("Chinese Traditional"), "Big5"			, ENCODING_CHINESEBIG5},
	{ tr_NOOP("Cyrillic")			, "KOI8-R"			, ENCODING_DEFAULT},
	{ tr_NOOP("Cyrillic")			, "CP 1251"			, ENCODING_RUSSIAN},
	{ tr_NOOP("Greek")				, "CP 1253"			, ENCODING_GREEK},
	{ tr_NOOP("Hebrew")				, "CP 1255"			, ENCODING_HEBREW},
	{ tr_NOOP("Japanese")			, "Shift-JIS"		, ENCODING_SHIFTJIS},
	{ tr_NOOP("Japanese")			, "JIS7"			, ENCODING_DEFAULT},
	{ tr_NOOP("Japanese")			, "eucJP"			, ENCODING_DEFAULT},
	{ tr_NOOP("Korean")				, "eucKR"			, ENCODING_DEFAULT},
	{ tr_NOOP("Western European")	, "CP 1252"			, ENCODING_ANSI},
	{ tr_NOOP("Tamil")				, "TSCII"			, ENCODING_DEFAULT},
	{ tr_NOOP("Thai")				, "TIS-620"			, ENCODING_THAI},
	{ tr_NOOP("Turkish")			, "CP 1254"			, ENCODING_TURKISH},
	{ tr_NOOP("Ukrainian")			, "KOI8-U"			, ENCODING_DEFAULT},
	{ NULL, NULL, 0}	
};

#define CHARSETS_SIZE sizeof(charsets) / sizeof(charsetType)





gchar* convertToCharset(gchar* toconvert, gchar* ics, gchar* ocs)
{
	gsize bIn, bOut;
	char *res;
	const char	*locale;
	
	g_get_charset(&locale);
	
	// check whether there is something to convert
	if (!toconvert)
		return NULL; 
	if (!*toconvert)
		return g_strdup(toconvert);

	// check whether this string is already in UTF-8
	if (g_utf8_validate(toconvert, -1, NULL))
		return g_strdup(toconvert);
	
	res = g_convert(toconvert, strlen(toconvert), ocs, ics, &bIn, &bOut, NULL);
	
	// conversion did not work -> try the preferred encoding
	if (!res)
		res = g_convert_with_fallback(toconvert, strlen(toconvert), locale, settings_getSettings()->fastAccess.preferredCharset, "?", &bIn, &bOut, NULL);

	// conversion STILL didn't work? go the hard way and guess the charset 
	// (latin comes first)
	if (!res)
	{
		guint i;
		i=0;
		while(guessCharsets[i].name)
		{
			res = g_convert_with_fallback(toconvert, strlen(toconvert), locale, guessCharsets[i].id, "?", &bIn, &bOut, NULL);
			if (res)
				break;
			i++;
		}	
	}		
	
	if (res)
		return res;
	else
	// conversion still doesn't work - nothing more we can do here ... 
		return g_strdup_printf("icqnd charset error");
}


gchar* convertToSystemCharset(gchar* toconvert, gchar* charset)
{
	if (!charset || !*charset)
		charset = settings_getSettings()->fastAccess.preferredCharset;	
	
	return convertToCharset(toconvert, charset, "UTF-8");
}



gchar* convertFromSystemCharset(gchar* toconvert, gchar* charset)
{
	if (!charset || !*charset)
		charset = settings_getSettings()->fastAccess.preferredCharset;	
	
	return convertToCharset(toconvert, "UTF-8", charset);	
}



gchar* localeToSystemCharset(gchar* toconvert)
{
	gchar* ret;
	gsize bIn, bOut;
	// FIXME: only valid for UTF-8
	
	ret = g_locale_to_utf8(toconvert, strlen(toconvert), &bIn, &bOut, NULL);

	// conversion failure -> fall back to ISO8859-1
	if (!ret)
		ret = g_convert_with_fallback(toconvert, strlen(toconvert), "UTF-8", "ISO8859-1", "?", &bIn, &bOut, NULL);		
	
	return ret;
}



gchar* systemCharsetToLocale(gchar* toconvert)
{
	gchar* ret;
	gsize bIn, bOut;
	// FIXME: only valid for UTF-8
	
	ret = g_locale_from_utf8(toconvert, strlen(toconvert), &bIn, &bOut, NULL);

	// conversion failure -> fall back to ISO8859-1
	if (!ret)
	{
		fprintf(stderr, "***WARNING: conversion of string (\"%s\") failed!\n", toconvert);
		ret = g_convert_with_fallback(toconvert, strlen(toconvert), "ISO8859-1", "UTF-8", "?", &bIn, &bOut, NULL);		
	}
	
	return ret;
}



charsetType* charset_getCharsetList()
{
	return charsets;	
}



charsetType* charset_findCharsetById(gchar* id)
{
	guint i;
	
	if (!id || !*id)
		return &charsets[0];
	
	for (i=0;i<CHARSETS_SIZE;i++)
	{
		if (!strcmp(charsets[i].id, id))
			return &charsets[i];
	}
	
	return NULL;
}



charsetType* charset_findCharsetByNumber(guchar number)
{
	guint i;
	
	for (i=0;i<CHARSETS_SIZE;i++)
	{
		if (charsets[i].number == number)
			return &charsets[i];
	}
	
	return NULL;
}
